/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets;

import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import fr.exemole.sctserver.*;
import javax.servlet.*;
import java.io.*;
import java.util.*;
import fr.exemole.sctserver.directory.*;
import fr.exemole.sctserver.servlets.selection.*;
import fr.exemole.sctserver.storage.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.xml.*;

/**
 *
 * @author Vincent Calame
 */
public class SctServerContext implements ConfigErrorKey {
    private final static String SCT_SERVER_PARAM = "fr.exemole.sctserver";
    private String confDirString = "";
    private String varDirString = "";
    private File confDir = null;
    private File varDir = null;
    private Map<String,SctBundle> sctBundleMap = new TreeMap<String,SctBundle>();
    private Map<String, Verrou> nameMap = new HashMap<String, Verrou>();
    private ResourceItem errorKey;
    
    private SctServerContext() {
    }
    
    public boolean hasError() {
        return (errorKey != null);
    }
    
    public ResourceItem getErrorKey() {
        return errorKey;
    }
    
    public SctBundle getSctBundle(String name) {
        SctBundle sctBundle = sctBundleMap.get(name);
        if (sctBundle == null) {
            sctBundle = createSctBundle(name);
        }
        return sctBundle;
    }
    
    public List<String> getNameList() {
        ArrayList<String> list = new ArrayList<String>();
        list.addAll(nameMap.keySet());
        return list;
    }
    
    private SctBundle createSctBundle(String name) {
        Verrou verrou = nameMap.get(name);
        if (verrou == null) return null;
        SctBundle sctBundle = null;
        synchronized(verrou) {
            sctBundle = sctBundleMap.get(name);
            if (sctBundle == null) {
                sctBundle = buildSctBundle(name);
                sctBundleMap.put(name, sctBundle);
            }
        }
        return sctBundle;
    }
    
    public static SctServerContext getSctServerContext(ServletContext servletContext) {
        SctServerContext sctServerContext = (SctServerContext) servletContext.getAttribute(SCT_SERVER_PARAM);
        if (sctServerContext == null) {
            sctServerContext = createSctServerContext(servletContext);
        }
        return sctServerContext;
    }
    
    private static synchronized SctServerContext createSctServerContext(ServletContext servletContext){
        SctServerContext sctServerContext = (SctServerContext) servletContext.getAttribute(SCT_SERVER_PARAM);
        if (sctServerContext == null) {
            sctServerContext = new SctServerContext();
            sctServerContext.reload(servletContext);
            servletContext.setAttribute(SCT_SERVER_PARAM,sctServerContext);
        }
        return sctServerContext;
    }
    
    public synchronized boolean reload(ServletContext servletContext) {
        errorKey = null;
        String scrutariConfFileString = servletContext.getInitParameter("scrutariConfFile");
        if (scrutariConfFileString != null) {
            scrutariConfFileString = scrutariConfFileString.trim();
            if (scrutariConfFileString.length() == 0) scrutariConfFileString = null;
        }
        if (scrutariConfFileString == null) {
            errorKey = new ResourceItem(INIT_PARAMETER);
            return false;
        }
        File scrutariConfFile = new File(scrutariConfFileString);
        if (!scrutariConfFile.exists()) {
            errorKey = new ResourceItem(MISSING_CONF_FILE,scrutariConfFileString);
            return false;
        }
        if (scrutariConfFile.isDirectory()) {
            errorKey = new ResourceItem(CONF_FILE_DIRECTORY,scrutariConfFileString);
            return false;
        }
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(scrutariConfFile);
            return readConf(doc.getDocumentElement());
        }
        catch(ParserConfigurationException pce) {
            pce.printStackTrace();
            errorKey = new ResourceItem(PARSER_CONFIG, pce.getMessage());
            return false;
        }
        catch(IOException ioe) {
            String[] val = new String[2];
            val[0] = scrutariConfFileString;
            val[1] = ioe.getMessage();
            errorKey = new ResourceItem(CONF_FILE_IOERROR,val);
            return false;
        }
        catch(SAXException sax) {
            String[] val = new String[2];
            val[0] = scrutariConfFileString;
            val[1] = sax.getMessage();
            errorKey = new ResourceItem(XML_PARSE,val);
            return false;
        }
    }
    
    private boolean readConf(Element element_xml) {
        String nvConfDirString = null;
        String nvVarDirString = null;
        Map<String,Verrou> hereMap = new HashMap<String,Verrou>();
        NodeList nodeList = element_xml.getChildNodes();
        for(int i = 0; i < nodeList.getLength(); i++) {
            Node nd = nodeList.item(i);
            if (nd instanceof Element) {
                Element el = (Element) nd;
                String tagName = el.getTagName();
                if (tagName.equals("sctserver")) {
                    String name = el.getAttribute("name").trim();
                    if (StringUtils.isTechnicalName(name,true)) {
                        hereMap.put(name, new Verrou());
                    }
                }
                else if (tagName.equals("conf-dir")) {
                    nvConfDirString = XMLUtils.getData(el);
                }
                else if (tagName.equals("var-dir")) {
                    nvVarDirString = XMLUtils.getData(el);
                }
            }
        }
        boolean change = false;
        if ((nvConfDirString == null) || (nvConfDirString.length() == 0)) {
            errorKey = new ResourceItem(XML_CONFDIR_MISSING);
            return false;
        }
        if ((nvVarDirString == null) || (nvVarDirString.length() == 0)){
            errorKey = new ResourceItem(XML_VARDIR_MISSING);
            return false;
        }
        if (!nvConfDirString.equals(confDirString)) {
            File dir = testDir(nvConfDirString, "<conf-dir>");
            if (dir != null) {
                confDirString = nvConfDirString;
                confDir = dir;
                change = true;
            }
            else {
                return false;
            }
        }
        if (!nvVarDirString.equals(varDirString)) {
            File dir = testDir(nvVarDirString, "<var-dir>");
            if (dir != null) {
                varDirString = nvVarDirString;
                varDir = dir;
                change = true;
            }
            else {
                return false;
            }
        }
        nameMap.clear();
        nameMap.putAll(hereMap);
        if (change) {
            sctBundleMap.clear();
        }
        else {
            Map<String,SctBundle> newMap = new HashMap<String,SctBundle>();
            for(String name : nameMap.keySet()) {
                SctBundle sctBundle = sctBundleMap.get(name);
                if (sctBundle != null) {
                    newMap.put(name, sctBundle);
                }
            }
            sctBundleMap = newMap;
        }
        return true;
    }
    
    private File testDir(String path, String elTag) {
        String[] erroval = new String[2];
        erroval[0] = elTag;
        erroval[1] = path;
        File dir = new File(path);
        if (!dir.exists()) {
            errorKey = new ResourceItem(XML_PATH_NOTEXIST, erroval);
            return null;
        }
        if (!dir.isDirectory()) {
            errorKey = new ResourceItem(XML_PATH_NOTDIR, erroval);
            return null;
        }
        return dir;
    }
    
    private SctBundle buildSctBundle(String name) {
        File sctserverConfDir = new File(confDir, name);
        if (!sctserverConfDir.exists()) {
            sctserverConfDir.mkdir();
        }
        File sctserverVarDir = new File(varDir, name);
        if (!sctserverVarDir.exists()) {
            sctserverVarDir.mkdir();
        }
        File selectionManagerDir = new File(sctserverConfDir, "selection");
        SctServerStorage sctServerStorage = new SctServerDirectory(sctserverConfDir, sctserverVarDir);
        SctServer sctServer = new SctServer(sctServerStorage);
        SelectionManager selectionManager = new SelectionManager(selectionManagerDir);
        SctBundle sctBundle = new SctBundle();
        sctBundle.setSctServer(sctServer);
        sctBundle.setSelectionManager(selectionManager);
        return sctBundle;
    }
    
    private static class Verrou {
        
        
    }
    
}
