/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.explore;

import javax.servlet.http.*;
import fr.exemole.sctserver.servlets.*;
import java.util.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.primitives.*;
import fr.exemole.sctserver.servlets.selection.*;
import net.mapeadores.util.text.*;
import fr.exemole.sctserver.servlets.explore.htmlpages.*;
import fr.exemole.sctserver.servlets.explore.xmldata.*;
import net.mapeadores.util.servlets.specialresponses.*;

/**
 *
 * @author Vincent Calame
 */
public class ExploreRequest {
    private static final String SUPERMOTCLE_PATH = "/supermotcle/";
    private static final String LISTS_PATH = "/lists/";
    private SctBundle sctBundle;
    private ExploreResponseParameters exploreResponseParameters;
    private String path;
    private RequestMap requestMap;
    private boolean isInit;
    
    private ExploreRequest(SctBundle sctBundle,String pathInfo, HttpServletRequest request) {
        this.sctBundle = sctBundle;
        path = request.getPathInfo();
        exploreResponseParameters = new ExploreResponseParameters(sctBundle);
        requestMap = new DefaultRequestMap(request);
        initRequestMap();
        isInit =  false;
        if (pathInfo.equals("/")) {
            
        } else if (pathInfo.startsWith(SUPERMOTCLE_PATH)) {
            isInit = checkSupermotcleRequest(pathInfo.substring(SUPERMOTCLE_PATH.length()));
        } else if (pathInfo.startsWith(LISTS_PATH)) {
            isInit = checkListRequest(pathInfo.substring(LISTS_PATH.length()));
        }
    }
    
    private void initRequestMap() {
        String q = requestMap.getParameter("q");
        if (q != null) {
            SearchSequence searchSequence = SearchSequence.createSearchSequence(q);
            if (searchSequence != null) exploreResponseParameters.setSearchSequence(searchSequence);
        }
    }
    
    public static ExploreRequest init(SctBundle sctBundle,String pathInfo, HttpServletRequest request) {
        ExploreRequest exploreRequest = new ExploreRequest(sctBundle, pathInfo, request);
        if (exploreRequest.isInit) {
            return exploreRequest;
        } else {
            return null;
        }
    }
    
    public ExploreResponseParameters getResponseParameters() {
        return exploreResponseParameters;
    }
    
    public ResponsePrinter getResponsePrinter() {
        switch(exploreResponseParameters.getCategory()) {
            case ResponseParameters.HTMLPAGE_CATEGORY :
                return ExploreHtmlPage.getPage(exploreResponseParameters);
            case ResponseParameters.XMLDATA_CATEGORY :
                return ExploreXmlData.getInstance(exploreResponseParameters);
            default :
                throw new SwitchException("category = " + exploreResponseParameters.getCategory());
        }
    }
    
    
    private boolean checkSupermotcleRequest(String fileName) {
        Page page = getPage(fileName);
        if (page == null) return false;
        String commandName = page.getCommandName();
        if (commandName.startsWith("liste")) {
            if (page.hasLangInteger()) {
                exploreResponseParameters.setHtmlPageType(ExploreResponseParameters.SUPERMOTCLE_LISTE_PAGE);
                return true;
            } else {
                
            }
        } else if (commandName.startsWith("croisement")) {
            if (page.hasLangInteger()) {
                exploreResponseParameters.setHtmlPageType(ExploreResponseParameters.SUPERMOTCLE_CROISEMENT_PAGE);
                return true;
            } else {
                
            }
        }
        return false;
    }
    
    private boolean checkListRequest(String fileName) {
        Page page = getPage(fileName);
        if (page == null) return false;
        if (!page.getExtension().equals("xml")) return false;
        String commandName = page.getCommandName();
        if (commandName.equals("base-list")) {
            exploreResponseParameters.setXmlDataType(ExploreResponseParameters.BASE_LIST_XMLDATA);
            return true;
        }
        if (commandName.equals("log-list")) {
            exploreResponseParameters.setXmlDataType(ExploreResponseParameters.LOG_LIST_XMLDATA);
            return true;
        }
        if (commandName.equals("log")) {
            int dateInteger = page.getDateInteger();
            if (IntDate.getDateIntegerType(dateInteger) == IntDate.TYPE_MOIS) {
                exploreResponseParameters.setXmlDataType(ExploreResponseParameters.MONTH_LOG_XMLDATA);
                exploreResponseParameters.setDateInteger(dateInteger);
                return true;
            } else {
                return false;
            }
        }
        return false;
        
    }
    
    private Page getPage(String fileName) {
        Page page = new Page();
        if (!page.init(fileName)) return null;
        boolean withLangInteger = page.hasLangInteger();
        if (withLangInteger) {
            exploreResponseParameters.setLangInteger(page.getLangInteger());
        }
        return page;
    }
    
    private class Page {
        private String extension;
        private int langInteger = 0;
        private int dateInteger = 0;
        private String commandName;
        
        Page() {
            
        }
        
        boolean init(String fileName) {
            int idx = fileName.lastIndexOf('.');
            if ((idx == -1) || (idx == (fileName.length()-1))) return false;
            extension = fileName.substring(idx+1);
            commandName = fileName.substring(0, idx);
            int idx2 = commandName.lastIndexOf('_');
            if (idx2 != -1) {
                String complement = commandName.substring(idx2+1);
                try {
                    langInteger = LangInteger.parse(complement);
                } catch(LangIntegerException lie) {
                    try {
                        dateInteger = IntDate.parse(complement);
                    } catch(IntDateException ide) {
                        return false;
                    }
                }
                commandName = commandName.substring(0,idx2);
            }
            return true;
        }
        
        public String getExtension() {
            return extension;
        }
        
        public int getLangInteger() {
            return langInteger;
        }
        
        public int getDateInteger() {
            return dateInteger;
        }
        
        public String getCommandName() {
            return commandName;
        }
        
        public boolean hasLangInteger() {
            return (langInteger != 0);
        }
        
    }
    
}
