/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import com.sleepycat.je.*;
import com.sleepycat.bind.tuple.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.primitives.*;
import java.util.*;


/**
 * Dans toutes les fonctions d'ajout, on suppose qu'il n'y a pas de doublons.
 *
 * @author Vincent Calame
 */
public class DataManager {
    private static final byte SCRUTARI_URI = 1;
    private static final byte INDEXATION = 2;
    private Environment environment;
    private LinkedIntHashMap baseDataMap = new LinkedIntHashMap();
    private LinkedIntHashMap corpusDataMap = new LinkedIntHashMap();
    private LinkedIntHashMap thesaurusDataMap = new LinkedIntHashMap();
    private Database uriDB;
    private Database indexationDB;
    private SecondaryDatabase uriIndex;
    private SecondaryDatabase motcleIndex;
    private InternalScrutariURICoder internalScrutariURICoder = new InternalScrutariURICoder();
    private HashSet indexationSet = new HashSet();
    private DataQuery dataQuery;
    
    
    public DataManager(Environment env) {
        this.environment = env;
        try {
            DatabaseConfig databaseConfig = new DatabaseConfig();
            databaseConfig.setAllowCreate(true);
            uriDB = env.openDatabase(null,"uriDB",databaseConfig);
            indexationDB = env.openDatabase(null,"indexationB",databaseConfig);
            SecondaryConfig uriIndexConfig = new SecondaryConfig();
            uriIndexConfig.setAllowCreate(true);
            uriIndexConfig.setSortedDuplicates(true);
            uriIndexConfig.setKeyCreator(new ScrutariURIKeyCreator());
            uriIndex = env.openSecondaryDatabase(null,"uriDB_uriIndex", uriDB,uriIndexConfig);
            SecondaryConfig motcleIndexConfig = new SecondaryConfig();
            motcleIndexConfig.setAllowCreate(true);
            motcleIndexConfig.setSortedDuplicates(true);
            motcleIndexConfig.setKeyCreator(new MotcleKeyCreator());
            motcleIndex = env.openSecondaryDatabase(null, "indexationDB_motcleIndex", indexationDB,motcleIndexConfig);
            dataQuery = new DataQuery(uriDB, indexationDB);
        } catch(com.sleepycat.je.DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    
    public int getBaseCount() {
        return baseDataMap.size();
    }
    
    public int getCorpusCount() {
        return corpusDataMap.size();
    }
    
    public int getThesaurusCount() {
        return thesaurusDataMap.size();
    }
    
    public DataQuery getDataQuery() {
        return dataQuery;
    }
    
    public int getIndexationCount() {
        try {
            return (int) indexationDB.count();
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    public boolean hasIndexation(int codecorpus,int codethesaurus) {
        long lg = PrimUtils.toLong(codecorpus, codethesaurus);
        return indexationSet.contains(new Long(lg));
    }
    
    
    public BaseData getBaseDataByCode(int codebase) {
        return (BaseData) baseDataMap.get(codebase);
    }
    
    public BaseData getBaseDataByIndex(int index) {
        return (BaseData) baseDataMap.getValueAt(index);
    }
    
    public CorpusData getCorpusDataByCode(int codecorpus) {
        return (CorpusData) corpusDataMap.get(codecorpus);
    }
    
    public CorpusData getCorpusDataByIndex(int index) {
        return (CorpusData) corpusDataMap.getValueAt(index);
    }
    
    public int[] getCodecorpusArray() {
        return corpusDataMap.getKeyArray();
    }
    
    public ThesaurusData getThesaurusDataByCode(int codethesaurus) {
        return (ThesaurusData) thesaurusDataMap.get(codethesaurus);
    }
    
    public ThesaurusData getThesaurusDataByIndex(int index) {
        return (ThesaurusData) thesaurusDataMap.getValueAt(index);
    }
    
    public int[] getCodethesaurusArray() {
        return thesaurusDataMap.getKeyArray();
    }
    
    public MotcleData getMotcleDataByCode(int codemotcle) {
        DatabaseEntry data = getData(codemotcle);
        if (data != null) {
            return MotcleData.fromDatabaseEntry(data);
        } else {
            return null;
        }
    }
    
    public FicheData getFicheDataByCode(int codefiche) {
        DatabaseEntry data = getData(codefiche);
        if (data != null) {
            return FicheData.fromDatabaseEntry(data);
        } else {
            return null;
        }
    }
    
    public int getFicheDateIntegerByCode(int codefiche) {
        FicheData ficheData = getFicheDataByCode(codefiche);
        if (ficheData != null) {
            return ficheData.getDateInteger();
        } else {
            return IntDate.NULL;
        }
    }
    
    public ScrutariURICoder getScrutariURICoder() {
        return internalScrutariURICoder;
    }
    
    public Iterator getIndexationDataIterator() {
        return new GlobalIndexationIterator();
    }
    
    public Iterator getIndexationDataIteratorForMotcle(int codemotcle, int codecorpus) {
        return new MotcleIndexationIterator(codemotcle,codecorpus);
    }
    
    
    
    void close() {
        try {
            String uriDBName = uriDB.getDatabaseName();
            String indexationDBName = indexationDB.getDatabaseName();
            uriIndex.close();
            motcleIndex.close();
            uriDB.close();
            indexationDB.close();
            environment.removeDatabase(null,uriDBName);
            environment.removeDatabase(null,indexationDBName);
            
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    private DatabaseEntry getData(int code) {
        DatabaseEntry key = new DatabaseEntry();
        DatabaseEntry data = new DatabaseEntry();
        IntegerBinding.intToEntry(code,key);
        try {
            OperationStatus op = uriDB.get(null,key,data,null);
            if (op == OperationStatus.SUCCESS) {
                return data;
            } else {
                return null;
            }
        } catch(IndexOutOfBoundsException iobe) {
            return null;
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    
    
    public void addBaseData(BaseData BaseData) {
        baseDataMap.put(BaseData.getCodeBase(),BaseData);
    }
    
    public void addCorpusData(CorpusData corpusData) {
        corpusDataMap.put(corpusData.getCodeCorpus(),corpusData);
    }
    
    public void addThesaurusData(ThesaurusData thesaurusData) {
        thesaurusDataMap.put(thesaurusData.getCodeThesaurus(),thesaurusData);
    }
    
    public void addMotcleData(MotcleData motcleData) {
        DatabaseEntry key = new DatabaseEntry();
        IntegerBinding.intToEntry(motcleData.getCodeMotcle(),key);
        try {
            uriDB.put(null,key,motcleData.toDatabaseEntry());
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    public void addFicheData(FicheData ficheData) {
        DatabaseEntry key = new DatabaseEntry();
        IntegerBinding.intToEntry(ficheData.getCodeFiche(),key);
        try {
            uriDB.put(null,key,ficheData.toDatabaseEntry());
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    public void addIndexationData(IndexationData indexationData) {
        long lg = PrimUtils.toLong(indexationData.getCodeFiche(), indexationData.getCodeMotcle());
        DatabaseEntry key = new DatabaseEntry();
        LongBinding.longToEntry(lg, key);
        try {
            indexationDB.put(null,key,indexationData.toDatabaseEntry());
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
        long lg2 = PrimUtils.toLong(indexationData.getCodeCorpus(), indexationData.getCodeThesaurus());
        indexationSet.add(new Long(lg2));
    }
    
    
    private class InternalScrutariURICoder implements ScrutariURICoder {
        Map uriMap = new HashMap();
        int maxCode = 1;
        
        public int getOrCreateCode(ScrutariURI scrutariURI) {
            int code = getCode(scrutariURI);
            if (code != -1) return code;
            code = maxCode;
            if (isInMemoryURI(scrutariURI)) {
                putInMemory(scrutariURI, code);
            } else {
                putInDatabase(scrutariURI, code);
            }
            maxCode++;
            return code;
        }
        
        public int getCode(ScrutariURI scrutariURI) {
            if (isInMemoryURI(scrutariURI)) {
                return getCodeInMemory(scrutariURI);
            } else {
                return getCodeInDatabase(scrutariURI);
            }
        }
        
        private int getCodeInDatabase(ScrutariURI scrutariURI) {
            DatabaseEntry key = URITupleBinding.toDatabaseEntry(scrutariURI);
            DatabaseEntry primaryKey = new DatabaseEntry();
            DatabaseEntry data = new DatabaseEntry();
            try {
                OperationStatus op = uriIndex.get(null,key,primaryKey,data,null);
                if (op == OperationStatus.SUCCESS) {
                    return IntegerBinding.entryToInt(primaryKey);
                } else {
                    return -1;
                }
            } catch(DatabaseException de) {
                throw new NestedLibraryException(de);
            }
        }
        
        private void putInDatabase(ScrutariURI scrutariURI, int code) {
            DatabaseEntry key = new DatabaseEntry();
            IntegerBinding.intToEntry(code,key);
            try {
                uriDB.put(null,key,URITupleBinding.toDatabaseEntry(scrutariURI));
            } catch(DatabaseException de) {
                throw new NestedLibraryException(de);
            }
        }
        
        
        private int getCodeInMemory(ScrutariURI scrutariURI) {
            Integer itg = (Integer) uriMap.get(scrutariURI);
            if (itg != null) return itg.intValue();
            else return -1;
        }
        
        private void putInMemory(ScrutariURI scrutariURI, int code) {
            uriMap.put(scrutariURI,new Integer(code));
        }
        
        private boolean isInMemoryURI(ScrutariURI scrutariURI) {
            short type = scrutariURI.getType();
            switch(type) {
                case ScrutariURI.BASEURI_TYPE :
                case ScrutariURI.CORPUSURI_TYPE :
                case ScrutariURI.THESAURUSURI_TYPE :
                    return true;
                case ScrutariURI.FICHEURI_TYPE :
                case ScrutariURI.MOTCLEURI_TYPE :
                    return false;
                default :
                    throw new SwitchException("scrutariURI.getType() = " + type);
                    
            }
        }
    }
    
    
    private static class ScrutariURIKeyCreator extends TupleTupleKeyCreator {
        
        public ScrutariURIKeyCreator() {
        }
        
        public boolean createSecondaryKey(TupleInput primaryKeyInput,TupleInput dataInput,TupleOutput indexKeyOutput) {
            ScrutariURI scrutariURI = URITupleBinding.entryToScrutariURI(dataInput);
            URITupleBinding.scrutariURIToEntry(scrutariURI,indexKeyOutput);
            return true;
        }
        
    }
    
    private static class MotcleKeyCreator implements SecondaryKeyCreator {
        
        public MotcleKeyCreator() {
        }
        
        public boolean createSecondaryKey(SecondaryDatabase secondary,DatabaseEntry key,DatabaseEntry data,DatabaseEntry result) throws DatabaseException {
            IndexationData indexationData = IndexationData.fromDatabaseEntry(data);
            long lg = PrimUtils.toLong(indexationData.getCodeMotcle(),indexationData.getCodeCorpus());
            LongBinding.longToEntry(lg,result);
            return true;
        }
        
    }
    
    private abstract class IndexationIterator implements Iterator {
        private IndexationData currentIndexationData = null;
        
        public boolean hasNext() {
            return (currentIndexationData != null);
        }
        
        public Object next() {
            IndexationData result = currentIndexationData;
            if (currentIndexationData != null) initNext();
            return result;
        }
        
        public void remove() {
            throw new UnsupportedOperationException();
        }
        
        protected void initFirstIndexationData(IndexationData currentIndexationData) {
            this.currentIndexationData = currentIndexationData;
        }
        
        private void initNext() {
            Cursor cursor = getCursor();
            DatabaseEntry value = new DatabaseEntry();
            try {
                OperationStatus op = null;
                if (needDuplicate()) {
                    op = cursor.getNextDup(new DatabaseEntry(), value, null);
                } else {
                    op = cursor.getNext(new DatabaseEntry(), value, null);
                }
                if (op == OperationStatus.SUCCESS) {
                    currentIndexationData = IndexationData.fromDatabaseEntry(value);
                } else {
                    cursor.close();
                    currentIndexationData = null;
                }
            } catch(DatabaseException de) {
                throw new NestedLibraryException(de);
            }
        }
        
        protected abstract boolean needDuplicate();
        
        protected abstract Cursor getCursor();
    }
    
    private class GlobalIndexationIterator extends IndexationIterator {
        Cursor cursor;
        
        GlobalIndexationIterator() {
            try {
                cursor = indexationDB.openCursor(null, null);
                DatabaseEntry value = new DatabaseEntry();
                OperationStatus op = cursor.getFirst(new DatabaseEntry(),value,null);
                if (op  == OperationStatus.SUCCESS) {
                    initFirstIndexationData(IndexationData.fromDatabaseEntry(value));
                } else {
                    cursor.close();
                }
                
            } catch(DatabaseException de) {
                throw new NestedLibraryException(de);
            }
        }
        
        protected Cursor getCursor() {
            return cursor;
        }
        
        protected boolean needDuplicate() {
            return false;
        }
        
    }
    
    private class MotcleIndexationIterator extends IndexationIterator {
        SecondaryCursor cursor;
        
        MotcleIndexationIterator(int codemotcle, int codecorpus) {
            DatabaseEntry secondaryKey = new DatabaseEntry();
            long lg = PrimUtils.toLong(codemotcle, codecorpus);
            LongBinding.longToEntry(lg,secondaryKey);
            try {
                DatabaseEntry value = new DatabaseEntry();
                cursor = motcleIndex.openSecondaryCursor(null, null);
                OperationStatus op = cursor.getSearchKey(secondaryKey,value,null);
                if (op == OperationStatus.SUCCESS) {
                    initFirstIndexationData(IndexationData.fromDatabaseEntry(value));
                } else {
                    cursor.close();
                }
                
            } catch(DatabaseException de) {
                throw new NestedLibraryException(de);
            }
        }
        
        protected Cursor getCursor() {
            return cursor;
        }
        
        protected boolean needDuplicate() {
            return true;
        }
        
    }
    
    
}
