/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.*;
import java.util.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.scrutari.data.*;
import com.sleepycat.je.*;
import com.sleepycat.bind.tuple.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.primitives.*;
import java.util.*;
import net.mapeadores.scrutari.db.lexiebuilder.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieManager2 {
    private Environment environment;
    private LexieQuery lexieQuery;
    Database lexieUnitDB;
    Database bestPositionDB;
    Database occurrenceListDB;
    Database lexificationDB;
    LinkedIntHashMap mapOfMap;
    SecondaryDatabase langIndex;
    
    public LexieManager2(Environment env) {
        this.environment = env;
        try {
            DatabaseConfig databaseConfig = new DatabaseConfig();
            databaseConfig.setAllowCreate(true);
            lexieUnitDB = env.openDatabase(null,"lexieUnitDB",databaseConfig);
            bestPositionDB = env.openDatabase(null,"bestPositionDB",databaseConfig);
            lexificationDB = env.openDatabase(null,"lexificationDB",databaseConfig);
            occurrenceListDB = env.openDatabase(null,"occurrenceListDB",databaseConfig);
            SecondaryConfig secConfig = new SecondaryConfig();
            secConfig.setAllowCreate(true);
            secConfig.setSortedDuplicates(true);
            secConfig.setKeyCreator(new LangKeyCreator());
            langIndex = env.openSecondaryDatabase(null, "lexieUnitDB_langIndex", lexieUnitDB,secConfig);
            
        } catch(com.sleepycat.je.DatabaseException de) {
            throw new NestedLibraryException(de);
        }
        lexieQuery = new LexieQuery(this);
    }
    
    public CollatedKeyMap getCollatedLexieMapByLang(int langInteger) {
        return (CollatedKeyMap) mapOfMap.get(langInteger);
    }
    
    public LexieQuery getLexieQuery() {
        return lexieQuery;
    }
    
    
    public int getLexieCount() {
        try {
            return (int) lexieUnitDB.count();
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    private void putBestPositionData(long lexieId, int code,BestPosition[] array) {
        DatabaseEntry key = LexieLinkKey.toDatabaseEntry(lexieId,code);
        try {
            bestPositionDB.put(null,key,LexieUtils.toDatabaseEntry(array));
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    private void putOccurenceListArray(long lexieId, int code, OccurrenceList[] array) {
        DatabaseEntry key = LexieLinkKey.toDatabaseEntry(lexieId,code);
        try {
            occurrenceListDB.put(null, key, OccurrenceList.toArrayDatabaseEntry(array));
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    private void putFicheLexification(int codefiche, FicheLexification ficheLexification) {
        DatabaseEntry key = new DatabaseEntry();
        IntegerBinding.intToEntry(codefiche,key);
        try {
            lexificationDB.put(null, key, ficheLexification.toDatabaseEntry());
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    private void putMotcleLexification(int codemotcle, MotcleLexification motcleLexification) {
        DatabaseEntry key = new DatabaseEntry();
        IntegerBinding.intToEntry(codemotcle,key);
        try {
            lexificationDB.put(null, key, motcleLexification.toDatabaseEntry());
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    private void putScrutariLexieUnit(ScrutariLexieUnit scrutariLexieUnit) {
        DatabaseEntry key = new DatabaseEntry();
        LongBinding.longToEntry(scrutariLexieUnit.getLexieId(),key);
        try {
            lexieUnitDB.put(null,key,scrutariLexieUnit.toDatabaseEntry());
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    void close() {
        try {
            langIndex.close();
            close(bestPositionDB);
            close(lexieUnitDB);
            close(occurrenceListDB);
            close(lexificationDB);
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    private void close(Database database) throws DatabaseException {
        String name = database.getDatabaseName();
        database.close();
        environment.removeDatabase(null, name);
    }
    
    public static LexieManager2.Builder newBuilder(Environment environment) {
        Builder builder = new Builder(environment);
        return builder;
    }
    
    public static class Builder {
        LexieManager2 lexieManager;
        FicheLexieBuilder ficheLexieBuilder;
        MotcleLexieBuilder motcleLexieBuilder;
        LexieUnitBuilder lexieUnitBuilder;
        
        private Builder(Environment environment) {
            lexieManager = new LexieManager2(environment);
            lexieUnitBuilder = new LexieUnitBuilder();
            ficheLexieBuilder = new FicheLexieBuilder(lexieUnitBuilder);
            motcleLexieBuilder = new MotcleLexieBuilder(lexieUnitBuilder);
        }
        
        public LexieManager2 toLexieManager() {
            checkCorpus();
            checkThesaurus();
            Collection coll = lexieUnitBuilder.getLexieUnitList();
            for(Iterator it = coll.iterator();it.hasNext();) {
                ScrutariLexieUnit scrutariLexieUnit = (ScrutariLexieUnit) it.next();
                lexieManager.putScrutariLexieUnit(scrutariLexieUnit);
            }
            lexieManager.mapOfMap = lexieUnitBuilder.getMapOfCollatedLexieMap();
            return lexieManager;
        }
        
        public void newCorpus(int codecorpus) {
            checkCorpus();
            ficheLexieBuilder.setCurrentCodecorpus(codecorpus);
        }
        
        public void newThesaurus(int codethesaurus) {
            checkThesaurus();
            motcleLexieBuilder.setCurrentCodethesaurus(codethesaurus);
        }
        
        private void checkThesaurus() {
            int currentCodethesaurus = motcleLexieBuilder.getCurrentCodethesaurus();
            if (currentCodethesaurus != -1) {
                checkLexieInfo(motcleLexieBuilder, currentCodethesaurus);
            }
        }
        
        private void checkCorpus() {
            int currentCodecorpus = ficheLexieBuilder.getCurrentCodeCorpus();
            if (currentCodecorpus != -1) {
                checkLexieInfo(ficheLexieBuilder,currentCodecorpus);
            }
        }
        
        private void checkLexieInfo(AbstractLexieBuilder abstractLexieBuilder, int code) {
            Map map = abstractLexieBuilder.popLexieInfoMap();
            for(Iterator it = map.entrySet().iterator(); it.hasNext();) {
                Map.Entry entry = (Map.Entry) it.next();
                long lg = ((Long) entry.getKey()).longValue();
                AbstractLexieBuilder.LexieInfo lexieInfo = (AbstractLexieBuilder.LexieInfo) entry.getValue();
                lexieManager.putBestPositionData(lg,code, lexieInfo.getBestPositionArray());
                lexieManager.putOccurenceListArray(lg,code,lexieInfo.getOccurrenceListArray());
            }
        }
        
        public void initFicheData(FicheData ficheData) {
            FicheLexification ficheLexification = ficheLexieBuilder.initFicheData(ficheData);
            lexieManager.putFicheLexification(ficheData.getCodeFiche(), ficheLexification);
        }
        
        public void initMotcleData(MotcleData motcleData) {
            MotcleLexification motcleLexification = motcleLexieBuilder.initMotcleData(motcleData);
            lexieManager.putMotcleLexification(motcleData.getCodeMotcle(), motcleLexification);
        }
    }
    
    private static class LangKeyCreator implements SecondaryKeyCreator {
        
        public LangKeyCreator() {
        }
        
        public boolean createSecondaryKey(SecondaryDatabase secondary,DatabaseEntry key,DatabaseEntry data,DatabaseEntry result) throws DatabaseException {
            long lg = LongBinding.entryToLong(key);
            IntegerBinding.intToEntry(PrimUtils.getInt1(lg),result);
            return true;
        }
        
    }
    
    
}
