/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.data.*;
import com.sleepycat.je.*;
import java.util.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.logging.*;
import java.io.*;

/**
 * Classe regroupant les différents élément d'une base.
 *
 * @author Vincent Calame
 */
public class ScrutariDB {
    private String name;
    private Environment environment;
    private DataManager dataManager;
    private LexieManager2 lexieManager;
    private LangStats langStats = new LangStats();
    private CountStats countStats = new CountStats();
    
    public ScrutariDB(String name,Environment environment) {
        this.name = name;
        this.environment = environment;
        dataManager = new DataManager(environment);
    }
    
    public String getName() {
        return name;
    }
    
    public DataManager getDataManager() {
        return dataManager;
    }
    
    
    public LexieManager2 getLexieManager() {
        return lexieManager;
    }
    
    public LangStats getLangStats() {
        return langStats;
    }
    
    public CountStats getCountStats() {
        return countStats;
    }
    
    public void updateStats() {
        langStats.update(this);
        countStats.update(this);
    }
    
    public void close()  {
        try {
            dataManager.close();
            lexieManager.close();
            environment.close();
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    
    public boolean containsBase(BaseURI baseURI) {
        int code = dataManager.getScrutariURICoder().getCode(baseURI);
        if (code == -1) return false;
        return (dataManager.getBaseDataByCode(code) != null);
    }
    
    public static ScrutariDB.Builder newBuilder(String name,Environment environment) {
        Builder builder = new Builder(name,environment);
        return builder;
    }
    
    public static class Builder {
        ScrutariDB scrutariDB;
        LexieManager2.Builder lexieManagerBuilder;
        
        private Builder(String name,Environment environment) {
            this.scrutariDB = new ScrutariDB(name, environment);
            lexieManagerBuilder = LexieManager2.newBuilder(environment);
        }
        
        public boolean containsBase(BaseURI baseURI) {
            return scrutariDB.containsBase(baseURI);
        }
        
        public ScrutariDB toScrutariDB() {
            scrutariDB.lexieManager = lexieManagerBuilder.toLexieManager();
            return scrutariDB;
        }
        
        public void addBase(BaseData.Builder baseDataBuilder) {
            DataManager dataManager = scrutariDB.dataManager;
            ScrutariURICoder uriCoder = dataManager.getScrutariURICoder();
            BaseData baseInfo = baseDataBuilder.toBaseData(uriCoder);
            dataManager.addBaseData(baseInfo);
            List corpusList = baseDataBuilder.getCorpusDataBuilderList();
            for (Iterator it = corpusList.iterator();it.hasNext();) {
                CorpusData.Builder corpusDataBuilder = (CorpusData.Builder) it.next();
                short defaultType = corpusDataBuilder.getDefaultType();
                String hrefParent = corpusDataBuilder.getHrefParent();
                CorpusData corpusData = corpusDataBuilder.toCorpusData(uriCoder);
                dataManager.addCorpusData(corpusData);
                List ficheList = corpusDataBuilder.getFicheDataBuilderList();
                lexieManagerBuilder.newCorpus(corpusData.getCodeCorpus());
                for(Iterator it2 = ficheList.iterator();it2.hasNext();) {
                    FicheData.Builder ficheInfoBuffer = (FicheData.Builder) it2.next();
                    FicheData ficheData = ficheInfoBuffer.toFicheData(uriCoder,defaultType,hrefParent);
                    dataManager.addFicheData(ficheData);
                    lexieManagerBuilder.initFicheData(ficheData);
                }
            }
            List thesaurusList = baseDataBuilder.getThesaurusDataBuilderList();
            for (Iterator it = thesaurusList.iterator();it.hasNext();) {
                ThesaurusData.Builder thesaurusDataBuilder = (ThesaurusData.Builder) it.next();
                ThesaurusData thesaurusData = thesaurusDataBuilder.toThesaurusData(uriCoder);
                dataManager.addThesaurusData(thesaurusData);
                List motcleList = thesaurusDataBuilder.getMotcleDataBuilderList();
                lexieManagerBuilder.newThesaurus(thesaurusData.getCodeThesaurus());
                for(Iterator it2 = motcleList.iterator();it2.hasNext();) {
                    MotcleData.Builder motcleInfoBuffer = (MotcleData.Builder) it2.next();
                    MotcleData motcleData = motcleInfoBuffer.toMotcleData(uriCoder);
                    dataManager.addMotcleData(motcleData);
                    lexieManagerBuilder.initMotcleData(motcleData);
                }
            }
        }
        
        public void addIndexationBuilderList(IndexationData.BuilderList indexationDataBuilderList) {
            DataManager dataManager = scrutariDB.dataManager;
            ScrutariURICoder uriCoder = dataManager.getScrutariURICoder();
            int poidsMax = indexationDataBuilderList.getPoidsMax();
            List list = indexationDataBuilderList.getIndexationDataBuilderList();
            for (Iterator it = list.iterator();it.hasNext();) {
                IndexationData.Builder indexationDataBuilder = (IndexationData.Builder) it.next();
                IndexationData indexationData = indexationDataBuilder.toIndexationData(uriCoder,poidsMax);
                if (indexationData != null) dataManager.addIndexationData(indexationData);
            }
        }
        
    }
    
}
