/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db.lexiebuilder;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractLexieBuilder {
    private LexieUnitBuilder lexieUnitBuilder;
    private InternalLexieParserListener listener = new InternalLexieParserListener();
    protected int currentWholeLength;
    protected String currentSource;
    private int currentLangInteger;
    private LexieFilter currentLexieFilter;
    private Map lexieInfoMap = new HashMap();
    private Map currentLexieOccurrenceBuilderMap = new HashMap();
    private List lexieIdList = new ArrayList();
    private List collationUnitPositionList = new ArrayList();
    
    public AbstractLexieBuilder(LexieUnitBuilder lexieUnitBuilder) {
        this.lexieUnitBuilder = lexieUnitBuilder;
    }
    
    protected void tokenize(String chaine) {
        if (chaine == null) return;
        currentSource = chaine;
        currentWholeLength = chaine.length();
        if (currentWholeLength == 0) return;
        LexieParser.parse(currentSource,listener);
    }
    
    public Map popLexieInfoMap() {
        Map result = lexieInfoMap;
        lexieInfoMap = new HashMap();
        return result;
    }
    
    protected void flushLexieOccurrenceBuilderMap() {
        for(Iterator it = currentLexieOccurrenceBuilderMap.values().iterator(); it.hasNext();) {
            LexieOccurrenceBuilder ll = (LexieOccurrenceBuilder) it.next();
            addLexieInfo(ll.lexieId, ll.toBestPosition(), ll.toOccurrenceList());
        }
        currentLexieOccurrenceBuilderMap.clear();
    }
    
    protected Lexification flushLexification() {
        int size = lexieIdList.size();
        if (size == 0) return null;
        long[] lexieIdArray = new long[size];
        CollationUnitPosition[] collationUnitPositionArray = new CollationUnitPosition[size];
        for(int i = 0; i < size; i++) {
            lexieIdArray[i] = ((Long) lexieIdList.get(i)).longValue();
            collationUnitPositionArray[i] = (CollationUnitPosition) collationUnitPositionList.get(i);
        }
        Lexification lexification = new Lexification();
        lexification.setWholeLength(currentWholeLength);
        lexification.setLexieIdArray(lexieIdArray);
        lexification.setCollationUnitPositionArray(collationUnitPositionArray);
        lexieIdList.clear();
        collationUnitPositionList.clear();
        return lexification;
    }
    
    protected void addLexieInfo(long lexieId, BestPosition bestPosition, OccurrenceList occurrenceList) {
        Long key = new Long(lexieId);
        LexieInfo lexieInfo = (LexieInfo) lexieInfoMap.get(key);
        if (lexieInfo == null) {
            lexieInfo = new LexieInfo();
            lexieInfoMap.put(key,lexieInfo);
        }
        lexieInfo.add(occurrenceList);
        lexieInfo.add(bestPosition);
    }
    
    private void addLexie(String lexie, int lexieStartIndex) {
        LexieOccurence lexieOccurence = lexieUnitBuilder.getLexieOccurence(getCurrentLangInteger(), lexie, isOnFiche());
        CollationUnitPosition collationUnitPosition = new CollationUnitPosition(lexieOccurence.getCollationUnit(),lexieStartIndex);
        addLexie(lexieOccurence.getLexieId(),collationUnitPosition);
    }
    
    private void addLexie(long lexieId,CollationUnitPosition collationUnitPosition) {
        Long lg = new Long(lexieId);
        LexieOccurrenceBuilder lexieLink = (LexieOccurrenceBuilder) currentLexieOccurrenceBuilderMap.get(lg);
        if (lexieLink == null) {
            lexieLink = new LexieOccurrenceBuilder(lexieId,collationUnitPosition);
            currentLexieOccurrenceBuilderMap.put(lg,lexieLink);
        }
        int currentIndex = lexieIdList.size();
        lexieIdList.add(lg);
        collationUnitPositionList.add(collationUnitPosition);
        lexieLink.addOccurrence(currentIndex);
    }
    
    
    public abstract boolean isOnFiche();
    
    private class InternalLexieParserListener implements LexieParserListener{
        
        InternalLexieParserListener() {
        }
        
        
        public void flushLexie(String lexie, int lexieStartIndex) {
            if (currentLexieFilter.acceptLexie(lexie)) {
                addLexie(lexie,lexieStartIndex);
            }
        }
        
        public void checkSpecialChar(char carac) {
            
        }
        
    }
    
    public int getCurrentLangInteger() {
        return currentLangInteger;
    }
    
    public void setCurrentLangInteger(int currentLangInteger) {
        this.currentLangInteger = currentLangInteger;
        currentLexieFilter = LocalisationLexieFilters.getLexieFilter(currentLangInteger);
    }
    
    public static class LexieInfo {
        List occurrenceListList = new ArrayList();
        List bestPositionList = new ArrayList();
        
        void add(OccurrenceList occurrenceList) {
            if (occurrenceList == null) return; //@todo
            occurrenceListList.add(occurrenceList);
        }
        
        void add(BestPosition bestPosition) {
            bestPositionList.add(bestPosition);
        }
        
        public BestPosition[] getBestPositionArray() {
            int size = bestPositionList.size();
            BestPosition[] array = new BestPosition[size];
            for(int i = 0; i < size; i++) {
                array[i] = (BestPosition) bestPositionList.get(i);
            }
            return array;
        }
        
        public OccurrenceList[] getOccurrenceListArray() {
            int size = occurrenceListList.size();
            OccurrenceList[] array = new OccurrenceList[size];
            for(int i = 0; i < size; i++) {
                array[i] = (OccurrenceList) occurrenceListList.get(i);
            }
            return array;
        }
    }
    
    protected abstract int getCurrentFieldNumber();
    
    protected abstract int getCurrentCode();
    
    protected class LexieOccurrenceBuilder {
        int wholeLength;
        int bestFieldNumber;
        CollationUnitPosition bestCollationUnitPosition;
        long lexieId;
        List occList = new ArrayList();
        
        protected LexieOccurrenceBuilder(long lexieId,CollationUnitPosition collationUnitPosition) {
            this.lexieId = lexieId;
            this.bestCollationUnitPosition = collationUnitPosition;
            this.bestFieldNumber = getCurrentFieldNumber();
            this.wholeLength = currentWholeLength;
        }
        
        protected void addOccurrence(int index) {
            Occ occ= new Occ();
            occ.setFieldNumber(getCurrentFieldNumber());
            occ.setLexieIndex(index);
            occList.add(occ);
        }
        
        public BestPosition toBestPosition() {
            BestPosition l = new BestPosition();
            l.setWholeLength(wholeLength);
            l.setCode(getCurrentCode());
            l.setBestFieldNumber(bestFieldNumber);
            l.setBestCollationUnitPosition(bestCollationUnitPosition);
            return l;
        }
        
        public OccurrenceList toOccurrenceList() {
            boolean onFiche = isOnFiche();
            OccurrenceList occurrenceList = new OccurrenceList();
            occurrenceList.setCode(getCurrentCode());
            int size = occList.size();
            int[] fieldNumberArray = null;
            if (onFiche) fieldNumberArray = new int[size];
            int[] indexArray = new int[size];
            for(int i = 0; i < size; i++) {
                Occ occ = (Occ) occList.get(i);
                if (onFiche) fieldNumberArray[i] = occ.getFieldNumber();
                indexArray[i] = occ.getLexieIndex();
            }
            if (onFiche) occurrenceList.setFieldNumberArray(fieldNumberArray);
            occurrenceList.setLexieIndexArray(indexArray);
            return occurrenceList;
        }
        
        class Occ {
            private int fieldNumber;
            private int lexieIndex;
            
            public int getFieldNumber() {
                return fieldNumber;
            }
            
            public void setFieldNumber(int fieldNumber) {
                this.fieldNumber = fieldNumber;
            }
            
            public int getLexieIndex() {
                return lexieIndex;
            }
            
            public void setLexieIndex(int lexieIndex) {
                this.lexieIndex = lexieIndex;
            }
            
            
        }
    }
    
}
