/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.data;


import java.io.Serializable;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.*;

/**
 *
 * @author  Vincent Calame
 */
public final class CorpusData implements Serializable {
    public final static int INTITULECORPUS_FIELDNUMBER = -11;
    public final static int INTITULEFICHE_FIELDNUMBER = -22;
    private int codecorpus;
    private int codebase;
    private int[] codesFiches;
    private int complementCount;
    private LinkedIntHashMap intLangMap;
    private CorpusURI corpusURI;
    
    public CorpusData(CorpusURI corpusURI,int code,int codebase,int complementCount,int[] codesFiches,LinkedIntHashMap intLangMap) {
        this.codesFiches = codesFiches;
        this.codecorpus = code;
        this.codebase = codebase;
        this.intLangMap = intLangMap;
        this.complementCount = complementCount;
        this.corpusURI = corpusURI;
    }
    
    /**
     * Retourne le nombre maximum de champs complémentaires.
     */
    public int getComplementMaxNumber() {
        return complementCount;
    }
    
    public CorpusURI getCorpusURI() {
        return corpusURI;
    }
    
    /**
     * Retourne le code du corpus.
     */
    public int getCodeCorpus() {
        return codecorpus;
    }
    
    /**
     * Retourne le code de la base.
     */
    public int getCodeBase() {
        return codebase;
    }
    
    /**
     * Retourne le total de fiches du corpus.
     */
    public int getFicheCount() {
        return codesFiches.length;
    }
    
    /**
     * Retourne le code de la i-ième fiche du corpus.
     */
    public int getCodeFiche(int i) {
        return codesFiches[i];
    }
    
    public CorpusLangData getLangDataByLang(int langInteger) {
        return (CorpusLangData) intLangMap.get(langInteger);
    }
    
    public int getFicheCountByLang(int langInteger) {
        CorpusLangData corpusLangInfo = getLangDataByLang(langInteger);
        if (corpusLangInfo == null) return 0;
        return corpusLangInfo.getFicheCount();
    }
    
    /**
     * Retourne null ou une chaîne de longueur non nulle et non composée d'espaces blancs.
     */
    public String getLibelleStringByLang(int fieldNumber,int langInteger) {
        CorpusLangData corpusLangInfo = getLangDataByLang(langInteger);
        if (corpusLangInfo == null) return null;
        Libelle lib = corpusLangInfo.getLibelle(fieldNumber);
        if (lib == null) return null;
        return lib.getLibelleString();
    }
    
    
    public int getLangDataCount() {
        return intLangMap.size();
    }
    
    public CorpusLangData getLangDataByIndex(int index) {
        return (CorpusLangData) intLangMap.getValueAt(index);
    }
    
    public static class CorpusLangData {
        int langInt;
        int ficheCount = 0;
        InternalLibelle corpusLibelle;
        InternalLibelle ficheLibelle;
        InternalLibelle[] complementsLibelle;
        
        CorpusLangData(int langInt, int complementCount) {
            this.langInt = langInt;
            complementsLibelle = new InternalLibelle[complementCount];
        }
        
        public int getFicheCount() {
            return ficheCount;
        }
        
        public Libelle getLibelle(int fieldNumber) {
            if (fieldNumber == INTITULECORPUS_FIELDNUMBER) return corpusLibelle;
            if (fieldNumber == INTITULEFICHE_FIELDNUMBER) return ficheLibelle;
            return complementsLibelle[fieldNumber-1];
        }
        
        public String getLibelleString(int fieldNumber) {
            Libelle libelle = getLibelle(fieldNumber);
            if (libelle == null) return null;
            return libelle.getLibelleString();
        }
        
        public int getLangInteger() {
            return langInt;
        }
        
        /*
         * Est appelé par CorpusInfoBuffer.putLibelle() qui s'est assuré que s n'était pas vide ni de longueur nulle après un s.trim().
         */
        void setLibelle(String s, int fieldNumber) {
            InternalLibelle lib  = new InternalLibelle(s);
            if (fieldNumber == INTITULECORPUS_FIELDNUMBER) corpusLibelle = lib;
            else if (fieldNumber == INTITULEFICHE_FIELDNUMBER) ficheLibelle = lib;
            else {
                complementsLibelle[fieldNumber-1] = lib;
            }
        }
        
        void increaseFicheCount() {
            ficheCount++;
        }
        
        class InternalLibelle implements Libelle {
            String lib;
            
            InternalLibelle(String lib) {
                this.lib = lib;
            }
            
            public int getLangInteger() {
                return langInt;
            }
            
            public String getLibelleString() {
                return lib;
            }
            
            public boolean isVo() {
                return true;
            }
            
        }
    }
    
    
    public static class Builder {
        private CorpusURI corpusURI;
        private ArrayList corpusLibelleList = new ArrayList();
        private ArrayList ficheLibelleList = new ArrayList();
        private ArrayList libelleListOfList = new ArrayList();
        private ArrayList ficheDataBuilderList = new ArrayList();
        private short defaultType = TypeConstants.UNKNOWN_TYPE;
        private String hrefParent;
        
        public Builder(CorpusURI corpusURI) {
            this.corpusURI = corpusURI;
        }
        
        public CorpusURI getCorpusURI() {
            return corpusURI;
        }
        
        public void addLibelle(int fieldNumber,Libelle libelle) {
            if (libelle == null) return;
            if (fieldNumber == CorpusData.INTITULECORPUS_FIELDNUMBER) corpusLibelleList.add(libelle);
            else if (fieldNumber == CorpusData.INTITULEFICHE_FIELDNUMBER) ficheLibelleList.add(libelle);
            else if (fieldNumber > libelleListOfList.size()) {
                throw new IllegalArgumentException("fieldNumber > libelleListOfList.size()");
            } else {
                ((ArrayList)libelleListOfList.get(fieldNumber-1)).add(libelle);
            }
        }
        
        public int createNewComplement() {
            int size = libelleListOfList.size();
            libelleListOfList.add(new ArrayList());
            return size;
        }
        
        public int getComplementMaxNumber() {
            return libelleListOfList.size();
        }
        
        public void addFicheDataBuilder(FicheData.Builder ficheInfoBuffer) {
            if (ficheInfoBuffer == null) throw new IllegalArgumentException("ficheInfoBuffer argument is null");
            ficheDataBuilderList.add(ficheInfoBuffer);
        }
        
        public List getFicheDataBuilderList() {
            return Collections.unmodifiableList(ficheDataBuilderList);
        }
        
        public CorpusData toCorpusData(ScrutariURICoder uriCoder) {
            int codecorpus = uriCoder.getOrCreateCode(corpusURI);
            int codebase = uriCoder.getOrCreateCode(corpusURI.getBaseURI());
            LinkedIntHashMap linkedHashMap = new LinkedIntHashMap();
            for (Iterator it = corpusLibelleList.iterator();it.hasNext();) {
                
                Libelle libelle = (Libelle) it.next();
                putLibelle(linkedHashMap, libelle, CorpusData.INTITULECORPUS_FIELDNUMBER);
            }
            for (Iterator it = ficheLibelleList.iterator();it.hasNext();) {
                Libelle libelle = (Libelle) it.next();
                putLibelle(linkedHashMap, libelle, CorpusData.INTITULEFICHE_FIELDNUMBER);
            }
            for (int i = 0; i < libelleListOfList.size();i++) {
                ArrayList liste = (ArrayList) libelleListOfList.get(i);
                for (Iterator it = liste.iterator();it.hasNext();) {
                    Libelle libelle = (Libelle) it.next();
                    putLibelle(linkedHashMap, libelle, i+1);
                }
            }
            int size = ficheDataBuilderList.size();
            int[] codesfiches = new int[size];
            for (int i = 0; i < size;i++) {
                FicheData.Builder ficheInfoBuffer = (FicheData.Builder) ficheDataBuilderList.get(i);
                codesfiches[i] = uriCoder.getOrCreateCode(ficheInfoBuffer.getFicheURI());
                int langInt = ficheInfoBuffer.getLangInteger();
                CorpusData.CorpusLangData langInfo = (CorpusData.CorpusLangData) linkedHashMap.get(langInt);
                if (langInfo == null) {
                    langInfo = new CorpusData.CorpusLangData(langInt,libelleListOfList.size());
                    linkedHashMap.put(langInt, langInfo);
                }
                langInfo.increaseFicheCount();
            }
            return new CorpusData(corpusURI,codecorpus,codebase, libelleListOfList.size(), codesfiches, linkedHashMap);
        }
        
    /*
     * Doit s'assurer que le libellé n'est pas null ou vide (après un trim).
     */
        private void putLibelle(LinkedIntHashMap linkedHashMap,Libelle libelle, int fieldNumber) {
            String s = libelle.getLibelleString();
            if (s == null) return;
            s = s.trim();
            if (s.length() == 0) return;
            int langInt = libelle.getLangInteger();
            CorpusData.CorpusLangData langInfo = (CorpusData.CorpusLangData) linkedHashMap.get(langInt);
            if (langInfo == null) {
                langInfo = new CorpusData.CorpusLangData(langInt,libelleListOfList.size());
                linkedHashMap.put(langInt, langInfo);
            }
            langInfo.setLibelle(s, fieldNumber);
        }

        public short getDefaultType() {
            return defaultType;
        }

        public void setDefaultType(short defaultType) {
            this.defaultType = defaultType;
        }

        public String getHrefParent() {
            return hrefParent;
        }

        public void setHrefParent(String hrefParent) {
            this.hrefParent = StringUtils.nullifie(hrefParent);
        }
    }
}
