/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.data;

import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.*;
import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;

/**
 *
 * @author Vincent Calame
 */
public class MotcleData {
    private int codemotcle;
    private int codethesaurus;
    private Libelle[] libelleArray;
    private MotcleURI motcleURI;
    
    private MotcleData() {
    }
    
    private MotcleData(MotcleURI motcleURI,int codemotcle,int codethesaurus,Libelle[] libelleArray) {
        this.codethesaurus = codethesaurus;
        this.codemotcle = codemotcle;
        this.libelleArray = libelleArray;
        this.motcleURI = motcleURI;
    }
    
    public int getCodeMotcle() {
        return codemotcle;
    }
    
    public MotcleURI getMotcleURI() {
        return motcleURI;
    }
    
    public int getCodeThesaurus() {
        return codethesaurus;
    }
    
    public int getLibelleCount() {
        return libelleArray.length;
    }
    
    public Libelle getLibelleByLang(int langInt) {
        int length = libelleArray.length;
        for (int i = 0; i < length; i++) {
            Libelle libelle = libelleArray[i];
            if (libelle.getLangInteger() == langInt) return libelle;
        }
        return null;
    }
    
    public Libelle getLibelleByIndex(int index) {
        return libelleArray[index];
    }
    
    public DatabaseEntry toDatabaseEntry() {
        DatabaseEntry data = new DatabaseEntry();
        MotcleDataTupleBinding bind = new MotcleDataTupleBinding();
        bind.objectToEntry(this,data);
        return data;
    }
    
    public static MotcleData fromDatabaseEntry(DatabaseEntry data) {
        MotcleDataTupleBinding bind = new MotcleDataTupleBinding();
        return (MotcleData) bind.entryToObject(data);
    }
    
    static class InternalLibelle implements Libelle {
        private String lib;
        private int langInteger;
        
        InternalLibelle(String lib,int langInteger) {
            this.lib = lib;
            this.langInteger = langInteger;
        }
        
        public int getLangInteger() {
            return langInteger;
        }
        
        public String getLibelleString() {
            return lib;
        }
        
        public boolean isVo() {
            return true;
        }
        
    }
    
    static private final class MotcleDataTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            MotcleData motcleData = new MotcleData();
            motcleData.motcleURI = (MotcleURI)  URITupleBinding.entryToScrutariURI(tupleInput);
            motcleData.codemotcle = tupleInput.readInt();
            motcleData.codethesaurus = tupleInput.readInt();
            int libelleLength = tupleInput.readInt();
            Libelle[] array = new Libelle[libelleLength];
            for(int i = 0; i < libelleLength; i++) {
                int langInteger = tupleInput.readInt();
                String libelleString = tupleInput.readString();
                array[i] = new InternalLibelle(libelleString, langInteger);
            }
            motcleData.libelleArray = array;
            return motcleData;
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            MotcleData motcleData = (MotcleData) obj;
            URITupleBinding.scrutariURIToEntry(motcleData.getMotcleURI(), tupleOutput);
            tupleOutput.writeInt(motcleData.codemotcle);
            tupleOutput.writeInt(motcleData.codethesaurus);
            int length = motcleData.libelleArray.length;
            tupleOutput.writeInt(length);
            for(int i = 0; i < length; i++) {
                Libelle libelle = motcleData.libelleArray[i];
                tupleOutput.writeInt(libelle.getLangInteger());
                tupleOutput.writeString(libelle.getLibelleString());
            }
            
        }
    }
    
    
    public static class Builder {
        private MotcleURI motcleURI;
        private ArrayList libelleList = new ArrayList();
        
        /**
         * Construit une instance de MotcleBuffer pour le mot-clé d'URI <code>motcleURI</code>.
         */
        public Builder(MotcleURI motcleURI) {
            this.motcleURI = motcleURI;
        }
        
        /**
         * Retourne l'URI du mot-clé.
         */
        public MotcleURI getMotcleURI() {
            return motcleURI;
        }
        
        /**
         * Ajoute une instance de l'interface Libelle.
         * S'il existe déjà un objet Libelle pour la même que le libellé en argument, ce premier libellé est remplacé.
         */
        public void addLibelle(Libelle libelle) {
            libelleList.add(libelle);
        }
        
        /**
         * Construit une instance de MotcleData à partir des données de l'objet MotcleInfoBuffer.
         */
        public MotcleData toMotcleData(ScrutariURICoder uriCoder) {
            int codemotcle = uriCoder.getOrCreateCode(motcleURI);
            int codethesaurus = uriCoder.getOrCreateCode(motcleURI.getThesaurusURI());
            int size = libelleList.size();
            Libelle[] libelleArray = new Libelle[size];
            for (int i = 0; i < size;i++) {
                Libelle libelle = (Libelle) libelleList.get(i);
                libelleArray[i] = new MotcleData.InternalLibelle(libelle.getLibelleString(),libelle.getLangInteger());
            }
            return new MotcleData(motcleURI,codemotcle,codethesaurus, libelleArray);
        }
    }
}
