/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.lexie;

import net.mapeadores.util.text.collation.CollationUnitPosition;

/**
 * Classe stockant la localisation d'un mot dans une fiche (champ auquel appartient le mot, position à l'intérieur du champ).
 *
 * @author Vincent Calame
 */
public class FicheAndLexieLink implements FieldNumbers {
    
    private int codefiche;
    private int codecorpus;
    private ScrutariLexieUnit scrutariLexieUnit;
    private InfichePosition bestPosition;
    private InfichePosition[] positionArray;
    
    /**
     * Construit une nouvelle instance de FicheAndLexieLink pour la fiche de code codefiche appartenant au corpus de code codecorpus,
     * le mot appartenant au champ de numéro fieldNumber et à la position décrite par collationUnitPosition.
     */
    public FicheAndLexieLink(ScrutariLexieUnit scrutariLexieUnit,int codefiche, int codecorpus, int fieldNumber,CollationUnitPosition wordPosition, InfichePosition previousPosition) {
        this.codefiche = codefiche;
        this.codecorpus = codecorpus;
        this.scrutariLexieUnit = scrutariLexieUnit;
        InfichePosition infichePosition = new InfichePosition(fieldNumber, wordPosition, previousPosition);
        this.bestPosition = infichePosition;
        positionArray = new InfichePosition[1];
        positionArray[0] = infichePosition;
    }
    
    /**
     * Retourne le code de la fiche considérée.
     */
    public int getCodeFiche() {
        return codefiche;
    }
    
    /**
     * Retourne le code du corups de la fiche considérée.
     *
     */
    public int getCodeCorpus() {
        return codecorpus;
    }
    
    /**
     * Retourne le numéro du champ de la fiche considérée où se trouve le mot pour la meilleur position.
     * Retourne la valeur TITRE_FIELDNUMBER,SOUSTITRE_FIELDNUMBER,NONE_FIELDNUMBER ou une valeur positive de 1 ou plus correspondant à l'index du champ complémentaire.
     */
    public int getFieldNumber() {
        return bestPosition.getFieldNumber();
    }
    
    public String getCollatedLexie() {
        return scrutariLexieUnit.getCollatedLexie();
    }
    
    public ScrutariLexieUnit getScrutariLexieUnit() {
        return scrutariLexieUnit;
    }
    
    /**
     * Retourne une instance de CollationUnitPosition décrivant la position du mot au sein du champ de la fiche considérée pour la meilleur position.
     */
    public CollationUnitPosition getCollationUnitPosition() {
        return bestPosition.getCollationUnitPosition();
    }
    
    public InfichePosition getBestPosition() {
        return bestPosition;
    }
    
    public InfichePosition[] getInfichePositionArray() {
        return positionArray;
    }
    
    /*
     * Change le numéro de champ et la position du mot. Utilisé lorsqu'un même mot se trouve dans plusieurs champs,
     * voire à plusieurs endroits dans le champ : seule la position la plus proche du début du champ de plus petit
     * numéro est conservé.
     */
    public InfichePosition add(int fieldNumber,CollationUnitPosition wordPosition, InfichePosition previousPosition) {
        InfichePosition infichePosition = new InfichePosition(fieldNumber, wordPosition, previousPosition);
        if (bestPosition.compareTo(infichePosition) == 1) {
            bestPosition = infichePosition;
        }
        int length = positionArray.length;
        InfichePosition[] temp = new InfichePosition[length + 1];
        System.arraycopy(positionArray, 0, temp, 0, length);
        temp[length] = infichePosition;
        positionArray = temp;
        return infichePosition;
    }
    
    public class InfichePosition {
        private int fieldNumber = NONE_FIELDNUMBER;
        private CollationUnitPosition collationUnitPosition;
        private InfichePosition previousPosition;
        
        public InfichePosition(int fieldNumber,CollationUnitPosition wordPosition, InfichePosition previousPosition) {
            this.fieldNumber = fieldNumber;
            this.collationUnitPosition = wordPosition;
            this.previousPosition = previousPosition;
        }
        
        public CollationUnitPosition getCollationUnitPosition() {
            return collationUnitPosition;
        }
        
        public int getFieldNumber() {
            return fieldNumber;
        }
        
        public FicheAndLexieLink getFicheWord() {
            return FicheAndLexieLink.this;
        }
        
        public InfichePosition getPrevious() {
            return previousPosition;
        }

        public int compareTo(InfichePosition infichePosition) {
            if ((fieldNumber == NONE_FIELDNUMBER) || (infichePosition.fieldNumber < fieldNumber)) {
               return 1;
            }
            else if (infichePosition.fieldNumber == fieldNumber) {
                if (infichePosition.collationUnitPosition.compareTo(collationUnitPosition) == -1) {
                  return 1;
                }
            }
            return -1;
        }
    }
    
}
