/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.searchengine;


import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.searchengine.inner.*;
import net.mapeadores.scrutari.searchengine.seeker.*;
import net.mapeadores.scrutari.searchengine.pertinence.*;
import net.mapeadores.scrutari.searchengine.util.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.langinteger.*;
import java.util.*;

/**
 * La classe SearchEngine effectue le traitement de la recherche et la sélection des fiches.
 * Cette classe n'est accessible que par une seule méthode publique statique runSearch()
 *
 * @author  Vincent Calame
 */
public class SearchEngine {
    private ScrutariDB scrutariDB;
    private GlobalSearchOptions globalSearchOptions;
    private DataManager dataManager;
    private LinkedIntHashMap motcleCodeMap = new LinkedIntHashMap();
    private LinkedIntHashMap ficheCodeMap = new LinkedIntHashMap();
    private boolean touteslangues = false;
    private int[] codescorpusArray;
    private int[] codesthesaurusArray;
    private int[] langIntArray;
    private int searchTokenCount = 0;
    private int searchTokenMask = 0;
    private SearchResultBuilder searchResultBuilder;
    private boolean avecMotscles = true;
    
    /**
     * Effectue une recherche sur la base generalDatabase en prenant les paramètres dans searchResult
     * et en stockant les résulats dans searchResult.
     */
    static public SearchResult search(SearchEngineParameters searchEngineParameters) {
        SearchEngine searchEngine =new SearchEngine(searchEngineParameters);
        return searchEngine.searchResultBuilder.toBean();
    }
    
    
    /*
     * Constructeur privé qui lance les différentes étapes de la recherche
     */
    private SearchEngine(SearchEngineParameters searchEngineParameters) {
        this.scrutariDB = searchEngineParameters.getScrutariDB();
        this.globalSearchOptions = searchEngineParameters.getGlobalSearchOptions();
        this.dataManager = scrutariDB.getDataManager();
        SearchSequence searchSequence = searchEngineParameters.getSearchSequence();
        SearchOptions searchOptions = searchEngineParameters.getSearchOptions();
        searchResultBuilder = new SearchResultBuilder(searchEngineParameters.getIdentifiant(), searchSequence, searchOptions);
        initArrays(searchOptions);
        this.searchTokenCount = searchSequence.getTokenCount();
        searchTokenMask = (int) Math.pow(2,searchTokenCount) - 1;

        searchTokens(searchSequence, searchOptions);
        checkMotclePertinence(searchOptions.getLangUi());
        finalizeSearch();
    }
    
    /*
     *Initialise les tableaux codescorpusArray, langIntArray en fonction des listes de l'argument SearchOptions
     */
    private void initArrays(SearchOptions searchOptions) {
        codescorpusArray = searchOptions.getCorpusCodeArray();
        langIntArray = searchOptions.getLangIntegerArray();
        codesthesaurusArray = searchOptions.getThesaurusCodeArray();
        searchResultBuilder.setFicheMaximum(DataUtils.getFicheCountByCorpus(dataManager,codescorpusArray, langIntArray));
        if (codescorpusArray == null) {
            codescorpusArray = dataManager.getCodecorpusArray();
        }
        if (codesthesaurusArray == null) {
            codesthesaurusArray = dataManager.getCodethesaurusArray();
        }
        if (langIntArray == null) {
            touteslangues = true;
            langIntArray = scrutariDB.getLangStats().getContentLangArray();
        }
    }
    
    public int[] getLangIntegerArray() {
        return langIntArray;
    }
    
    public int[] getCodescorpusArray() {
        return codescorpusArray;
    }
    
    public int[] getCodesthesaurusArray() {
        return codesthesaurusArray;
    }
    
    public boolean isWithMotscles() {
        return avecMotscles;
    }
    
    /*
     * Balaie les différents mots de la séquence de recherche et récupère les informations sur les fiches et les mots-clés
     *qui contiennent ces différents mots.
     */
    private void searchTokens(SearchSequence sequence, SearchOptions searchOptions) {
        LexieSeeker lexieSeeker = new LexieSeeker(this,scrutariDB.getLexieManager());
        lexieSeeker.seek(sequence, searchOptions);
        
    }
    
    private void checkMotclePertinence(int uiLangInteger) {
        if ((uiLangInteger == 0) || (uiLangInteger == LangInteger.UNDETERMINED)) return;
        if (!avecMotscles) return;
        for (int i = 0; i < motcleCodeMap.size();i++) {
            MotclePertinence motclePertinence = (MotclePertinence)  motcleCodeMap.getValueAt(i);
            motclePertinence.finalizePertinence(uiLangInteger);
        }
    }
    
    /**
     * Appelé par WordSeeker
     */
    public void addFichePertinence(int codefiche, int codecorpus, int tokenIndex,int fieldNumber,SubstringPosition sourcePosition) {
        FichePertinence pertinence = (FichePertinence) ficheCodeMap.get(codefiche);
        if (pertinence == null) {
            pertinence = new FichePertinence(codefiche,codecorpus,searchTokenCount);
            ficheCodeMap.put(codefiche,pertinence);
        }
        pertinence.addPertinence(tokenIndex,fieldNumber,sourcePosition);
    }
    
    /**
     * Appelé par WordSeeker
     */
    public void addMotclePertinence(int codemotcle, int codethesaurus, int tokenIndex,int langInteger,SubstringPosition sourcePosition, float coverRate) {
        MotclePertinence pertinence = (MotclePertinence) motcleCodeMap.get(codemotcle);
        if (pertinence == null) {
            pertinence = new MotclePertinence(codemotcle,codethesaurus,searchTokenCount);
            motcleCodeMap.put(codemotcle,pertinence);
        }
        pertinence.addPertinence(tokenIndex,langInteger,sourcePosition,coverRate);
    }

    /**
     * Modifie la pertinence des fiches en fonction de l'indexation par les mots-clés sélectionné et en profite pour 
     * trier les mots-clés. Renvoie l'ensemble trié.
     */
    private TreeSet searchIndexation() {
        if (!avecMotscles) return null;
        TreeSet indexationPertinenceSortedSet = new TreeSet(new IndexationComparator());
        int mx = searchTokenMask;
        int totalcomplets = 0;
        for (int i = 0; i < motcleCodeMap.size();i++) {
            MotclePertinence motclePertinence = (MotclePertinence)  motcleCodeMap.getValueAt(i);
            int codemotcle = motclePertinence.getCodeMotcle();
            IndexationPertinence indexationPertinence = new IndexationPertinence(motclePertinence);
            for (int k = 0; k < codescorpusArray.length;k++) {
                Iterator it = dataManager.getIndexationDataIteratorForMotcle(codemotcle,codescorpusArray[k]);
                if (it != null) {
                    while(it.hasNext()) {
                    IndexationData indexationInfo = (IndexationData) it.next();
                    FichePertinence fichePertinence = (FichePertinence) ficheCodeMap.get(indexationInfo.getCodeFiche());
                    if (fichePertinence == null) {
                        if (touteslangues) {
                            fichePertinence = new FichePertinence(indexationInfo.getCodeFiche(),indexationInfo.getCodeCorpus(),searchTokenCount);
                            ficheCodeMap.put(indexationInfo.getCodeFiche(),fichePertinence);
                        }
                        else {
                            FicheData ficheInfo = dataManager.getFicheDataByCode(indexationInfo.getCodeFiche());
                            boolean contains = false;
                            int langInt = ficheInfo.getLangInteger();
                            for (int j = 0; j < langIntArray.length;j++) {
                                if (langIntArray[j] == langInt) {
                                        contains = true;
                                        break;
                                }
                            }
                            if (contains) {
                                 fichePertinence = new FichePertinence(indexationInfo.getCodeFiche(),indexationInfo.getCodeCorpus(),searchTokenCount);
                                ficheCodeMap.put(indexationInfo.getCodeFiche(),fichePertinence);
                            }
                            else continue;
                        }
                    }
                    fichePertinence.addIndexationPertinence(indexationPertinence,indexationInfo.getPonderation());
                }
                }
            }
            if ((motclePertinence.getCompletudeMask() & mx) == mx) {
                totalcomplets ++;
            }
            indexationPertinenceSortedSet.add(indexationPertinence);
        }
        searchResultBuilder.initMotcleCount(indexationPertinenceSortedSet.size());
        searchResultBuilder.setCompleteMotcleCount(totalcomplets);
        return indexationPertinenceSortedSet;
    }
    
    /*
     * Finalise la recherche en conservant que les fiches qui remplissent tous les critères et en les classant suivant la pertinence.
     */
    private void finalizeSearch() {
        TreeSet indexationPertinenceSortedSet =  searchIndexation();
        int mx = searchTokenMask;
        PertinenceComparator pertinenceComparator = new PertinenceComparator(searchTokenCount,searchTokenMask);
        if (globalSearchOptions != null) {
            pertinenceComparator.setGlobalSearchOptions(globalSearchOptions);
        }
        TreeSet pertinenceSortedSet = new TreeSet(pertinenceComparator);
        for (int i = 0; i < ficheCodeMap.size();i++) {
            int codefiche = ficheCodeMap.getKeyAt(i);
            FichePertinence fichePertinence = (FichePertinence) ficheCodeMap.getValueAt(i);
            int bits = fichePertinence.getFicheCompletudeMask() | fichePertinence.getMotcleCompletudeMask();
            if ((bits & mx) == mx) {
                int dateInteger = dataManager.getFicheDateIntegerByCode(codefiche);
                fichePertinence.setDateInteger(dateInteger);
                pertinenceSortedSet.add(fichePertinence);
            }
            else {
                fichePertinence.setPertinenceValue(-1);
            }
        }
        searchResultBuilder.initFicheCount(pertinenceSortedSet.size());
        int p = 1;
        for (Iterator it = pertinenceSortedSet.iterator();it.hasNext();) {
            FichePertinence pertinence = (FichePertinence) it.next();
            pertinence.setPertinenceValue(p);
            p++;
            searchResultBuilder.addFicheResult(pertinence);
        }
        for (Iterator it2 = indexationPertinenceSortedSet.iterator();it2.hasNext();) {
            IndexationPertinence indexationPertinence = (IndexationPertinence) it2.next();
            indexationPertinence.sortPertinenceFiche();
            searchResultBuilder.addMotcleResult(indexationPertinence);
        }
        
    }
}
