package net.mapeadores.scrutari.searchengine.inner;

import java.io.Serializable;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import net.mapeadores.util.text.SubstringPosition;


/**
 * Classe qui conserve la liste des sous-positions des mots de la séquence de recherche
 * à l'intérieur d'un même texte (le libellé d'un mot-clé, un champ d'une fiche).
 * Les éléments de cette liste sont triés par proximité par rapport au début du texte
 * (voir l'implémentation de l'interface Comparable de la classe SubstringPosition).
 */
public class InnerSubstringPositionList implements Serializable, SearchTokenPositionList {
    private SubstringPosition[] positionArray;
    private int[] tokenIndices;
    private int size = 0;
    
    /*
     * Construit une instance à partir d'une première position. Cet unique constructeur
     * garantit que la liste n'est jamais vide.
     */
    InnerSubstringPositionList(int tokenCount, SubstringPosition substringPosition, int tokenIndex) {
        positionArray = new SubstringPosition[tokenCount];
        tokenIndices = new int[tokenCount];
        positionArray[0] = substringPosition;
        tokenIndices[0] = tokenIndex;
        size = 1;
    }
    
    /**
     * Retourne la taille de la liste.
     */
    public int size() {
        return size;
    }
    
    /**
     * Retourne la position dans la séquence de recherche du mot correspondant à la i-ième instance
     * de SubstringPosition stockée dans cette liste.
     */
    public int getSearchTokenIndexForPosition(int i) {
        return tokenIndices[i];
    }
    
    /**
     * Retourne la i-ième instance de SubstringPosition stockée dans cette liste.
     */
    public SubstringPosition getSearchTokenPosition(int i) {
        return positionArray[i];
    }
    
    /**
     * Indique si cette liste contient une instance de SubstringPosition pour tous
     * les mots de la séquence de recherche.
     */
    public boolean containsAllSearchTokens() {
        return (size == positionArray.length);
    }
    
    /**
     * AJoute une nouvelle instance de SubstringPosition correspond au mot de
     * la séquence de recherche à la position tokenIndex.
     * Un calcul est effectué pour classer les instances suivant la position décrite, voire
     * en altérant ces positions pour gérer les recouvrements.
     * Un tokenIndex n'est entré qu'une seule fois.
     */
    void add(int tokenIndex, SubstringPosition substringPosition) {
        boolean insert = false;
        for (int i = 0; i < size; i++) {
            if (substringPosition.compareTo(positionArray[i]) < 0) {
                System.arraycopy(positionArray, i, positionArray,i+1, size -i);
                System.arraycopy(tokenIndices, i, tokenIndices,i+1, size -i);
                tokenIndices[i] = tokenIndex;
                positionArray[i] = substringPosition;
                insert = true;
                break;
            }
        }
        if (!insert) {
            tokenIndices[size] = tokenIndex;
            positionArray[size] = substringPosition;
        }
        size++;
        
        /*completeSize++;
        int beginIndex = substringPosition.getBeginIndex();
        for (int i = effectiveSize-1; i >= 0;i--) {
            SubstringPosition otherPosition = substringPositions[i];
            if (otherPosition.contains(substringPosition)) {
                return;
            } else if (substringPosition.contains(otherPosition)) {
                if (i == 0) {
                    substringPositions[0] = substringPosition;
                    tokenIndices[0] = tokenIndex;
                    return;
                } else if (beginIndex > substringPositions[i-1].getEndIndex()) {
                    substringPositions[i] = substringPosition;
                    tokenIndices[i] = tokenIndex;
                    return;
                } else if (beginIndex > substringPositions[i-1].getBeginIndex()) {
                    int newBegin = substringPositions[i-1].getEndIndex()+1;
                    substringPositions[i] = new SubstringPosition(newBegin,substringPosition.getEndIndex() - newBegin+1);
                    tokenIndices[i] = tokenIndex;
                    return;
                } else {
                    if (i == (effectiveSize-1)) effectiveSize--;
                    else {
                        System.arraycopy(substringPositions, i+1,substringPositions,i, effectiveSize -i - 1);
                        System.arraycopy(tokenIndices, i+1,tokenIndices,i, effectiveSize -i - 1);
                        effectiveSize --;
                    }
                    continue;
                }
            } else if (beginIndex > otherPosition.getBeginIndex()) {
                if (beginIndex <= otherPosition.getEndIndex()) {
                    int newBegin = substringPositions[i].getEndIndex()+1;
                    substringPosition = new SubstringPosition(newBegin,substringPosition.getEndIndex() - newBegin+1);
                }
                if (i < (effectiveSize-1)) {
                    System.arraycopy(substringPositions, i+1,substringPositions,i+2, effectiveSize -i - 1);
                    System.arraycopy(tokenIndices, i+1,tokenIndices,i+2, effectiveSize -i - 1);
                }
                substringPositions[i+1] = substringPosition;
                tokenIndices[i+1] = tokenIndex;
                effectiveSize++;
            } else if (substringPosition.getEndIndex() >= otherPosition.getBeginIndex()) {
                substringPosition = new SubstringPosition(beginIndex, otherPosition.getBeginIndex() - beginIndex);
            }
        }*/
        
       /* for (int i = 0; i < size;i++) {
            SubstringPosition otherPosition = substringPositions[i];
            if (substringPosition.contains(otherPosition)) {
                substringPositions[i] = new SubstringPosition(otherPosition.getBeginIndex(),0);
            } else if (otherPosition.contains(substringPosition)) {
                substringPosition = new SubstringPosition(substringPosition.getBeginIndex(),0);
                break;
            } else if (substringPosition.getEndIndex() >= otherPosition.getBeginIndex()) {
                substringPosition = new SubstringPosition(substringPosition.getBeginIndex(), otherPosition.getBeginIndex() - substringPosition.getBeginIndex());
            } else if (substringPosition.getBeginIndex() <= otherPosition.getEndIndex()) {
                int newBegin = otherPosition.getEndIndex() +1;
                substringPosition = new SubstringPosition(newBegin,substringPosition.getEndIndex() - newBegin+1);
            }
        }*/
    }
}