/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.searchengine.pertinence;

import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.primitives.*;
import java.util.Comparator;

/**
 * Classe servant au calcul de la pertinence d'une fiche. Les instances de cette
 * classe sont utilisées tout au long du traitement effectué par la classe SearchEngine
 * et servent à construire le résultat final via les instances de SearchResult.  
 *
 * @author Vincent Calame
 */
public class FichePertinence implements FieldNumbers {
    static final AcceptedComparator ACCEPTED_COMPARATOR = new AcceptedComparator();
    private int codefiche;
    private int codecorpus;
    private int minimalFieldNumber = 999;
    private int[] fieldNumberArray;
    private float[] ponderationArray;
    private float ponderationMax = 0;
    private int minimalBeginIndex = 9999;
    private SubstringPosition[] substringPositionArray;
    private MotclePertinence pertinencemotclemax;
    private MotclePertinence[] motclePertinenceArray;
    private int bitscompletudetexte = 0;
    private int bitscompletudemotscles = 0;
    private int searchTokenCountInTitre = 0;
    private int searchTokenCountInSoustitre = 0;
    private int searchTokenCountInMotscles = 0;
    
    private int pertinenceValue = 0;
    
    private int dateInteger;
    private int annee;
    
    /**
     * Construit une instance de FichePertinence pour une fiche donnée.
     */
    public FichePertinence(int codefiche,int codecorpus,int sequenceTokenCount) {
        this.codefiche = codefiche;
        this.codecorpus = codecorpus;
        fieldNumberArray = new int[sequenceTokenCount];
        substringPositionArray = new SubstringPosition[sequenceTokenCount];
        motclePertinenceArray = new MotclePertinence[sequenceTokenCount];
        ponderationArray = new float[sequenceTokenCount];
    }
    
    /**
     * Retourne le code de la fiche dont traite l'instance de FichePertinence.
     */
    public int getCodeFiche() {
        return codefiche;
    }
    
    /**
     * Retourne le code du corpus de la fiche dont traite l'instance de FichePertinence.
     */
    public int getCodeCorpus() {
        return codecorpus;
    }
    
    /**
     * Retourne le « masque de complétude » de la fiche. Autrement dit, l'information
     * sur quel mot de la séquence de la recherche se trouve effectivement dans la
     * fiche (dans le titre, le sous-titre ou un champ complémentaire). Un mot 
     * de la recherche se trouve dans la fiche si son « bit » correspondant vaut 1. 
     */
    public int getFicheCompletudeMask() {
        return bitscompletudetexte;
    }
    
    /**
     * Retourne le « masque de complétude » des mots-clés indexant la fiche. Autrement dit, l'information
     * sur quel mot de la séquence de la recherche se trouve effectivement dans un des
     * mots-clés qui indexe la fiche. Un mot 
     * de la recherche se trouve dans un des mots-clés si son « bit » correspondant vaut 1.
     * <p>Si <em> getMotcleCompletudeMask()</em> retourne 0, cela signifie qu'aucun
     * mot-clé répondant à la recherche n'indexe la fiche.
     */
    public int getMotcleCompletudeMask() {
        return bitscompletudemotscles;
    }
    
    /**
     * Retourne un tableau indiquant le numéro du champ dans lequel se trouve chaque mot
     * de la séquence de la recherche. Ce tableau a donc comme longueur le nombre de mots
     * de la séquence de recherche. Si le i-ème mot ne se trouve pas aucun champ, la valeur du tableau en i
     * est 0 (valeur FicheWord.NONE_FIELDNUMBER).
     */
    public int[] getFieldNumberByTokenArray() {
        return fieldNumberArray;
    }
    
    /**
     * Retourne un tableau indiquant la position de chaque mot de la séquence de recherche
     * dans du champ dans lequel il se trouve chaque mot. Ce tableau a donc comme longueur le nombre de mots
     * de la séquence de recherche.
     * <p>Si le i-ème mot de la séquence ne se trouve dans aucun champ, la valeur du tableau en i
     * est nulle. On a donc une correspondance avec le tableau renvoyé par la méthode <em>getFieldNumberByTokenArray()</em>
     * puisque (getFieldNumberByTokenArray()[i] == 0) == (getSubstringPositionByTokenArray()[i]);
     */
    public SubstringPosition[] getSubstringPositionByTokenArray() {
        return substringPositionArray;
    }
    
    /**
     * Retourne un tableau contenant les instances de MotclePertinence les plus « pertinentes »
     * pour chaque mot de la séquence de recherche. Ce tableau a donc comme longueur le nombre de mots
     * de la séquence de recherche.
     *<p>si le i-ème mot de la séquence ne se trouve dans aucun mot indexant la fiche, la valeur
     * du tableau en i est nulle. On a alors :
     * (getMotclePertinenceByTokenArray()[i] == null) == (getMotcleCompletudeMask() & Math.pow(i,2) == 0)
     */
    public MotclePertinence[] getMotclePertinenceByTokenArray() {
        return motclePertinenceArray;
    }
    
    public MotclePertinence getMotclePertinenceMax() {
        return pertinencemotclemax;
    }
    
    public float getPonderationMax() {
        return ponderationMax;
    }
    
    /**
     * Il s'agit du plus petit champ (le titre étant le champ le plus petit)
     * dans lequel se trouve un des mots de la recherche.
     */
    public int getMinimalFieldNumber() {
        return minimalFieldNumber;
    }
    
    /**
     * Il s'agit du plus petit index auquel commence un des mots de la recherche
     *dans le plus petit champ. Cette valeur peut permettre de départager les égalités de getMinimalFieldNumber()
     */
    public int getMinimalBeginIndex() {
        return minimalBeginIndex;
    }
    
    /**
     * Il s'agit du nombre de mots de la recherche qui se trouvent dans le titre.
     */
    public int getSearchTokenCountInTitre() {
        return searchTokenCountInTitre;
    }
    
    /**
     * Il s'agit du nombre de mots de la recherche qui se trouvent dans le sous-titre.
     */
    public int getSearchTokenCountInSoustitre() {
        return searchTokenCountInSoustitre;
    }
    
    /**
     * Il s'agit du nombre de mots de la recherche qui se trouvent dans les mots-clés.
     */
    public int getSearchTokenCountInMotscles() {
        return searchTokenCountInMotscles;
    }
        
    /**
     * Retourne la valeur de pertinence, à savoir -1 si la fiche ne s'est pas révélee
     * pertinente ou une valeur strictement positive indiquant le rang dans la pertinence.
     * Les valeurs de pertinence sont toutes difféntes.
     */
    public int getPertinenceValue() {
        return pertinenceValue;
    }
    
    /*
     * Ajoute une information de pertinence (numéro de champ, position dans le champ) pour le mot de la séquence de recherche
     * à la position tokenIndex. Cette méthode est abondamment utilisée dans la classe SearchEngine.
     */
    public void addPertinence(int tokenIndex,int fieldNumber,SubstringPosition substringPosition) {
        bitscompletudetexte = (bitscompletudetexte | ((int) Math.pow(2,tokenIndex)));
        if (fieldNumberArray[tokenIndex] == NONE_FIELDNUMBER) {
            substringPositionArray[tokenIndex] = substringPosition;
            fieldNumberArray[tokenIndex] = fieldNumber;
            if (fieldNumber == TITRE_FIELDNUMBER) {
                searchTokenCountInTitre++;
            } else if (fieldNumber == SOUSTITRE_FIELDNUMBER) {
                searchTokenCountInSoustitre++;
            }
        } else if (fieldNumber != NOPERTINENT_FIELDNUMBER) {
            if ((fieldNumberArray[tokenIndex] == NOPERTINENT_FIELDNUMBER) || (fieldNumber < fieldNumberArray[tokenIndex])) {
                substringPositionArray[tokenIndex] = substringPosition;
                if (fieldNumber  == TITRE_FIELDNUMBER) {
                    searchTokenCountInTitre++;
                    if (fieldNumberArray[tokenIndex] == SOUSTITRE_FIELDNUMBER) searchTokenCountInSoustitre--;
                } else if (fieldNumber == SOUSTITRE_FIELDNUMBER) {
                    searchTokenCountInSoustitre++;
                }
                fieldNumberArray[tokenIndex] = fieldNumber;
            } else if (fieldNumber == fieldNumberArray[tokenIndex]) {
                if (substringPosition.getBeginIndex() < substringPositionArray[tokenIndex].getBeginIndex()) {
                    substringPositionArray[tokenIndex] = substringPosition;
                }
            }
            
        }
        if (fieldNumber != NOPERTINENT_FIELDNUMBER) {
            if (fieldNumber < minimalFieldNumber) {
                minimalFieldNumber = fieldNumber;
                minimalBeginIndex = substringPosition.getBeginIndex();
            } else if (fieldNumber == minimalFieldNumber) {
                minimalBeginIndex = Math.min(minimalBeginIndex,substringPosition.getBeginIndex());
            }
        }
    }
    
    /**
     * Ajoute une information de pertinence d'un mot-clé. Un seul mot-clé est retenu par mot
     * de la séquence comme mot-clé le plus pertinent. La valeur relativeWeight donne
     * le poids de l'indexation de la fiche par le mot-clé. Cette valeur joue un rôle à la fois 
     * dans la sélection du mot-clé le plus pertinent et dans le tri des instances de FichePertinence
     * entre elles.
     * Cette méthode est abondamment utilisée dans la classe SearchEngine.
     */
    public void addIndexationPertinence(IndexationPertinence indexationPertinence, float ponderation) {
        indexationPertinence.addFichePertinence(this);
        MotclePertinence motclePertinence = indexationPertinence.getMotclePertinence();
        int bitsmotcle = motclePertinence.getCompletudeMask();
        for (int i = 0;i < motclePertinenceArray.length;i++) {
            int rg = (int) Math.pow(2, i);
            if ((bitsmotcle & rg) == rg) {
                if (motclePertinenceArray[i] == null) {
                    motclePertinenceArray[i] = motclePertinence;
                    ponderationArray[i] = ponderation;
                } else {
                    int compcompl = motclePertinence.compareCompletude(motclePertinenceArray[i]);
                    if (compcompl < 0) {
                        motclePertinenceArray[i] = motclePertinence;
                        ponderationArray[i] = ponderation;
                    } else if (compcompl == 0) {
                        if (ponderation > ponderationArray[i]) {
                            motclePertinenceArray[i] = motclePertinence;
                            ponderationArray[i] = ponderation;
                        } else if (ponderation == ponderationArray[i]) {
                            if (motclePertinence.compareTo(motclePertinenceArray[i]) < 0) {
                                motclePertinenceArray[i] = motclePertinence;
                            }
                        }
                    }
                }
                if ((bitscompletudemotscles & rg) == 0) {
                    bitscompletudemotscles =  bitscompletudemotscles | rg;
                    searchTokenCountInMotscles++;
                }
            }
        }
        if (pertinencemotclemax == null) {
            pertinencemotclemax = motclePertinence;
            ponderationMax = ponderation;
        } else {
            int compcompl = motclePertinence.compareCompletude(pertinencemotclemax);
            if (compcompl < 0) {
                pertinencemotclemax = motclePertinence;
                ponderationMax = ponderation;
            } else if (compcompl == 0) {
                if (ponderation > ponderationMax) {
                    pertinencemotclemax = motclePertinence;
                    ponderationMax = ponderation;
                } else if (ponderation == ponderationMax) {
                    if (motclePertinence.compareTo(pertinencemotclemax) < 0) {
                        pertinencemotclemax = motclePertinence;
                    }
                }
            }
        }
    }
    
    /*
     * Fixe la valeur de pertience. Cette méthode est utilisée en toute fin de traitement
     * au sein de la classe SearchEngine afin de fixer la valeur définitive de pertinence.
     * Cette valeur est -1 si la fiche n'est pas retenue, sinon 1 ou plus dans l'ordre de pertinence.
     */
    public void setPertinenceValue(int pertinenceValue) {
        this.pertinenceValue = pertinenceValue;
    }
    
    /*
     * Comparateur qui intervient en fin de traitement, une fois que les valeurs de pertinence ont été fixées et
     * qui sert à trier les fiches en fonction de cette valeur. Ce comparateur n'est intéressant que pour les fiches
     * déjà triées.
     */
    static class AcceptedComparator implements Comparator {
        
        public int compare(Object obj1,Object obj2) {
            if (obj1 == obj2) return 0;
            FichePertinence pf1 = (FichePertinence) obj1;
            FichePertinence pf2 = (FichePertinence) obj2;
            if (pf1.pertinenceValue < pf2.pertinenceValue) return -1;
            else return 1;
        }
    } 

    public int getDateInteger() {
        return dateInteger;
    }

    public void setDateInteger(int dateInteger) {
        this.dateInteger = dateInteger;
        if (dateInteger != IntDate.NULL) {
            annee = IntDate.getAnnee(dateInteger);
        }
        else {
            annee = IntDate.NULL;
        }
    }

    public int getAnnee() {
        return annee;
    }
}