/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.xml;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.uri.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.logging.*;


/**
 *
 * @author Vincent Calame
 */
public class XMLReader implements ErrorKeys {
    private HashSet uriMap = new HashSet();
    private ScrutariDB.Builder scrutariDBBuilder;
    private ScrutariXmlErrorHandler scrutariXmlErrorHandler;
    
    public XMLReader(ScrutariDB.Builder scrutariDBBuilder,ScrutariXmlErrorHandler scrutariXmlErrorHandler) {
        this.scrutariDBBuilder = scrutariDBBuilder;
        this.scrutariXmlErrorHandler = scrutariXmlErrorHandler;
    }
    
    public void read(Document doc) {
        BaseData.Builder baseDataBuilder = toBaseDataBuilder(doc.getDocumentElement());
        if (baseDataBuilder != null) {
            scrutariDBBuilder.addBase(baseDataBuilder);
            IndexationData.BuilderList igb =toIndexationDataBuilderList(doc.getDocumentElement(), baseDataBuilder.getBaseURI());
            scrutariDBBuilder.addIndexationBuilderList(igb);
        }
    }
    
    private IndexationData.BuilderList toIndexationDataBuilderList(Element element_xml,BaseURI baseURI) {
        IndexationData.BuilderList indexationDataBuilderList = new IndexationData.BuilderList();
        NodeList childList = element_xml.getChildNodes();
        for (int i = 0; i < childList.getLength(); i ++) {
            if (childList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) childList.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("indexation-group")) {
                    appendIndexationInfoBuffer(element,baseURI,indexationDataBuilderList);
                }
            }
        }
        return indexationDataBuilderList;
    }
    
    
    
    private BaseData.Builder toBaseDataBuilder(Element element_xml) {
        Element el_metadata = XMLUtils.getFirstElement(element_xml);
        if ((el_metadata == null) || (!el_metadata.getTagName().equals("base-metadata"))) {
            scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_BASE_MISSINGMETADATA);
            return null;
        }
        NodeList metadataList = el_metadata.getChildNodes();
        String authorityuuid=null;
        String baseName =null;
        for (int i = 0; i < metadataList.getLength(); i ++) {
            if (metadataList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) metadataList.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("authority-uuid")) {
                    authorityuuid = XMLUtils.getData(el);
                } else if (tagname.equals("base-name")) {
                    baseName = XMLUtils.getData(el);
                }
            }
        }
        if ((authorityuuid == null) || (baseName == null)) {
            if (authorityuuid == null) scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_AUTHORITYUUID);
            if (baseName == null) scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_NOMBASE);
            return null;
        }
        BaseURI baseURI = new BaseURI(authorityuuid,baseName);
        if (scrutariDBBuilder.containsBase(baseURI)) {
            scrutariXmlErrorHandler.addSevereLog(SEVERE_DATABASE_DOUBLE,baseURI);
            return null;
        }
        BaseData.Builder baseDataBuilder = new BaseData.Builder(baseURI);
        for (int i = 0; i < metadataList.getLength(); i ++) {
            if (metadataList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) metadataList.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("intitule-short")) {
                    addLibelles(el,BaseData.INTITULE_SHORT,baseDataBuilder);
                } else if (tagname.equals("intitule-long")) {
                    addLibelles(el,BaseData.INTITULE_LONG,baseDataBuilder);
                } else if (tagname.equals("langs-ui")) {
                    addLangsUi(el,baseDataBuilder);
                } else if (tagname.equals("base-icon")) {
                    String urlString = XMLUtils.getData(el);
                    baseDataBuilder.setBaseIcon(urlString);
                }
            }
        }
        NodeList childList = element_xml.getChildNodes();
        for (int i = 0; i < childList.getLength(); i ++) {
            if (childList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) childList.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("corpus")) {
                    CorpusData.Builder corpusDataBuilder = toCorpusDataBuilder(element,baseURI);
                    if (corpusDataBuilder != null) baseDataBuilder.addCorpusDataBuilder(corpusDataBuilder);
                } else if (tagname.equals("thesaurus")) {
                    ThesaurusData.Builder thesaurusDataBuilder = toThesaurusDataBuilder(element,baseURI);
                    if (thesaurusDataBuilder != null) baseDataBuilder.addThesaurusDataBuilder(thesaurusDataBuilder);
                }
            }
        }
        return baseDataBuilder;
    }
    
    
    private CorpusData.Builder toCorpusDataBuilder(Element element_xml, BaseURI baseURI) {
        String corpusName = element_xml.getAttribute("corpus-name");
        if (corpusName.length() == 0) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_CORPUS_MISSINGCORPUSNAME);
            return null;
        }
        CorpusURI corpusURI = new CorpusURI(baseURI, corpusName);
        if (uriMap.contains(corpusURI)) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_CORPUS_DOUBLECORPUSNAME,corpusURI);
            return null;
        }
        uriMap.add(corpusURI);
        CorpusData.Builder corpusDataBuilder = new CorpusData.Builder(corpusURI);
        Element el_metadata = XMLUtils.getFirstElement(element_xml);
        if ((el_metadata == null) || (!el_metadata.getTagName().equals("corpus-metadata"))) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_CORPUS_MISSINGMETADATA,corpusURI);
            return null;
        }
        NodeList metadataList = el_metadata.getChildNodes();
        for (int i = 0; i < metadataList.getLength(); i ++) {
            if (metadataList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) metadataList.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("intitule-corpus")) {
                    addLibelles(el, CorpusData.INTITULECORPUS_FIELDNUMBER, corpusDataBuilder);
                } else if (tagname.equals("intitule-fiche")) {
                    addLibelles(el, CorpusData.INTITULEFICHE_FIELDNUMBER, corpusDataBuilder);
                } else if (tagname.equals("complement-metadata")) {
                    int index = corpusDataBuilder.createNewComplement();
                    addLibelles(el, index+1, corpusDataBuilder);
                } else if (tagname.equals("href-parent")) {
                    corpusDataBuilder.setHrefParent(XMLUtils.getData(el));
                } else if (tagname.equals("type")) {
                    corpusDataBuilder.setDefaultType(getType(el,corpusURI));
                }
            }
        }
        int complementMaxNumber = corpusDataBuilder.getComplementMaxNumber();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("fiche")) {
                    FicheData.Builder ficheDataBuilder = toFicheDataBuilder(el,complementMaxNumber, corpusURI);
                    if (ficheDataBuilder != null) {
                        corpusDataBuilder.addFicheDataBuilder(ficheDataBuilder);
                    }
                }
            }
        }
        return corpusDataBuilder;
    }
    
    private FicheData.Builder toFicheDataBuilder(Element element_xml, int complementCount,CorpusURI corpusURI) {
        String ficheId = element_xml.getAttribute("fiche-id");
        if (ficheId.length() == 0) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_FICHE_MISSINGFICHEID);
            return null;
        }
        FicheURI ficheURI = new FicheURI(corpusURI, ficheId);
        if (uriMap.contains(ficheURI)) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_FICHE_DOUBLEFICHEID);
            return null;
        }
        uriMap.add(ficheURI);
        FicheData.Builder ficheDataBuilder = new FicheData.Builder(ficheURI, complementCount);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("titre")) {
                    ficheDataBuilder.setTitre(XMLUtils.getData(el));
                } else if (tagname.equals("soustitre")) {
                    ficheDataBuilder.setSoustitre(XMLUtils.getData(el));
                } else if (tagname.equals("date")) {
                    try {
                        String strg_date = XMLUtils.getData(el);
                        if ((strg_date != null) && (strg_date.length() != 0)) ficheDataBuilder.setDateInteger(IntDate.parse(strg_date));
                    } catch(IntDateException ide) {
                        scrutariXmlErrorHandler.addWarningLog(WARNING_XML_FICHE_DATEERROR,ficheURI);
                    }
                }else if (tagname.equals("type")) {
                    ficheDataBuilder.setType(getType(el,ficheURI));
                } else if (tagname.equals("lang")) {
                    try {
                        String strg_lang = XMLUtils.getData(el);
                        if ((strg_lang != null) && (strg_lang.length() != 0)) ficheDataBuilder.setLangInteger(LangInteger.parse(strg_lang));
                    } catch(LangIntegerException ile) {
                        scrutariXmlErrorHandler.addWarningLog(WARNING_XML_FICHE_LANGERROR,ficheURI);
                    }
                } else if (tagname.equals("href")) {
                    ficheDataBuilder.setHref(XMLUtils.getData(el));
                } else if (tagname.equals("complement")) {
                    boolean done = ficheDataBuilder.addComplement(XMLUtils.getData(el));
                    if (!done) {
                        scrutariXmlErrorHandler.addWarningLog(WARNING_XML_FICHE_TOMANYCOMPLEMENTS,ficheURI);
                    }
                }
            }
        }
        return ficheDataBuilder;
    }
    
    private ThesaurusData.Builder toThesaurusDataBuilder(Element element_xml, BaseURI baseURI) {
        String thesaurusName = element_xml.getAttribute("thesaurus-name");
        if (thesaurusName.length() == 0) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_THESAURUS_MISSINGTHESAURUSNAME);
            return null;
        }
        ThesaurusURI thesaurusURI = new ThesaurusURI(baseURI, thesaurusName);
        if (uriMap.contains(thesaurusURI)) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_THESAURUS_DOUBLETHESAURUSNAME,thesaurusURI);
            return null;
        }
        uriMap.add(thesaurusURI);
        ThesaurusData.Builder thesaurusDataBuilder = new ThesaurusData.Builder(thesaurusURI);
        Element el_metadata = XMLUtils.getFirstElement(element_xml);
        if ((el_metadata == null) || (!el_metadata.getTagName().equals("thesaurus-metadata"))) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_THESAURUS_MISSINGMETADATA,thesaurusURI);
            return null;
        }
        NodeList metadataList = el_metadata.getChildNodes();
        for (int i = 0; i < metadataList.getLength(); i ++) {
            if (metadataList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) metadataList.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("intitule-thesaurus")) {
                    addLibelles(el, thesaurusDataBuilder);
                }
            }
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) liste.item(i);
                String tagname = el.getTagName();
                if (tagname.equals("motcle")) {
                    MotcleData.Builder motcleDataBuilder = toMotcleDataBuilder(el, thesaurusURI);
                    if (motcleDataBuilder != null) {
                        thesaurusDataBuilder.addMotcleDataBuilder(motcleDataBuilder);
                    }
                }
            }
        }
        return thesaurusDataBuilder;
    }
    
    private MotcleData.Builder toMotcleDataBuilder(Element element_xml,ThesaurusURI thesaurusURI) {
        String motcleId = element_xml.getAttribute("motcle-id");
        if (motcleId.length() == 0) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_MOTCLE_MISSINGMOTCLEID);
            return null;
        }
        MotcleURI motcleURI = new MotcleURI(thesaurusURI, motcleId);
        if (uriMap.contains(motcleURI)) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_MOTCLE_DOUBLEMOTCLEID);
            return null;
        }
        uriMap.add(motcleURI);
        MotcleData.Builder motcleDataBuilder = new MotcleData.Builder(motcleURI);
        NodeList liblist = element_xml.getChildNodes();
        for (int j = 0;j < liblist.getLength();j++) {
            Node libnode = liblist.item(j);
            if (libnode.getNodeType() == Node.ELEMENT_NODE) {
                Element ellib = (Element) libnode;
                if (ellib.getTagName().equals("lib")) {
                    try {
                        Libelle libelle = TagUtils.getLibelle(ellib);
                        motcleDataBuilder.addLibelle(libelle);
                    } catch(LangIntegerException ile) {
                        scrutariXmlErrorHandler.addWarningLog(WARNING_XML_MOTCLE_LANGERROR,motcleURI);
                    }
                }
            }
        }
        return motcleDataBuilder;
    }
    
    
    
    private void addLibelles(Element el, int fieldNumber, CorpusData.Builder corpusDataBuilder) {
        NodeList liblist = el.getChildNodes();
        for (int j = 0;j < liblist.getLength();j++) {
            Node libnode = liblist.item(j);
            if (libnode.getNodeType() == Node.ELEMENT_NODE) {
                Element ellib = (Element) libnode;
                if (ellib.getTagName().equals("lib")) {
                    try {
                        Libelle libelle = TagUtils.getLibelle(ellib);
                        corpusDataBuilder.addLibelle(fieldNumber,libelle);
                    } catch(LangIntegerException ile) {
                        scrutariXmlErrorHandler.addWarningLog(WARNING_XML_CORPUS_LANGERROR,corpusDataBuilder.getCorpusURI());
                    }
                }
            }
        }
    }
    
    private void addLibelles(Element el, ThesaurusData.Builder thesaurusDataBuilder) {
        NodeList liblist = el.getChildNodes();
        for (int j = 0;j < liblist.getLength();j++) {
            Node libnode = liblist.item(j);
            if (libnode.getNodeType() == Node.ELEMENT_NODE) {
                Element ellib = (Element) libnode;
                if (ellib.getTagName().equals("lib")) {
                    try {
                        Libelle libelle = TagUtils.getLibelle(ellib);
                        thesaurusDataBuilder.addLibelle(libelle);
                    } catch(LangIntegerException ile) {
                        scrutariXmlErrorHandler.addWarningLog(WARNING_XML_THESAURUS_LANGERROR,thesaurusDataBuilder.getThesaurusURI());
                    }
                }
                
            }
        }
    }
    
    private short getType(Element el,ScrutariURI mapeadoresURI) {
        String typeString = XMLUtils.getData(el);
        try {
            return TypeConstants.typeToShort(typeString);
        } catch(Exception e) {
            String key = null;
            if (mapeadoresURI instanceof CorpusURI) key = WARNING_XML_CORPUS_TYPEERROR;
            else key = WARNING_XML_FICHE_TYPEERROR;
            scrutariXmlErrorHandler.addWarningLog(key,mapeadoresURI);
            return TypeConstants.UNKNOWN_TYPE;
        }
    }
    
    private void addLibelles(Element el, short type,BaseData.Builder baseDataBuilder) {
        NodeList liblist = el.getChildNodes();
        for (int j = 0;j < liblist.getLength();j++) {
            Node libnode = liblist.item(j);
            if (libnode.getNodeType() == Node.ELEMENT_NODE) {
                Element ellib = (Element) libnode;
                if (ellib.getTagName().equals("lib")) {
                    try {
                        Libelle libelle = TagUtils.getLibelle(ellib);
                        baseDataBuilder.addLibelle(type,libelle);
                    } catch(LangIntegerException ile) {
                        scrutariXmlErrorHandler.addWarningLog(WARNING_XML_BASE_LANGERROR,baseDataBuilder.getBaseURI());
                    }
                }
                
            }
        }
    }
    
    private void addLangsUi(Element el,BaseData.Builder baseDataBuilder) {
        NodeList liblist = el.getChildNodes();
        for (int j = 0;j < liblist.getLength();j++) {
            Node libnode = liblist.item(j);
            if (libnode.getNodeType() == Node.ELEMENT_NODE) {
                Element ellang = (Element) libnode;
                if (ellang.getTagName().equals("lang")) {
                    String langString = XMLUtils.getData(ellang);
                    try {
                        int langInteger = LangInteger.parse(langString);
                        baseDataBuilder.addLangUI(langInteger);
                    } catch(LangIntegerException ile) {
                        scrutariXmlErrorHandler.addWarningLog(WARNING_XML_BASE_LANGERROR,baseDataBuilder.getBaseURI());
                    }
                }
                
            }
        }
    }
    
    
    void appendIndexationInfoBuffer(Element element_xml,BaseURI baseURI,IndexationData.BuilderList indexationDataBuilderList) {
        String corpuspath = element_xml.getAttribute("corpus-path");
        if (corpuspath.length() == 0) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_INDEXATION_MISSINGCORPUSPATH);
            return;
        }
        String thesauruspath = element_xml.getAttribute("thesaurus-path");
        if (thesauruspath.length() == 0) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_INDEXATION_MISSINGTHESAURUSPATH);
            return;
        }
        CorpusURI corpusURI = new CorpusURI(baseURI, corpuspath);
        ThesaurusURI thesaurusURI = new ThesaurusURI(baseURI,thesauruspath);
        NodeList childList = element_xml.getChildNodes();
        for (int i = 0; i < childList.getLength(); i ++) {
            if (childList.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) childList.item(i);
                String tagname = element.getTagName();
                if (tagname.equals("indexation")) {
                    IndexationData.Builder indexationDataBuilder = toIndexationDataBuilder(element,corpusURI,thesaurusURI);
                    if (indexationDataBuilder != null) {
                        indexationDataBuilderList.addIndexationDataBuilder(indexationDataBuilder);
                    }
                }
            }
        }
    }
    
    IndexationData.Builder toIndexationDataBuilder(Element element_xml,CorpusURI corpusURI,ThesaurusURI thesaurusURI) {
        String ficheid = element_xml.getAttribute("fiche-id");
        if (ficheid.length() == 0) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_INDEXATION_MISSINGFICHEID);
            return null;
        }
        FicheURI ficheURI = new FicheURI(corpusURI, ficheid);
        if (!uriMap.contains(ficheURI)) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_DATABASE_INDEXATION_UNKNOWNFICHEURI,ficheURI);
        }
        String motcleid = element_xml.getAttribute("motcle-id");
        if (motcleid.length() == 0) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_INDEXATION_MISSINGMOTCLEID);
            return null;
        }
        
        MotcleURI motcleURI = new MotcleURI(thesaurusURI,motcleid);
        if (!uriMap.contains(motcleURI)) {
            scrutariXmlErrorHandler.addWarningLog(WARNING_DATABASE_INDEXATION_UNKNOWNMOTCLEURI,motcleURI);
        }
        IndexationKey indexationKey = new IndexationKey(ficheURI, motcleURI);
        if (!uriMap.contains(indexationKey)) {
            int poids = 1;
            String poidsString = element_xml.getAttribute("poids");
            if (poidsString.length() == 0) {
                poidsString = element_xml.getAttribute("weight");
            }
            try {
                poids = Integer.parseInt(poidsString);
                if (poids < 1) poids = 1;
            } catch(NumberFormatException nfe) {
                
            }
            IndexationData.Builder indexationInfoBuffer = new IndexationData.Builder(indexationKey, poids);
            uriMap.add(indexationKey);
            return indexationInfoBuffer;
        } else {
            scrutariXmlErrorHandler.addWarningLog(WARNING_XML_INDEXATION_DOUBLEINDEXATION);
            return null;
        }
    }
    
}