/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.xml;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.uri.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.logging.*;
import org.xmlpull.v1.*;
import java.io.*;
import net.mapeadores.util.xml.xmlpull.*;
import net.mapeadores.scrutari.xml.xmlpull.*;

/**
 *
 * @author Vincent Calame
 */
public class XmlPullReader implements ErrorKeys {
    private ScrutariDB.Builder scrutariDBBuilder;
    private ScrutariXmlErrorHandler scrutariXmlErrorHandler;
    private BaseData.Builder baseDataBuilder;
    private Parameters parameters;
    public static final int AUTHORITY_UUID = 1;
    public static final int BASE_NAME = 2;
    public static final int BASE_ICON = 3;
    public static final int LANG = 4;
    
    
    public XmlPullReader(ScrutariDB.Builder scrutariDBBuilder,ScrutariXmlErrorHandler scrutariXmlErrorHandler) {
        this.scrutariDBBuilder = scrutariDBBuilder;
        this.scrutariXmlErrorHandler = scrutariXmlErrorHandler;
        parameters = new Parameters(scrutariDBBuilder,scrutariXmlErrorHandler);
    
    }
    
    public BaseURI process(XmlPullParser xpp) throws XmlPullParserException, IOException {
        baseDataBuilder = null;
        RootHandler rootHandler = null;
        try {
            int eventType = xpp.getEventType();
            do {
                if(eventType == xpp.START_TAG) {
                    if (rootHandler == null) {
                        rootHandler = new RootHandler();
                    } else {
                        Map map = null;
                        int count = xpp.getAttributeCount();
                        if (count > 0) {
                            map = new HashMap();
                            for(int i = 0; i < count; i++) {
                                map.put(xpp.getAttributeName(i), xpp.getAttributeValue(i));
                            }
                        }
                        rootHandler.processStartElement(xpp.getName(),map);
                    }
                } else if(eventType == xpp.END_TAG) {
                    rootHandler.processEndElement(xpp.getName());
                } else if(eventType == xpp.TEXT) {
                    if (rootHandler != null) rootHandler.processText(xpp.getText());
                }
                eventType = xpp.next();
            } while (eventType != xpp.END_DOCUMENT);
            if (rootHandler != null) {
                BaseData.Builder builder = rootHandler.getBaseDataBuilder();
                scrutariDBBuilder.addBase(baseDataBuilder);
                scrutariDBBuilder.addIndexationBuilderList(parameters.getIndexationDataBuilderList());
                return baseDataBuilder.getBaseURI();
            }
            else {
                return null;
            }
        } catch(InterruptionException ie) {
            return null;
        }
    }
    
    private class RootHandler extends DefaultElementHandler {
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (baseDataBuilder == null) {
                if (!tagname.equals("base-metadata")) {
                    scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_BASE_MISSINGMETADATA);
                    throw new InterruptionException();
                }
                return new MetadataHandler();
            } else if (tagname.equals("corpus")) {
                try {
                    return new CorpusHandler(attributeMap, parameters);
                } catch(IllegalArgumentException iae) {
                    return null;
                }
                
            } else if (tagname.equals("thesaurus")) {
                try {
                    return new ThesaurusHandler(attributeMap, parameters);
                } catch(IllegalArgumentException iae) {
                    return null;
                }
            } else if (tagname.equals("indexation-group")) {
                try {
                    return new IndexationGroupHandler(attributeMap, parameters);
                } catch(IllegalArgumentException iae) {
                    return null;
                }
            
            } else {
                return null;
            }
        }
        
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof CorpusHandler) {
                CorpusData.Builder corpusDataBuilder = ((CorpusHandler) elementHandler).getCorpusDataBuilder();
                if (corpusDataBuilder != null) baseDataBuilder.addCorpusDataBuilder(corpusDataBuilder);
            }
            else if (elementHandler instanceof ThesaurusHandler) {
                ThesaurusData.Builder thesaurusDataBuilder = ((ThesaurusHandler)elementHandler).getThesaurusDataBuilder();
                if (thesaurusDataBuilder != null) baseDataBuilder.addThesaurusDataBuilder(thesaurusDataBuilder);
            }
            else if (elementHandler instanceof IndexationGroupHandler) {
                
            }
        }
        
        BaseData.Builder getBaseDataBuilder() {
            return baseDataBuilder;
        }
    }
    
    private class MetadataHandler extends DefaultElementHandler {
        String authorityuuid=null;
        String baseName =null;
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("authority-uuid")) {
                return new SimpleElementHandler(AUTHORITY_UUID);
            }
            if (tagname.equals("base-name")) {
                return new SimpleElementHandler(BASE_NAME);
            }
            initBaseDataBuilder();
            if (tagname.equals("base-icon")) {
                return new SimpleElementHandler(BASE_ICON);
            } else if (tagname.equals("intitule-short")) {
                return new IntituleHandler(BaseData.INTITULE_SHORT);
            } else if (tagname.equals("intitule-long")) {
                return new IntituleHandler(BaseData.INTITULE_LONG);
            } else if (tagname.equals("langs-ui")) {
                return new LangUiHandler();
            }
            return null;
        }
        
        public void endSimpleSubElement(int type, String text) {
            if (text.length() == 0) text = null;
            switch(type) {
                case AUTHORITY_UUID :
                    authorityuuid = text;
                    break;
                case BASE_NAME :
                    baseName = text;
                    break;
                case BASE_ICON :
                    baseDataBuilder.setBaseIcon(text);
                    break;
            }
        }
        
        private void initBaseDataBuilder() {
            if (baseDataBuilder != null) return;
            if ((authorityuuid == null) || (baseName == null)) {
                if (authorityuuid == null) scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_AUTHORITYUUID);
                if (baseName == null) scrutariXmlErrorHandler.addSevereLog(SEVERE_XML_NOMBASE);
                throw new InterruptionException();
            }
            BaseURI baseURI = new BaseURI(authorityuuid,baseName);
            if (scrutariDBBuilder.containsBase(baseURI)) {
                scrutariXmlErrorHandler.addSevereLog(SEVERE_DATABASE_DOUBLE,baseURI);
                throw new InterruptionException();
            }
            baseDataBuilder = new BaseData.Builder(baseURI);
            parameters.setBaseURI(baseURI);
        }
        
    }
    
    private class IntituleHandler extends DefaultElementHandler {
        short type;
        
        IntituleHandler(short type) {
            this.type = type;
        }
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch(LangIntegerException lie) {
                    scrutariXmlErrorHandler.addWarningLog(WARNING_XML_BASE_LANGERROR,baseDataBuilder.getBaseURI());
                    return null;
                }
            }
            return null;
        }
        
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof Libelle) {
                baseDataBuilder.addLibelle(type,(Libelle)elementHandler);
            }
        }
    }
    
    private class LangUiHandler extends DefaultElementHandler {
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("lang")) {
                return new SimpleElementHandler(LANG);
            }
            return null;
        }
        
        public void endSimpleSubElement(int type, String text) {
            try {
                int langInteger = LangInteger.parse(text);
                baseDataBuilder.addLangUI(langInteger);
            } catch(LangIntegerException ile) {
                scrutariXmlErrorHandler.addWarningLog(WARNING_XML_BASE_LANGERROR,baseDataBuilder.getBaseURI());
            }
        }
        
    }

    
    private static class InterruptionException extends RuntimeException {
        
    }
    
}
