/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.xml.xmlpull;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.uri.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.logging.*;
import org.xmlpull.v1.*;
import java.io.*;
import net.mapeadores.util.xml.xmlpull.*;
import net.mapeadores.scrutari.xml.*;
import net.mapeadores.scrutari.uri.*;

/**
 *
 * @author Vincent Calame
 */
public class CorpusHandler extends DefaultElementHandler implements ErrorKeys {
    private static final int HREF_PARENT = 1;
    private static final int TYPE = 2;
    private static final int TITRE = 3;
    private static final int SOUSTITRE = 4;
    private static final int DATE = 5;
    private static final int LANG = 6;
    private static final int HREF = 7;
    private static final int COMPLEMENT = 8;
    private static final int FICHE_ICON = 9;
    private Parameters parameters;
    private boolean metadataDone = false;
    private CorpusURI corpusURI;
    private CorpusData.Builder corpusDataBuilder;
    private FicheHandler ficheHandler = new FicheHandler();
    
    public CorpusHandler(Map attributeMap, Parameters parameters) {
        String corpusName = getAttributeString(attributeMap,"corpus-name");
        if (corpusName == null) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_CORPUS_MISSINGCORPUSNAME);
            throw new IllegalArgumentException();
        }
        this.parameters = parameters;
        corpusURI = new CorpusURI(parameters.getBaseURI(), corpusName);
        if (!parameters.testDoublon(corpusURI)) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_CORPUS_DOUBLECORPUSNAME,corpusURI);
            throw new IllegalArgumentException();
        }
        corpusURI = (CorpusURI) parameters.intern(corpusURI);
        corpusDataBuilder = new CorpusData.Builder(corpusURI);
    }
    
    public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
        if (!metadataDone) {
            if (tagname.equals("corpus-metadata")) {
                metadataDone = true;
                return new MetadataHandler();
            } else {
                parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_CORPUS_MISSINGMETADATA,corpusURI);
                return null;
            }
        } else {
            if (tagname.equals("fiche")) {
                String ficheId = getAttributeString(attributeMap,"fiche-id");
                if (ficheId == null) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_MISSINGFICHEID);
                    return null;
                }
                FicheURI ficheURI = new FicheURI(corpusURI, ficheId);
                if (!parameters.testDoublon(ficheURI)) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_DOUBLEFICHEID,ficheURI);
                    return null;
                }
                ficheHandler.setFicheURI(ficheURI);
                return ficheHandler;
            } else {
                return null;
            }
        }
    }
    
    public void endSubElement(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof FicheHandler) {
            FicheData.Builder ficheDataBuilder = ((FicheHandler) elementHandler).getFicheDataBuilder();
            corpusDataBuilder.addFicheDataBuilder(ficheDataBuilder);
        }
    }
    
    public CorpusData.Builder getCorpusDataBuilder() {
        if (!metadataDone) return null;
        return corpusDataBuilder;
    }
    
    private class MetadataHandler extends DefaultElementHandler {
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("intitule-corpus")) {
                return new IntituleHandler(CorpusData.INTITULECORPUS_FIELDNUMBER);
            } else if (tagname.equals("intitule-fiche")) {
                return new IntituleHandler(CorpusData.INTITULEFICHE_FIELDNUMBER);
            } else if (tagname.equals("complement-metadata")) {
                int index = corpusDataBuilder.createNewComplement();
                return new IntituleHandler(index+1);
            } else if (tagname.equals("href-parent")) {
                return new SimpleElementHandler(HREF_PARENT);
            } else if (tagname.equals("type")) {
                return new SimpleElementHandler(TYPE);
            } else {
                return null;
            }
        }
        
        public void endSimpleSubElement(int type, String text) {
            switch(type) {
                case HREF_PARENT :
                    corpusDataBuilder.setHrefParent(text);
                    break;
                case TYPE :
                    corpusDataBuilder.setDefaultType(getType(text, corpusURI));
                    break;
            }
        }
    }
    
    private class FicheHandler extends DefaultElementHandler {
        FicheURI ficheURI;
        private FicheData.Builder ficheDataBuilder;
        
        FicheHandler() {
            
        }
        
        void setFicheURI(FicheURI ficheURI) {
            this.ficheURI = ficheURI;
            ficheDataBuilder = new FicheData.Builder(ficheURI, corpusDataBuilder.getComplementMaxNumber());
        }
        
        FicheData.Builder getFicheDataBuilder() {
            return ficheDataBuilder;
        }
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("titre")) {
                return new SimpleElementHandler(TITRE);
            } else if (tagname.equals("soustitre")) {
                return new SimpleElementHandler(SOUSTITRE);
            } else if (tagname.equals("date")) {
                return new SimpleElementHandler(DATE);
            } else if (tagname.equals("lang")) {
                return new SimpleElementHandler(LANG);
            } else if (tagname.equals("href")) {
                return new SimpleElementHandler(HREF);
            } else if (tagname.equals("type")) {
                return new SimpleElementHandler(TYPE);
            } else if (tagname.equals("complement")) {
                return new SimpleElementHandler(COMPLEMENT);
            } else if (tagname.equals("fiche-icon")) {
                return new SimpleElementHandler(FICHE_ICON);
            } else {
                return null;
            }
        }
        
        
        public void endSimpleSubElement(int type, String text) {
            switch(type) {
                case TITRE :
                    ficheDataBuilder.setTitre(text);
                    break;
                case SOUSTITRE :
                    ficheDataBuilder.setSoustitre(text);
                    break;
                case DATE:
                    try {
                        if ((text != null) && (text.length() != 0)) ficheDataBuilder.setDateInteger(IntDate.parse(text));
                    } catch(IntDateException ide) {
                        parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_DATEERROR,ficheURI);
                    }
                    break;
                case LANG:
                    try {
                        if ((text != null) && (text.length() != 0)) ficheDataBuilder.setLangInteger(LangInteger.parse(text));
                    } catch(LangIntegerException ile) {
                        parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_LANGERROR,ficheURI);
                    }
                    break;
                case HREF:
                    ficheDataBuilder.setHref(text);
                    break;
                case TYPE:
                    ficheDataBuilder.setType(getType(text, ficheURI));
                    break;
                case COMPLEMENT:
                    boolean done = ficheDataBuilder.addComplement(text);
                    if (!done) {
                        parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_TOMANYCOMPLEMENTS,ficheURI);
                    }
                    break;
                case FICHE_ICON :
                    ficheDataBuilder.setFicheIcon(text);
                    break;
            }
        }
        
    }
    
    private short getType(String text,ScrutariURI mapeadoresURI) {
        try {
            return TypeConstants.typeToShort(text);
        } catch(Exception e) {
            String key = null;
            if (mapeadoresURI instanceof CorpusURI) key = WARNING_XML_CORPUS_TYPEERROR;
            else key = WARNING_XML_FICHE_TYPEERROR;
            parameters.getScrutariXmlErrorHandler().addWarningLog(key,mapeadoresURI);
            return TypeConstants.UNKNOWN_TYPE;
        }
    }
    
    private class IntituleHandler extends DefaultElementHandler {
        int fieldNumber;
        
        IntituleHandler(int fieldNumber) {
            this.fieldNumber = fieldNumber;
        }
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch(LangIntegerException lie) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_CORPUS_LANGERROR,corpusURI);
                    return null;
                }
            }
            return null;
        }
        
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof Libelle) {
                corpusDataBuilder.addLibelle(fieldNumber,(Libelle)elementHandler);
            }
        }
    }
    
}
