/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver;

import fr.exemole.sctserver.admin.*;
import fr.exemole.sctserver.conf.*;
import fr.exemole.sctserver.request.RequestInfo;
import fr.exemole.sctserver.storage.*;
import java.io.*;
import net.mapeadores.scrutari.searchengine.beans.SearchResultBean;
import net.mapeadores.scrutari.searchengine.*;
import net.mapeadores.scrutari.uri.*;
import org.w3c.dom.*;
import java.util.*;
import java.util.logging.*;
import javax.xml.parsers.*;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;
import java.text.*;
import com.sleepycat.je.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.scrutari.xml.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.io.*;
import java.net.*;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;
import net.mapeadores.scrutari.supermotcle.*;
import fr.exemole.sctserver.searchlog.*;

/**
 *
 * @author Vincent Calame
 */
public class SctServer implements LogConstants, ConfConstants {
    public final static String MALFORMED_XML_CATEGORY = "MALFORMED_XML";
    public final static String SEVERE_CATEGORY = "SEVERE";
    public final static String WARNING_CATEGORY = "WARNING";
    private final static short INIT_TYPE = 1;
    private final static short FORCE_TYPE = 2;
    private final static short CRON_TYPE = 3;
    private final static short UPDATECONF_TYPE = 4;
    private final static DateFormat dateformattotal = new SimpleDateFormat("yyyyMMdd");
    private final static DateFormat isoformat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
    private SctServerStorage sctServerStorage;
    private SourceInfoManager sourceInfoManager;
    private ScrutariDB scrutariDB;
    private SupermotcleManager supermotcleManager;
    private int idrequete = 1;
    private int lastDateInteger = IntDate.current();
    private Timer timer;
    private OptionsManager optionsManager;
    private boolean hasUpdateProcessLog = false;
    private SearchLog searchLog;
    private InternalConfState confState = new InternalConfState();

    public SctServer(SctServerStorage sctServerStorage) {
        this.sctServerStorage = sctServerStorage;
        this.supermotcleManager = new SupermotcleManager();
        this.searchLog = sctServerStorage.getSearchLogAccess().getSearchLog();
        this.sourceInfoManager = new SourceInfoManager();
        this.optionsManager = new OptionsManager(this);
        Reloader reloader = new Reloader(INIT_TYPE );
        reloader.reload();
        this.timer = new Timer(true);
        timer.scheduleAtFixedRate(new UpdateTask(), getFirstTime(),getPeriod());
    }
    
    public boolean hasUpdateProcessLog() {
        return hasUpdateProcessLog;
    }
    
    public void reload(boolean forceDataDownload) {
        short type = (forceDataDownload)?FORCE_TYPE:INIT_TYPE;
        Reloader reloader = new Reloader(type);
        reloader.reload();
        sctServerStorage.cleanSearchResults();
    }
    
    
    
    public ScrutariDB getScrutariDB(){
        return scrutariDB;
    }
    
    public SourceInfoManager getScrutariInfoManager() {
        return sourceInfoManager;
    }
    
    public OptionsManager getOptionsManager() {
        return optionsManager;
    }
    
    public SupermotcleManager getSupermotcleManager() {
        synchronized(supermotcleManager) {
            if (!supermotcleManager.isInit()) {
                supermotcleManager.update(scrutariDB.getDataManager());
            }
        }
        return supermotcleManager;
    }
    
    public SearchResult getSearchResult(String identifiant) {
        return sctServerStorage.getSearchResultBean(identifiant);
    }
    
    public LogAccess getLogAccess() {
        return sctServerStorage.getLogAccess();
    }
    
    public SearchLog getSearchLog() {
        return searchLog;
    }
    
    public SearchResult newSearch(SearchSequence searchSequence,SearchOptions searchOptions,RequestInfo requestInfo) {
        String identifiant = getNewIdentifiant(searchSequence,searchOptions,requestInfo);
        SearchEngineParameters searchEngineParameters = new SearchEngineParameters();
        searchEngineParameters.setScrutariDB(scrutariDB);
        searchEngineParameters.setIdentifiant(identifiant);
        searchEngineParameters.setSearchOptions(searchOptions);
        searchEngineParameters.setSearchSequence(searchSequence);
        searchEngineParameters.setGlobalSearchOptions(optionsManager.getGlobalSearchOptions());
        SearchResult bean = SearchEngine.search(searchEngineParameters);
        sctServerStorage.saveSearchResult((SearchResultBean) bean);
        logSearch(bean,requestInfo);
        return bean;
    }
    
    private synchronized void logSearch(SearchResult searchResult, RequestInfo requestInfo) {
        if (!optionsManager.filterLog(requestInfo)) return;
        MutableQueryLog queryLog = new MutableQueryLog();
        queryLog.setSearchSequence(searchResult.getSearchSequence().toString());
        queryLog.setRemoteAddress(requestInfo.getRemoteAddress());
        queryLog.setReferrer(requestInfo.getReferrer());
        queryLog.setSite(requestInfo.getSite());
        queryLog.setFicheCount(searchResult.getFicheCount());
        queryLog.setAllMotcleCount(searchResult.getAllMotcleCount());
        queryLog.setCompleteMotcleCount(searchResult.getCompleteMotcleCount());
        SearchOptions searchOptions = searchResult.getSearchOptions();
        if (searchOptions.hasFilter()) {
            queryLog.setFicheMaximum(searchResult.getFicheMaximum());
        }
        sctServerStorage.getSearchLogAccess().log(this,queryLog);
    }
    
    public void updateConf() {
        Reloader reloader = new Reloader(UPDATECONF_TYPE);
        reloader.reload();
    }
    
    public ConfState getConfState() {
        return confState;
    }

    
    private boolean checkSourceInfo(UpdateProcessLog updateProcessLog,boolean forceDataDownload) {
        boolean dataChange = false;
        UrlCollector urlCollector = new UrlCollector(sctServerStorage,updateProcessLog);
        int count = getScrutariInfoManager().getSourceCount();
        for(int i = 0; i  < count;i++) {
            ScrutariInfo scrutariInfo = getScrutariInfoManager().getSourceInfo(i);
            urlCollector.setScrutariInfo(scrutariInfo);
            urlCollector.collectData(forceDataDownload);
            short newState = urlCollector.getFinalState();
            int update = urlCollector.getLastUpdate();
            if (ScrutariInfoUtils.hasChange(scrutariInfo, newState, update)) {
                dataChange = true;
            }
            sourceInfoManager.setState(scrutariInfo.getKeyString(),newState);
            sourceInfoManager.setScrutariDataUrl(scrutariInfo.getKeyString(), urlCollector.getDataURL());
            sourceInfoManager.setLastUpdate(scrutariInfo.getKeyString(),update);
        }
        if (forceDataDownload) return true;
        else return dataChange;
    }
    
    private void saveUpdateProcessLog(UpdateProcessLog updateProcessLog) {
        hasUpdateProcessLog = saveProcessLog(updateProcessLog, "updateprocess");
    }
    
    private boolean saveProcessLog(ProcessLog processLog, String name) {
        boolean hasLog = processLog.hasLog();
        try {
            OutputStream outputStream = sctServerStorage.getLogAccess().getLogOutputStream("log_" + name + ".txt");
            Writer writer = new OutputStreamWriter(outputStream, "UTF-8");
            writer.write(ProcessLogUtils.toString(processLog));
            writer.close();
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
        return hasLog;
    }
    
    public String getLog(String name) {
        try {
            return IOUtils.toString(sctServerStorage.getLogAccess().getLogInputStream("log_" + name + ".txt"));
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }
    
    
    private void reinitScrutariDB(UpdateProcessLog updateProcessLog) {
        boolean logStarted = false;
        try {
            if (!TimeLog.isStarted()) {
                logStarted = true;
                TimeLog.startLog(sctServerStorage.getLogAccess().getLogOutputStream("timelog.txt"),"SctServer.reinitScrutariDB()","Début de la réinitialisation");
                //TimeLog.startLog(new File("/home/vic/tomcat/log2.txt"),"SctServer.reinitScrutariDB()","Début de la réinitialisation");
            } else {
                TimeLog.log("SctServer.reinitScrutariDB()","Début de la réinitialisation");
            }
        } catch(IOException ioe) {
            throw new NestedIOException(ioe);
        }
        ScrutariDB newScrutariDB = newScrutariDB(updateProcessLog);
        ScrutariDB oldScrutariDB = scrutariDB;
        scrutariDB = newScrutariDB;
        supermotcleManager.clear();
        if (oldScrutariDB != null) {
            TimeLog.log("SctServer.reinitScrutariDB()","Début de la suppression de la base existante");
            String name = oldScrutariDB.getName();
            oldScrutariDB.close();
            sctServerStorage.deleteBerkelyDirectory(name);
            TimeLog.log("SctServer.reinitScrutariDB()","Fin de la suppression de la base existante");
        }
        if (logStarted) {
            try {
                TimeLog.endLog("SctServer.reinitScrutariDB()","Fin de la réinitialisation");
            } catch(IOException ioe) {
                throw new NestedIOException(ioe);
            }
        } else {
            TimeLog.log("SctServer.reinitScrutariDB()","Fin de la réinitialisation");
        }
    }
    
    private ScrutariDB newScrutariDB(UpdateProcessLog updateProcessLog) {
        TimeLog.log("SctServer.newScrutariDB()","Début de la création de la nouvelle base");
        try {
            Date current = new Date();
            String name = "scrutaridb_" + isoformat.format(current);
            EnvironmentConfig firstEnvConfig = new EnvironmentConfig();
            firstEnvConfig.setReadOnly(false);
            firstEnvConfig.setCacheSize(8 * 1024 * 1024);
            firstEnvConfig.setAllowCreate(true);
            Environment  environment = new Environment(sctServerStorage.getBerkeleyDirectory(name), firstEnvConfig);
            ScrutariDB.Builder builder = ScrutariDB.newBuilder(name,environment);
            TimeLog.log("SctServer.newScrutariDB()","Fin de la création / début du remplissage");
            //buildWithDOM(builder, updateProcessLog);
            builWithXmlPull(builder, updateProcessLog);
            TimeLog.log("SctServer.newScrutariDB()","Début de la conversion finale");
            ScrutariDB newScrutariDB = builder.toScrutariDB();
            TimeLog.log("SctServer.newScrutariDB()","Fin de la conversion / début des stats");
            newScrutariDB.updateStats();
            TimeLog.log("SctServer.newScrutariDB()","Fin des stats / base finie");
            return newScrutariDB;
        } catch(com.sleepycat.je.DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
    private void buildWithDOM(ScrutariDB.Builder builder,UpdateProcessLog updateProcessLog) {
        XMLReader xmlReader = new XMLReader(builder,updateProcessLog);
        int count = sourceInfoManager.getSourceCount();
        for(int i = 0; i  < count;i++) {
            ScrutariInfo scrutariInfo = sourceInfoManager.getSourceInfo(i);
            updateProcessLog.setCurrentURL(scrutariInfo.getUrl());
            Document doc = getScrutariDataDocument(scrutariInfo,updateProcessLog);
            if (doc != null) {
                xmlReader.read(doc);
            }
        }
    }
    
    private void builWithXmlPull(ScrutariDB.Builder builder,UpdateProcessLog updateProcessLog) {
        XmlPullReader xmlPullReader = new XmlPullReader(builder,updateProcessLog);
        int count = sourceInfoManager.getSourceCount();
        for(int i = 0; i  < count;i++) {
            ScrutariInfo scrutariInfo = sourceInfoManager.getSourceInfo(i);
            URL url = scrutariInfo.getUrl();
            String urlString = url.toString();
            updateProcessLog.setCurrentURL(url);
            TimeLog.log(urlString,"Début du traitement des données");
            parseXmlPull(xmlPullReader, scrutariInfo, updateProcessLog);
            TimeLog.log(urlString,"Fin du traitement des données");
        }
    }
    
    private void parseXmlPull(XmlPullReader xmlPullReader,ScrutariInfo scrutariInfo,UpdateProcessLog updateProcessLog) {
        InputStream inputStream = sctServerStorage.getScrutariData(scrutariInfo);
        TimeLog.log("SctServer.parseXmlPull()","Récupération des données faites / début de la lecture");
        if (inputStream == null) {
            return;
        }
        XmlPullParser xpp = null;
        try {
            XmlPullParserFactory factory = XmlPullParserFactory.newInstance(System.getProperty(XmlPullParserFactory.PROPERTY_NAME), null);
            xpp = factory.newPullParser();
        } catch(XmlPullParserException xppe) {
            throw new NestedLibraryException(xppe);
        }
        try {
            try {
                xpp.setInput(inputStream, null);
                BaseURI baseURI = xmlPullReader.process(xpp);
                sourceInfoManager.setBaseURI(scrutariInfo.getKeyString(), baseURI);
                
            } catch(XmlPullParserException xppe) {
                updateProcessLog.xmlPullParserException(xppe);
            } finally {
                inputStream.close();
            }
        } catch(IOException ioe) {
            throw new NestedIOException(ioe);
        }
        
    }
    
    private Document getScrutariDataDocument(ScrutariInfo scrutariInfo,UpdateProcessLog updateProcessLog) {
        if (scrutariInfo.getState() != ScrutariInfo.OK_STATE) return null;
        InputStream inputStream = sctServerStorage.getScrutariData(scrutariInfo);
        if (inputStream == null) {
            return null;
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch(ParserConfigurationException e) {
            throw new NestedLibraryException(e);
        }
        Document doc = null;
        try {
            doc = db.parse(inputStream);
            inputStream.close();
        } catch(IOException ioe) {
            throw new NestedIOException(ioe);
        } catch(SAXException saxe) {
            sourceInfoManager.setState(scrutariInfo.getKeyString(), ScrutariInfo.DATA_NOTXML_STATE);
            updateProcessLog.saxException(saxe);
            return null;
        }
        return doc;
    }
    
    
    
    
    private Date getFirstTime() {
        Calendar calendar = Calendar.getInstance();
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        calendar.set(Calendar.DAY_OF_MONTH,day+1);
        calendar.set(Calendar.HOUR_OF_DAY,(int) (4*Math.random()));
        calendar.set(Calendar.MINUTE, (int) (60*Math.random()));
        return calendar.getTime();
    }
    
    private long getPeriod() {
        return 24 * 60 * 60 * 1000;
    }
    
    
    private synchronized String getNewIdentifiant(SearchSequence sequence, SearchOptions options,RequestInfo requestInfo) {
        Date current = new Date();
        int currentDate = IntDate.fromDate(current);
        if (currentDate != lastDateInteger) {
            idrequete = 1;
            lastDateInteger = currentDate;
        }
        String identifiant = dateformattotal.format(current) + "-" + String.valueOf(idrequete);
        idrequete++;
        return identifiant;
    }
    
    private class Reloader {
        private short type;
        private ConfDirectory confDirectory;
        
        
        Reloader(short type) {
            this.type = type;
        }
        
        public void reload() {
            initConfDirectory();
            sourceInfoManager.reloadInfoUrlArray(confDirectory.getInfoUrlKeyArray());
            optionsManager.reloadSearchLogDenyArray(confDirectory.getSearchLogDenyArray());
            if (type != UPDATECONF_TYPE) {
                UpdateProcessLog updateProcessLog = new UpdateProcessLog();
                boolean forceReload = (type == FORCE_TYPE);
                boolean done = checkSourceInfo(updateProcessLog,forceReload);
                if ((type != CRON_TYPE) || (done)) {
                    reinitScrutariDB(updateProcessLog);
                    saveUpdateProcessLog(updateProcessLog);
                    sctServerStorage.cleanSearchResults();
                }
            }
            optionsManager.reloadCorpusClasseArray(confDirectory.getCorpusClasseArray());
        }
        
        private void initConfDirectory() {
            confDirectory = sctServerStorage.getConfDirectory();
            try {
                confDirectory.reload();
                checkProcessLog(SOURCES_NAME);
                checkProcessLog(CORPUS_CLASSES_NAME);
                checkProcessLog(SEARCHLOG_FILTER_NAME);
                confState.setOkState();
            } catch(ConfIOException cioe) {
                confState.setGlobalConfigException(cioe);
            }
        }
        
        private void checkProcessLog(String name) {
            ConfFileInfo confFileInfo = confDirectory.getConfFileInfo(name);
            if (!confFileInfo.isHere()) {
                confState.setFileState(name, FILE_MISSING);
            } else {
                ProcessLog processLog = confFileInfo.getProcessLog();
                String processURIString = confFileInfo.getProcessURI().getProcessURIString();
                if (processLog.hasLog()) {
                    int type = 0;
                    List liste = processLog.getProcessLogItemList(processURIString);
                    
                    for(Iterator it = liste.iterator(); it.hasNext();) {
                        ProcessLogItem logItem = (ProcessLogItem) it.next();
                        String category = logItem.getCategory();
                        if (category.equals(MALFORMED_XML_CATEGORY)) {
                            type = 2;
                            break;
                        } else if (category.equals(SEVERE_CATEGORY)) {
                            type = 1;
                        }
                    }
                    if (type == 2) {
                        confState.setFileState(name, FILE_XML_MALFORMED);
                    } else if (type == 1) {
                        confState.setFileState(name, FILE_XML_ERROR);
                    } else {
                        confState.setFileState(name, FILE_XML_WARNING);
                    }
                } else {
                    confState.setFileState(name, FILE_OK);
                }
                saveProcessLog(processLog, name);
            }
            
        }
        
    }
    
    
    private class UpdateTask extends TimerTask {
        
        public void run() {
            Reloader reloader = new Reloader(CRON_TYPE);
            reloader.reload();
        }
        
    }
    
    private class UpdateProcessLog extends MutableProcessLog implements ScrutariXmlErrorHandler, UrlCollector.CollectErrorHandler  {
        
        public void setCurrentURL(URL url) {
            setCurrentURI(ProcessURI.fromURL(url));
        }
        
        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }
        
        public void xmlPullParserException(XmlPullParserException xppe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(xppe.getMessage());
        }
        
        public void addSevereLog(String key, ScrutariURI mapeadoresURI) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SEVERE_CATEGORY);
            logItem.setKey(key);
            logItem.setObject(mapeadoresURI);
        }
        
        public void addWarningLog(String key, ScrutariURI mapeadoresURI) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(WARNING_CATEGORY);
            logItem.setKey(key);
            logItem.setObject(mapeadoresURI);
        }
        
        public void addSevereLog(String key) {
            addSevereLog(key,null);
        }
        
        public void addWarningLog(String key) {
            addWarningLog(key, null);
        }
        
        public void warningDownload(URL url, String message) {
            setCurrentURL(url);
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(WARNING_CATEGORY);
            logItem.setKey("warning.download");
            logItem.setObject(message);
        }
        
    }
    
    private class InternalConfState implements ConfState {
        private short state = GLOBAL_UNDEFINED;
        private String confIOMessage = null;
        private Map stateMap = new HashMap();
        
        private void setGlobalConfigException(ConfIOException ioe) {
            state = GLOBAL_CONFIO_ERROR;
            confIOMessage = ioe.getFullMessage();
        }
        private void clear() {
            this.confIOMessage = null;
            this.state = GLOBAL_UNDEFINED;
            stateMap.clear();
        }
        
        public String getConfIOMessage() {
            return confIOMessage;
        }
        
        public short getGlobalState() {
            return state;
        }
        
        void setOkState() {
            this.state = GLOBAL_OK;
        }
        
        void setFileState(String name, short fileState) {
            stateMap.put(name, new Short(fileState));
        }
        
        public short getFileState(String name) {
            Short sh = (Short) stateMap.get(name);
            if (sh == null) return FILE_STATE_UNKNOWN;
            return sh.shortValue();
        }
        
    }
    
}
