/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.admin;

import fr.exemole.sctserver.storage.*;
import java.util.logging.*;
import java.net.*;
import java.io.*;
import net.mapeadores.util.exceptions.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import net.mapeadores.util.xml.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.exceptions.*;
import javax.net.ssl.*;
import java.security.cert.X509Certificate;

/**
 *
 * @author Vincent Calame
 */
public class UrlCollector {
    private SctServerStorage sctServerStorage;
    private ScrutariInfo scrutariInfo;
    private short finalState;
    private CollectErrorHandler collectErrorHandler;
    private int lastUpdate;
    private URL dataUrl;
    
    
    public UrlCollector(SctServerStorage sctServerStorage, CollectErrorHandler collectErrorHandler) {
        this.sctServerStorage = sctServerStorage;
        this.collectErrorHandler = collectErrorHandler;
        try {
            SSLContext sc = SSLContext.getInstance("SSLv3");
            TrustManager[] tma = { new InternalTrustManager() };
            sc.init(null, tma, null);
            SSLSocketFactory ssf = sc.getSocketFactory();
            HttpsURLConnection.setDefaultSSLSocketFactory(ssf);
        }
        catch(Exception e) {
            throw new NestedLibraryException(e);
        }
        
    }
    
    public void setScrutariInfo(ScrutariInfo scrutariInfo) {
        this.scrutariInfo = scrutariInfo;
        dataUrl = null;
        finalState = ScrutariInfo.UNKNOWN_STATE;
        lastUpdate = scrutariInfo.getLastUpdate();
    }
    
    public short getFinalState() {
        return finalState;
    }
    
    public int getLastUpdate() {
        return lastUpdate;
    }
    
    public URL getDataURL() {
        return dataUrl;
    }
    
    public boolean collectData(boolean forceDataDownload) {
        boolean done = checkInfo();
        if (!done) return false;
        if (!forceDataDownload) {
            if (lastUpdate <= scrutariInfo.getLastUpdate()) {
                finalState = ScrutariInfo.OK_STATE;
                lastUpdate = scrutariInfo.getLastUpdate();
                return true;
            }
        }
        URLConnection con = null;
        try {
            con=  dataUrl.openConnection();
            if (!testConnection(con)) {
                finalState = ScrutariInfo.DATA_BROKENURL_STATE;
                return false;
            }
        } catch(IOException ioe) {
            finalState = ScrutariInfo.DATA_NOCONNECTION_STATE;
            collectErrorHandler.warningDownload(dataUrl,ioe.getMessage());
            return false;
        }
        
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            InputStream in= con.getInputStream();
            byte[] buffer= new byte[1024];
            int size;
            while( (size= in.read(buffer, 0, 1024)) > 0 ){
                out.write(buffer, 0, size);
            }
            in.close();
            out.flush();
            out.close();
        } catch(IOException ioe) {
            finalState = ScrutariInfo.DATA_DOWNLOADERROR_STATE;
            collectErrorHandler.warningDownload(dataUrl,ioe.getMessage());
            return false;
        }
        ByteArrayInputStream byteArrayInputStream = new  ByteArrayInputStream(out.toByteArray());
        sctServerStorage.saveScrutariData(scrutariInfo,byteArrayInputStream);
        finalState = ScrutariInfo.OK_STATE;
        return true;
    }
    
    private boolean testConnection(URLConnection urlConnection) throws IOException {
        URL url = urlConnection.getURL();
        if (url.getProtocol().equals("https")) {
            try {
            HttpsURLConnection httpsCon = (HttpsURLConnection) urlConnection;
            int reponse = httpsCon.getResponseCode();
            if (reponse != HttpURLConnection.HTTP_OK) {
                collectErrorHandler.warningDownload(dataUrl,httpsCon.getResponseMessage());
                finalState = ScrutariInfo.DATA_BROKENURL_STATE;
                return false;
            }
            else {
                return true;
            }
            }
            catch(Exception e) {
                e.printStackTrace();
                return false;
            }
            
        }
        else {
            HttpURLConnection con = (HttpURLConnection) urlConnection;
            int reponse = con.getResponseCode();
            if (reponse != HttpURLConnection.HTTP_OK) {
                collectErrorHandler.warningDownload(dataUrl,con.getResponseMessage());
                finalState = ScrutariInfo.DATA_BROKENURL_STATE;
                return false;
            }
            else {
                return true;
            }
        }
    }
    
    
    private boolean checkInfo() {
        URL url = scrutariInfo.getUrl();
        URLConnection con = null;
        try {
            con= (HttpURLConnection) url.openConnection();
            if (!testConnection(con)) {
                finalState = ScrutariInfo.INFO_BROKENURL_STATE;
                return false;
            }
        } catch(IOException ioe) {
            finalState = ScrutariInfo.INFO_NOCONNECTION_STATE;
            collectErrorHandler.warningDownload(url,ioe.getMessage());
            return false;
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch(ParserConfigurationException e) {
            throw new NestedLibraryException(e);
        }
        Document doc = null;
        try {
            doc = db.parse(con.getInputStream());
        } catch(IOException ioe) {
            finalState = ScrutariInfo.INFO_DOWNLOADERROR_STATE;
            collectErrorHandler.warningDownload(url,ioe.getMessage());
            return false;
        } catch(SAXException saxe) {
            finalState = ScrutariInfo.INFO_NOTXML_STATE;
            collectErrorHandler.warningDownload(url,saxe.getMessage());
            return false;
        }
        return checkInfoDocument(doc.getDocumentElement());
    }
    
    private boolean checkInfoDocument(Element element_xml) {
        String pathString = null;
        String lastDateString = null;
        NodeList nodeList = element_xml.getChildNodes();
        int completeMask = 0;
        for(int i = 0; i < nodeList.getLength();i++) {
            Node node = nodeList.item(i);
            if (node.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) node;
                String tagname = el.getTagName();
                if (tagname.equals("date")) {
                    lastDateString = XMLUtils.getData(el);
                } else if (tagname.equals("path")) {
                    pathString = XMLUtils.getData(el);
                } else if (tagname.equals("scrutaridata-url")) {
                    pathString = XMLUtils.getData(el);
                }
            }
        }
        if ((pathString == null) || (lastDateString == null)) {
            finalState = ScrutariInfo.INFO_WRONGXML_STATE;
            return false;
        }
        try {
            lastUpdate = IntDate.parse(lastDateString);
        } catch(IntDateException ide) {
            finalState = ScrutariInfo.INFO_WRONGXML_STATE;
            return false;
        }
        try {
            dataUrl = buildDataUrl(pathString);
        } catch(MalformedURLException mue) {
            finalState = ScrutariInfo.INFO_WRONGXML_STATE;
            return false;
        }
        return true;
    }
    
    private URL buildDataUrl(String pathString) throws MalformedURLException{
        try {
            URL url = new URL(pathString);
            return url;
        } catch(MalformedURLException mue) {
            String infoUrlString = scrutariInfo.getUrl().toString();
            int idx = infoUrlString.lastIndexOf('/');
            URL url = new URL(infoUrlString.substring(0,idx+1) + pathString);
            return url;
        }
    }

    public interface CollectErrorHandler {
        
        
        public void warningDownload(URL url, String message);
    }
    
    class InternalTrustManager implements javax.net.ssl.X509TrustManager {
        public void checkClientTrusted(X509Certificate[] chain, String authType) {
    }
 
    public void checkServerTrusted(X509Certificate[] chain, String authType) {
    }
 
    public X509Certificate[] getAcceptedIssuers() {
        return null;
    }
    }
    
}
