/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.conf;

import java.io.*;
import fr.exemole.sctserver.conf.xml.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.util.logging.*;
import javax.xml.parsers.*;

/**
 *
 * @author Vincent Calame
 */
public class DefaultConfDirectory implements ConfDirectory {
    private File confDir;
    private InternalConfFileInfo sourcesFileInfo = new InternalConfFileInfo();
    private InternalConfFileInfo corpusClassesFileInfo = new InternalConfFileInfo();
    private InternalConfFileInfo searchLogFilterFileInfo = new InternalConfFileInfo();
    private UrlKey[] sourceUrlKeyArray = null;
    private CorpusClasse[] corpusClasseArray = null;
    private SearchLogDeny[] searchLogDenyArray = null;
    
    public DefaultConfDirectory(File confDir) {
        this.confDir = confDir;
    }
    
    public UrlKey[] getInfoUrlKeyArray() {
        return sourceUrlKeyArray;
    }
    
    public CorpusClasse[] getCorpusClasseArray() {
        return corpusClasseArray;
    }
    
    public SearchLogDeny[] getSearchLogDenyArray() {
        return searchLogDenyArray;
    }
    
    public void reload() throws ConfIOException {
        clear();
        try {
            if (!confDir.exists()) {
                boolean done = confDir.mkdirs();
                if (!done) throw new ConfIOException("confDirectory cannot be created : " + confDir.getPath());
            } else if (!confDir.isDirectory()) {
                throw new ConfIOException("confDirectory is not a directory : " + confDir.getPath());
            }
            File[] fileList = confDir.listFiles();
            for(int i = 0; i < fileList.length; i++) {
                File f = fileList[i];
                if (!f.isDirectory()) {
                    String name = f.getName();
                    if ((name.length() > 4) && (name.endsWith(".xml"))) {
                        String baseName = name.substring(0, name.length() - 4);
                        if (baseName.equals(SOURCES_NAME)) {
                            initSources(f);
                        } else if (baseName.equals(CORPUS_CLASSES_NAME)) {
                            initCorpusClasses(f);
                        } else if (baseName.equals(SEARCHLOG_FILTER_NAME)) {
                            initSearchLogFilter(f);
                        }
                    }
                }
            }
        }
        catch(SecurityException se) {
            throw new ConfIOException(se);
        }
        catch(IOException ioe) {
            throw new ConfIOException(ioe);
        }
        catch(ParserConfigurationException pe) {
            throw new ConfIOException(pe);
        }
    }
    
    private void clear() {
        sourcesFileInfo.clear();
        corpusClassesFileInfo.clear();
        searchLogFilterFileInfo.clear();
        sourceUrlKeyArray = null;
        corpusClasseArray = null;
        searchLogDenyArray = null;
    }
    
    private void initSources(File f) throws IOException, ParserConfigurationException {
        sourcesFileInfo.setHere(true);
        SourcesConf sourcesConf = SourcesConf.load(f);
        sourcesFileInfo.setProcessLog(sourcesConf.getProcessLog());
        sourcesFileInfo.setProcessURI(sourcesConf.getProcessURI());
        sourceUrlKeyArray = sourcesConf.getInfoUrlKeyArray();
    }
    
    private void initCorpusClasses(File f) throws IOException, ParserConfigurationException {
        corpusClassesFileInfo.setHere(true);
        CorpusClassesConf corpusClassesConf = CorpusClassesConf.load(f);
        corpusClassesFileInfo.setProcessLog(corpusClassesConf.getProcessLog());
        corpusClassesFileInfo.setProcessURI(corpusClassesConf.getProcessURI());
        corpusClasseArray = corpusClassesConf.getCorpusClasseArray();
    }
    
    private void initSearchLogFilter(File f) throws IOException, ParserConfigurationException {
        searchLogFilterFileInfo.setHere(true);
        SearchLogFilterConf searchLogFilterConf = SearchLogFilterConf.load(f);
        searchLogFilterFileInfo.setProcessLog(searchLogFilterConf.getProcessLog());
        searchLogFilterFileInfo.setProcessURI(searchLogFilterConf.getProcessURI());
        searchLogDenyArray = searchLogFilterConf.getSearchLogDenyArray();
    }
    
    public ConfFileInfo getConfFileInfo(String name) {
        if (name.equals(SOURCES_NAME)) {
            return sourcesFileInfo;
        } else if (name.equals(CORPUS_CLASSES_NAME)) {
            return corpusClassesFileInfo;
        } else if (name.equals(SEARCHLOG_FILTER_NAME)) {
            return searchLogFilterFileInfo;
        }
        else {
            throw new IllegalArgumentException("Unknown name : " + name);
        }
    }

    
    private static class InternalConfFileInfo implements ConfFileInfo {
        private boolean here = false;
        private ProcessLog processLog;
        private ProcessURI processURI;
        
        private InternalConfFileInfo() {
            
        }
        
        private void clear() {
            this.here = false;
            this.processLog = null;
            this.processURI = null;
        }
        
        public boolean isHere() {
            return here;
        }
    
        public ProcessLog getProcessLog() {
            return processLog;
        }
        
        public ProcessURI getProcessURI() {
            return processURI;
        }
        
        private void setHere(boolean here) {
            this.here = here;
        }
        
        private void setProcessLog(ProcessLog processLog) {
            this.processLog = processLog;
        }
        
        private void setProcessURI(ProcessURI processURI) {
            this.processURI = processURI;
        }
        
    }
    
}
