/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2007-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.simple;

import fr.exemole.sctserver.*;
import fr.exemole.sctserver.request.*;
import fr.exemole.sctserver.request.simple.catalog.*;
import net.mapeadores.scrutari.searchengine.util.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.searchengine.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.exceptions.*;
import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public class SimpleRequestFactory {
    public final static String LOG_PARAMETER = "log";
    public final static String SITE_PARAMETER = "site";
    public final static String REMOTE_PARAMETER = "remote";
    public final static String REFERRER_PARAMETER = "referrer";
    
    
    private SimpleRequestFactory() {
    }
    
    public static SimpleRequest getSimpleRequest(SctServer sctServer, SimpleRequestMap requestMap) throws SimpleParameterErrorException {
        String param_chiffre = requestMap.getParameter("chiffre");
        if (param_chiffre != null) {
            return getChiffreRequest(sctServer.getScrutariDB(), param_chiffre,requestMap);
        }
        String param_tableau = requestMap.getParameter("tableau");
        if (param_tableau != null) {
            return getTableauRequest(sctServer, param_tableau, requestMap);
        }
        String param_data = requestMap.getParameter("data");
        if (param_data != null) {
            return getDataRequest(sctServer.getScrutariDB(), param_data, requestMap);
        }
        String param_idrecherche = requestMap.getParameter("idrecherche");
        if (param_idrecherche != null) {
            return getIdrechercheRequest(sctServer, param_idrecherche, requestMap);
        }
        String param_recherche = requestMap.getCleanParameter("recherche");
        if (param_recherche != null) {
            return getRechercheRequest(sctServer, param_recherche, requestMap);
        }
        throw new SimpleParameterErrorException();
    }
    
    private static SimpleRequest getRechercheRequest(SctServer sctServer,String param_recherche,SimpleRequestMap requestMap) throws SimpleParameterErrorException  {
        SearchSequence sequence = SearchSequence.createSearchSequence(param_recherche);
        if ((sequence == null) || (sequence.getTokenCount() == 0)) {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.EMPTY_SEARCHSEQUENCE_PARAMETER,"recherche", param_recherche);
        }
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        SearchOptions searchOptions = getSearchOptions(scrutariDB,requestMap);
        RequestInfo requestInfo = getRequestInfo(requestMap);
        SearchResult searchResult = sctServer.newSearch(sequence,searchOptions,requestInfo);
        return new SearchHeaderRequest(scrutariDB, searchResult);
    }
    
    private static SearchOptions getSearchOptions(ScrutariDB scrutariDB,SimpleRequestMap requestMap) {
        SearchOptions searchOptions = new SearchOptions();
        int langUi = getLangUi(requestMap);
        searchOptions.setLangUi(langUi);
        int[] langIntegerArray = getLangIntegerArray(requestMap);
        searchOptions.setLangIntegerArray(langIntegerArray);
        int[] codecorpusArray = getIntArray(requestMap.getParameter("codescorpus"));
        searchOptions.setCorpusCodeArray(codecorpusArray);
        int[] codethesaurusArray = null;
        if (codecorpusArray != null) {
            codethesaurusArray = SearchOptionsUtils.getCodethesaurusArray(scrutariDB, codecorpusArray);
        }
        searchOptions.setThesaurusCodeArray(codethesaurusArray);
        searchOptions.setDefaultSearchType(TextConstants.STARTSWITH);
        return searchOptions;
    }
    
    private static SimpleRequest getIdrechercheRequest(SctServer sctServer, String param_idrecherche, SimpleRequestMap requestMap) throws SimpleParameterErrorException {
        SearchResult searchResult = sctServer.getSearchResult(param_idrecherche);
        if (searchResult == null) {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.UNKNWON_IDRECHERCHE_PARAMETER,"idrecherche",param_idrecherche);
        }
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        String param_info = requestMap.getParameter("info");
        if (param_info == null) {
            throw new SimpleParameterErrorException("info");
        }
        //int langUi = getLangUi(requestMap);
        if (param_info.equals("f")) {
            int plageStart = getPlageStart(requestMap);
            int plageLength = getPlageLength(requestMap);
            FicheResultRequest ficheResultRequest = new FicheResultRequest(scrutariDB,searchResult, plageStart, plageLength);
            ficheResultRequest.setGlobalSearchOptions(sctServer.getOptionsManager().getGlobalSearchOptions());
            ficheResultRequest.setDecalageUTF8(getDecalageUTF8(requestMap));
            return ficheResultRequest;
        } else if (param_info.equals("h")) {
            return new SearchHeaderRequest(scrutariDB, searchResult);
        } else if (param_info.equals("m")) {
            MotcleResultRequest motcleResultRequest = new MotcleResultRequest(scrutariDB,searchResult);
            motcleResultRequest.setDecalageUTF8(getDecalageUTF8(requestMap));
            motcleResultRequest.setUiLangInteger(searchResult.getSearchOptions().getLangUi());
            return motcleResultRequest;
        } else {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.WRONG_PARAMETER,"info", param_info);
        }
    }
    
    private static SimpleRequest getDataRequest(ScrutariDB scrutariDB,String param_data,SimpleRequestMap requestMap) throws SimpleParameterErrorException {
        int langUi = getLangUi(requestMap);
        if (param_data.equals("corpus")) {
            int code = getCode(requestMap);
            CorpusData corpusData = scrutariDB.getDataManager().getCorpusDataByCode(code);
            if (corpusData == null) {
                throw new SimpleParameterErrorException(SimpleParameterErrorException.UNKNWON_CORPUSCODE_PARAMETER,"code",requestMap.getParameter("code"));
            }
            return new CorpusDataRequest(scrutariDB,langUi, corpusData);
        } else if (param_data.equals("base")) {
            int code = getCode(requestMap);
            BaseData baseData = scrutariDB.getDataManager().getBaseDataByCode(code);
            if (baseData == null) {
                throw new SimpleParameterErrorException(SimpleParameterErrorException.UNKNWON_BASECODE_PARAMETER,"code",requestMap.getParameter("code"));
            }
            return new BaseDataRequest(scrutariDB,langUi, baseData);
        } else if (param_data.equals("thesaurus")) {
            int code = getCode(requestMap);
            ThesaurusData thesaurusData = scrutariDB.getDataManager().getThesaurusDataByCode(code);
            if (thesaurusData == null) {
                throw new SimpleParameterErrorException(SimpleParameterErrorException.UNKNWON_THESAURUSCODE_PARAMETER,"code",requestMap.getParameter("code"));
            }
            return new ThesaurusDataRequest(scrutariDB,langUi, thesaurusData);
        }
        throw new SimpleParameterErrorException(SimpleParameterErrorException.WRONG_PARAMETER,"info",param_data);
    }
    
    private static SimpleRequest getTableauRequest(SctServer sctServer,String param_tableau,SimpleRequestMap requestMap) throws SimpleParameterErrorException {
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        int langUi = getLangUi(requestMap);
        if (param_tableau.equals("corpus")) {
            return new CorpusDataRequest(scrutariDB,langUi);
        } else if (param_tableau.equals("thesaurus")) {
            return new ThesaurusDataRequest(scrutariDB,langUi);
        } else if (param_tableau.equals("base")) {
            return new BaseDataRequest(scrutariDB,langUi);
        } else if (param_tableau.equals("classe")) {
            return new ClasseRequest(sctServer,langUi);
        }
        throw new SimpleParameterErrorException(SimpleParameterErrorException.WRONG_PARAMETER,"tableau",param_tableau);
    }
    
    private static SimpleRequest getChiffreRequest(ScrutariDB scrutariDB,String chiffreParam,SimpleRequestMap requestMap) throws SimpleParameterErrorException {
        if (chiffreParam.equals("codecorpus")) {
            int codebase = getCodeBase(requestMap);
            String param_nomcorpus = requestMap.getParameter("nomcorpus");
            if (param_nomcorpus == null) {
                throw new SimpleParameterErrorException("nomcorpus");
            }
            return new CorpusChiffreRequest(scrutariDB,codebase,param_nomcorpus);
        } else if (chiffreParam.equals("codethesaurus")) {
            int codebase = getCodeBase(requestMap);
            String param_nomthesaurus = requestMap.getParameter("nomthesaurus");
            if (param_nomthesaurus == null) {
                throw new SimpleParameterErrorException("nomthesaurus");
            }
            return new ThesaurusChiffreRequest(scrutariDB,codebase,param_nomthesaurus);
        } else if (chiffreParam.equals("total")) {
            int[] langIntegerArray = getLangIntegerArray(requestMap);
            String param_codescorpus = requestMap.getParameter("codescorpus");
            if (param_codescorpus != null) {
                int[] codecorpusArray = getIntArray(param_codescorpus);
                return new TotalParCorpusRequest(scrutariDB,codecorpusArray,langIntegerArray);
            }
            String param_codesbases = requestMap.getParameter("codesbases");
            int[] codebaseArray = getIntArray(param_codesbases);
            return new TotalParBaseRequest(scrutariDB,codebaseArray,langIntegerArray);
        }
        throw new SimpleParameterErrorException(SimpleParameterErrorException.WRONG_PARAMETER,"chiffre",chiffreParam);
    }
    
    
    
    private static int getCodeBase(SimpleRequestMap requestMap) throws SimpleParameterErrorException {
        String param_codebase = requestMap.getParameter("codebase");
        if (param_codebase == null) {
            throw new SimpleParameterErrorException("codebase");
        }
        try {
            int codebase = Integer.parseInt(param_codebase);
            return codebase;
        } catch(NumberFormatException nfe) {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.NOTINTEGER_PARAMETER,"codebase",param_codebase);
        }
        
    }
    
    private static int[] getLangIntegerArray(SimpleRequestMap requestMap) {
        String param_langs = requestMap.getParameter("langs");
        if (param_langs == null) return null;
        int[] langIntegerArray = LangIntegerUtils.toCleanLangIntegerArray(param_langs);
        if (langIntegerArray.length == 0) return null;
        return langIntegerArray;
    }
    
    private static int[] getIntArray(String values) {
        if (values == null) return null;
        int[] array = ArrayUtils.toIntArray(values,'c',true);
        if (array.length == 0) return null;
        return array;
    }
    
    private static int getLangUi(SimpleRequestMap requestMap) {
        String langUi = requestMap.getParameter("langui");
        if (langUi == null) return LangInteger.UNDETERMINED;
        try {
            int langInteger = LangInteger.parse(langUi);
            return langInteger;
        } catch(LangIntegerException lie) {
            return LangInteger.UNDETERMINED;
        }
    }
    
    private static int getCode(SimpleRequestMap requestMap) throws SimpleParameterErrorException {
        String param_code = requestMap.getParameter("code");
        if (param_code == null) {
            throw new SimpleParameterErrorException("code");
        }
        try {
            int code = Integer.parseInt(param_code);
            return code;
        } catch(NumberFormatException nfe) {
            throw new SimpleParameterErrorException(SimpleParameterErrorException.NOTINTEGER_PARAMETER,"code",param_code);
        }
    }
    
    private static boolean getDecalageUTF8(SimpleRequestMap requestMap) {
        String param_dclg = requestMap.getParameter("dclg");
        boolean decalageutf8 = false;
        if ((param_dclg != null) && (param_dclg.equals("1"))) decalageutf8 = true;
        return decalageutf8;
    }
    
    private static RequestInfo getRequestInfo(SimpleRequestMap requestMap) {
        boolean withLog = true;
        String param_journal = requestMap.getParameter(LOG_PARAMETER);
        if (param_journal == null) param_journal = requestMap.getParameter("journal");
        if ((param_journal != null) && (param_journal.equals("0"))) withLog = false;
        String remote = requestMap.getParameter(REMOTE_PARAMETER);
        String site = requestMap.getParameter(SITE_PARAMETER);
        String referrer = requestMap.getParameter(REFERRER_PARAMETER);
        return new RequestInfo(remote, withLog, site, referrer);
    }
    
    
    
    private static int getPlageStart(SimpleRequestMap requestMap) {
        String param_debutplage = requestMap.getParameter("dbtplage");
        if (param_debutplage == null) return 0;
        try {
            return Integer.parseInt(param_debutplage);
        } catch(NumberFormatException nfe) {
            return 0;
        }
    }
    
    private static int getPlageLength(SimpleRequestMap requestMap) {
        String param_debutplage = requestMap.getParameter("lngplage");
        if (param_debutplage == null) return -1;
        try {
            return Integer.parseInt(param_debutplage);
        } catch(NumberFormatException nfe) {
            return -1;
        }
    }
    
}
