/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.directory;

import fr.exemole.sctserver.conf.*;
import fr.exemole.sctserver.admin.*;
import fr.exemole.sctserver.storage.*;
import fr.exemole.sctserver.*;
import java.io.*;
import net.mapeadores.scrutari.searchengine.beans.*;
import net.mapeadores.util.exceptions.*;
import java.util.*;
import net.mapeadores.util.text.*;
import java.text.*;
import net.mapeadores.util.io.*;
import net.mapeadores.util.primitives.*;
import fr.exemole.sctserver.searchlog.*;
import net.mapeadores.scrutari.db.*;

/**
 *
 * @author Vincent Calame
 */
public class SctServerDirectory implements SctServerStorage {
    private static final DateFormat dateformatanneemois = new SimpleDateFormat("yyyyMM");
    private static final DateFormat dateformatjour = new SimpleDateFormat("dd");
    File varDir;
    File berkeleyDirectory;
    File logsDirectory;
    File searchresultsDirectory;
    File dataDirectory;
    private InternalLogAccess internalLogAccess = new InternalLogAccess();
    private InternalSearchLogAccess searchLogAccess = new InternalSearchLogAccess();
    private InternalSearchLog searchLog = new InternalSearchLog();
    private DefaultConfDirectory confDirectory;
    
    public SctServerDirectory(File confDir, File varDir) {
        this.varDir = varDir;
        confDirectory = new DefaultConfDirectory(confDir);
        berkeleyDirectory = getDirectory("berkeley");
        try {
            IOUtils.cleanDirectory(berkeleyDirectory);
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
        logsDirectory = getDirectory("logs");
        searchresultsDirectory = getDirectory("searchresults");
        dataDirectory = getDirectory("data");
        
        File jeLock = new File(berkeleyDirectory,"je.lck");
        if (jeLock.exists()) jeLock.delete();
    }
    
    public File getBerkeleyDirectory(String baseName) {
        File directory = new File(berkeleyDirectory, baseName);
        directory.mkdirs();
        return directory;
    }
    
    public void deleteBerkelyDirectory(String baseName) {
        try {
            File directory = new File(berkeleyDirectory, baseName);
            if (directory.exists()) IOUtils.cleanDirectory(directory);
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }
    
    public synchronized void saveSearchResult(SearchResultBean bean) {
        try {
            String identifiant = bean.getIdentifiant();
            int idx = identifiant.indexOf("-");
            File sousdir = new File(searchresultsDirectory,identifiant.substring(0,idx));
            sousdir.mkdirs();
            File f = new File(sousdir,"r"+identifiant.substring(idx+1));
            ObjectOutputStream outputobj = new ObjectOutputStream(new FileOutputStream(f));
            outputobj.writeObject(bean);
            outputobj.close();
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }
    
    public LogAccess getLogAccess() {
        return internalLogAccess;
    }
    
    public ConfDirectory getConfDirectory() {
        return confDirectory;
    }
    
    public SearchResultBean getSearchResultBean(String identifiant) {
        SearchResultBean searchResult = null;
        try {
            int idx = identifiant.indexOf("-");
            if (idx == -1) return null;
            File f = new File(searchresultsDirectory,identifiant.substring(0,idx)+ File.separator + "r"+identifiant.substring(idx+1));
            if (!f.exists()) return null;
            FileInputStream fis = new FileInputStream(f);
            ObjectInputStream ois = new ObjectInputStream(fis);
            try {
                searchResult = (SearchResultBean) ois.readObject();
            } catch(ClassNotFoundException cnfe) {
                throw new NestedClassLoaderException(cnfe);
            }
            ois.close();
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
        return searchResult;
    }
    
    private File getDirectory(String name) {
        File directory = new File(varDir,name);
        testDirectory(directory);
        return directory;
    }
    
    private void testDirectory(File directory) {
        if (!directory.exists()) {
            directory.mkdirs();
        } else if (!directory.isDirectory()) {
            throw new SctServerDirectoryException(SctServerDirectoryException.NOT_A_DIRECTORY,directory.getPath());
        }
    }
    
    public SearchLogAccess getSearchLogAccess() {
        return searchLogAccess;
    }
    
    public void clean() {
        try {
            cleanDirectory(berkeleyDirectory);
            cleanDirectory(logsDirectory);
            cleanDirectory(searchresultsDirectory);
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }
    
    public InputStream getScrutariData(ScrutariInfo scrutariInfo) {
        File file = getDataFile(scrutariInfo);
        if (!file.exists()) return null;
        try {
            return new FileInputStream(file);
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }
    
    public void saveScrutariData(ScrutariInfo scrutariInfo,InputStream inputStream) {
        File file = getDataFile(scrutariInfo);
        try {
            OutputStream outputStream = new FileOutputStream(file);
            IOUtils.copy(inputStream,outputStream);
            outputStream.close();
        } catch(IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }
    
    private File getDataFile(ScrutariInfo sourceInfo) {
        return new File(dataDirectory, sourceInfo.getKeyString() + ".xml");
    }
    
    public void cleanSearchResults() {
        long time = System.currentTimeMillis();
        long past = time - (1000 * 60 * 60 * 48);
        File[] files = searchresultsDirectory.listFiles();
        for(int i = 0; i < files.length;i++) {
            File f = files[i];
            if (f.isDirectory()) {
                long lastModified = f.lastModified();
                if (lastModified < past) {
                    try {
                        IOUtils.deleteDirectory(f);
                    } catch(IOException ioe) {
                        throw new SctStorageException(ioe);
                    }
                }
            }
        }
    }
    
    private void cleanDirectory(File directory) throws IOException {
        IOUtils.deleteDirectory(directory);
        directory.mkdirs();
    }
    
    private void checkData(List scrutariInfoList) throws IOException {
        HashSet set = new HashSet();
        for(Iterator it = scrutariInfoList.iterator(); it.hasNext();) {
            ScrutariInfo scrutariInfo = (ScrutariInfo) it.next();
            set.add(scrutariInfo.getKeyString() + ".xml");
        }
        File[] files = dataDirectory.listFiles();
        for(int i = 0; i < files.length; i++) {
            File f = files[i];
            if (!f.isDirectory()) {
                if (!set.contains(f.getName())) {
                    f.delete();
                }
            }
        }
    }
    
    private int getId(String name) {
        if ((name.length() > 17) && (name.startsWith("scrutari-data_"))) {
            String sub = name.substring(14, 17);
            try {
                int i = Integer.parseInt(sub);
                return i;
            } catch(NumberFormatException nfe) {
                return -2;
            }
        }
        return -1;
    }
    
    private class InternalLogAccess implements LogAccess {
        
        public InputStream getLogInputStream(String logName) {
            File f = new File(logsDirectory, logName);
            if (!f.exists()) return null;
            try {
                return new FileInputStream(f);
            } catch(IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }
        
        public OutputStream getLogOutputStream(String logName) {
            File f = new File(logsDirectory, logName);
            try {
                return new FileOutputStream(f);
            } catch(IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }
    }
    
    private class InternalSearchLogAccess implements SearchLogAccess {
        
        public void log(SctServer sctServer, SearchLog.QueryLog queryLog)  {
            Date date = new Date();
            String anneemois = dateformatanneemois.format(date);
            String jour = dateformatjour.format(date);
            File repjournal = new File(logsDirectory,anneemois);
            File logFile = new File(repjournal,jour);
            StringBuffer buf = new StringBuffer();
            if (!logFile.exists()) {
                CountStats countStats = sctServer.getScrutariDB().getCountStats();
                append(buf, "fiche", countStats.getFicheCount());
                append(buf, "motcle", countStats.getMotcleCount());
                append(buf, "corpus", countStats.getCorpusCount());
                append(buf, "thesaurus", countStats.getThesaurusCount());
                append(buf, "base", countStats.getBaseCount());
                append(buf, "lexie", countStats.getLexieCount());
                append(buf, "indexation", countStats.getIndexationCount());
            }
            buf.append(queryLog.getRemoteAddress());
            buf.append(':');
            buf.append(queryLog.getSearchSequence());
            String site = queryLog.getSite();
            buf.append(':');
            buf.append(site);
            buf.append(':');
            buf.append(queryLog.getReferrer());
            appendInt(buf, queryLog.getFicheCount());
            appendInt(buf, queryLog.getFicheMaximum());
            appendInt(buf, queryLog.getAllMotcleCount());
            appendInt(buf, queryLog.getCompleteMotcleCount());
            buf.append('\n');
            try {
                repjournal.mkdirs();
                synchronized(this) {
                    Writer writer = new OutputStreamWriter(new FileOutputStream(logFile, true),"UTF-8");
                    writer.write(buf.toString());
                    writer.close();
                }
            } catch(IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }
        
        public SearchLog getSearchLog() {
            return searchLog;
        }
        
        private void appendInt(StringBuffer buf, int value) {
            buf.append(':');
            if (value >= 0) {
                buf.append(value);
            }
        }
        
        private void append(StringBuffer buf, String type, int value) {
            buf.append("!");
            buf.append(type);
            buf.append("=");
            buf.append(value);
            buf.append('\n');
        }
        
    }
    
    private class InternalSearchLog implements SearchLog {
        
        public int[] getMonthIntegerArray() {
            int[] result;
            if ((!logsDirectory.exists()) || (!logsDirectory.isDirectory())) {
                return new int[0];
            }
            File[] liste = logsDirectory.listFiles();
            TreeSet monthSet = new TreeSet();
            for(int i = 0; i < liste.length; i++) {
                File dir = liste[i];
                if (!dir.isDirectory()) continue;
                String name = dir.getName();
                if (name.length() != 6) continue;
                String dateString = name.substring(0,4) + "-" + name.substring(4);
                try {
                    int mois = IntDate.parse(dateString);
                    monthSet.add(new Integer(mois));
                } catch(IntDateException ide) {
                    continue;
                }
                
            }
            int size = monthSet.size();
            result = new int[size];
            int i = size - 1;
            for(Iterator it = monthSet.iterator(); it.hasNext();) {
                Integer itg = (Integer) it.next();
                result[i] = itg.intValue();
                i--;
            }
            return result;
        }
        
        public DayLog[] getDayLogArray(int monthInteger) {
            TreeMap tree = new TreeMap();
            InternalDayLog[] result;
            String moisString = IntDate.toString(monthInteger);
            File repmois = new File(logsDirectory,moisString.substring(0,4) + moisString.substring(5));
            if ((!repmois.exists()) || (!repmois.isDirectory())) {
                return new InternalDayLog[0];
            }
            File[] liste = repmois.listFiles();
            for (int j = 0; j < liste.length;j++) {
                File fich = liste[j];
                if (!fich.isDirectory()) {
                    String nom = fich.getName();
                    if (nom.length() == 2) {
                        int dateInteger = monthInteger;
                        try {
                            int jour = IntDate.parse(moisString + "-" + nom);
                            InternalDayLog dayLog = new InternalDayLog(jour);
                            dayLog.init(fich);
                            
                            tree.put(nom,dayLog);
                        } catch(IntDateException ide) {
                            
                        }
                        
                    }
                }
            }
            result = new InternalDayLog[tree.size()];
            int i = 0;
            for(Iterator it = tree.values().iterator(); it.hasNext();) {
                result[i] = (InternalDayLog) it.next();
                i++;
            }
            return result;
        }
        
    }
    
    private class InternalDayLog implements SearchLog.DayLog {
        private int dateInteger;
        private InternalQueryLog[] queryLogArray = new InternalQueryLog[0];
        private InternalDayStats dayStats = new InternalDayStats();
        
        InternalDayLog(int dateInteger) {
            this.dateInteger = dateInteger;
        }
        
        public int getDateInteger() {
            return dateInteger;
        }
        
        public SearchLog.QueryLog[] getQueryLogArray() {
            return queryLogArray;
        }
        
        public SearchLog.DayStats getDayStats() {
            return dayStats;
        }
        
        private void init(File f) {
            List list = new ArrayList();
            try {
                BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(f), "UTF-8"));
                String chaine = null;
                while((chaine = reader.readLine()) != null) {
                    chaine = chaine.trim();
                    if (chaine.length() == 0) continue;
                    if (chaine.charAt(0) == '!') {
                        initDayStats(chaine);
                        continue;
                    }
                    InternalQueryLog queryLog = new InternalQueryLog();
                    String[] tokens = StringUtils.getTokens(chaine, ':', StringUtils.EMPTY_INCLUDE);
                    for(int i = 0; i < tokens.length; i++) {
                        String token = tokens[i];
                        boolean hasToBreak = false;
                        switch(i) {
                            case 0 :
                                queryLog.setRemoteAddress(token);
                                break;
                            case 1 :
                                queryLog.setSearchSequence(token);
                                break;
                            case 2 :
                                queryLog.setSite(token);
                                break;
                            case 3 :
                                queryLog.setReferrer(token);
                                break;
                            case 4 :
                                queryLog.setFicheCount(getInteger(token));
                                break;
                            case 5 :
                                queryLog.setFicheMaximum(getInteger(token));
                                break;
                            case 6 :
                                queryLog.setAllMotcleCount(getInteger(token));
                                break;
                            case 7 :
                                queryLog.setCompleteMotcleCount(getInteger(token));
                                break;
                            default :
                                hasToBreak = true;
                        }
                        if (hasToBreak) break;
                    }
                    list.add(queryLog);
                }
                reader.close();
            } catch(IOException ioe) {
                
            }
            int size = list.size();
            queryLogArray = new InternalQueryLog[size];
            for(int i = 0; i < size; i++) {
                queryLogArray[i] = (InternalQueryLog) list.get(i);
            }
        }
        
        private void initDayStats(String s) {
            int idx = s.indexOf('=');
            if (idx == -1) return;
            String key = s.substring(1,idx).trim();
            String value = s.substring(idx+1).trim();
            int val = 0;
            try {
                val = Integer.parseInt(value);
            } catch(NumberFormatException nfe) {
                return;
            }
            if (key.equals("fiche")) {
                dayStats.setFicheCount(val);
            } else if (key.equals("motcle")) {
                dayStats.setMotcleCount(val);
            } else if (key.equals("corpus")) {
                dayStats.setCorpusCount(val);
            } else if (key.equals("thesaurus")) {
                dayStats.setThesaurusCount(val);
            } else if (key.equals("base")) {
                dayStats.setBaseCount(val);
            } else if (key.equals("lexie")) {
                dayStats.setLexieCount(val);
            } else if (key.equals("indexation")) {
                dayStats.setIndexationCount(val);
            }
        }
    }
    
    private static int getInteger(String s) {
        try {
            return Integer.parseInt(s);
        } catch(NumberFormatException nfe) {
            return -1;
        }
    }
    
    private class InternalQueryLog extends DefaultQueryLog {
        
        private void setRemoteAddress(String remoteAddress) {
            innerSetRemoteAddress(remoteAddress);
        }
        
        private void setSearchSequence(String searchSequence) {
            innerSetSearchSequence(searchSequence);
        }
        
        private void setSite(String site) {
            innerSetSite(site);
        }
        
        private void setFicheCount(int ficheCount) {
            innerSetFicheCount(ficheCount);
        }
        
        private void setFicheMaximum(int ficheMaximum) {
            innerSetFicheMaximum(ficheMaximum);
        }
        
        private void setAllMotcleCount(int allMotcleCount) {
            innerSetAllMotcleCount(allMotcleCount);
        }
        
        private void setCompleteMotcleCount(int completeMotcleCount) {
            innerSetCompleteMotcleCount(completeMotcleCount);
        }
        
        private void setReferrer(String referrer) {
            innerSetReferrer(referrer);
        }
    }
    
    private class InternalDayStats implements SearchLog.DayStats {
        private int ficheCount = -1;
        private int motcleCount = -1;
        private int thesaurusCount = -1;
        private int corpusCount = -1;
        private int baseCount = -1;
        private int lexieCount = -1;
        private int indexationCount = -1;
        
        public int getCorpusCount() {
            return corpusCount;
        }
        
        public int getThesaurusCount() {
            return thesaurusCount;
        }
        
        public int getMotcleCount() {
            return motcleCount;
        }
        
        public int getFicheCount() {
            return ficheCount;
        }
        
        public int getBaseCount() {
            return baseCount;
        }
        
        public int getLexieCount() {
            return lexieCount;
        }
        
        public int getIndexationCount() {
            return indexationCount;
        }
        
        private void setFicheCount(int ficheCount) {
            this.ficheCount = ficheCount;
        }
        
        private void setMotcleCount(int motcleCount) {
            this.motcleCount = motcleCount;
        }
        
        private void setThesaurusCount(int thesaurusCount) {
            this.thesaurusCount = thesaurusCount;
        }
        
        private void setCorpusCount(int corpusCount) {
            this.corpusCount = corpusCount;
        }
        
        private void setBaseCount(int baseCount) {
            this.baseCount = baseCount;
        }
        
        private void setLexieCount(int lexieCount) {
            this.lexieCount = lexieCount;
        }
        
        private void setIndexationCount(int indexationCount) {
            this.indexationCount = indexationCount;
        }
    }
    
    
    
}
