/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.servlets;

import java.util.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.localisation.*;

/**
 ** La classe ResponseParameters est utilisée pour stocker toutes les informations nécessaires
 * à l'écriture de la réponse des servlets. Cette classe est étendue pour préciser des paramètres
 * supplémentaires (même si ResponseParameters propose avec les méthodes putParameter et getParameter un
 * moyen souple pour stocker des paramètres.
 * <p>Il y a quatre catégories de réponses possibles :
 *<ul>
 *<li>HTMLPAGE : réponse issue d'une interaction avec les servlets de la base.
 * Le format exact dépend des paramètres de l'interrogation envoyé : il peut s'agir d'une page HTML quand les servlets sont intérrogées via un navigateur classique ou d'un rapport en XML si la servlet est interrogée par un client plus complexe.
 *<li>XMLDATA : il s'agit de données XML qui doivent être ensuite traitée par le client.
 *<li>CONTENTSTREAM : il s'agit de données à des formats divers.
 *<li>SPECIALRESPONSE_CATEGORY : il s'agit deux réponses particulières : la redirection et la page non trouvée, comme la redirection ou un message 404 (FileNotFound).
 *</ul>
 *<p>La catégorie d'une réponse est déterminée de deux manières différentes.
 *<ul>
 *<li>Soit par les méthodes setHtmlPageType(), setContentStreamType(), setContentStreamType(), setSpecialResponseType().
 *Ces méthodes ont toute pour argument un entier court (« short »), qui doit être différent de 0. La valeur de ce type dépend de la classe
 * qui étend ResponseParameters. Si une des quatre méthodes est utilisée, getResponsePrinterInstance() renverra vers getResponsePrinterInstanceByType(), méthode abstraite qui doit
 *être écrite pour chaque classe étendant ResponseParameter.
 *<li>Soit par la méthode setResponsePrinterFactory() dans lequel on précise la catégorie ainsi que le nom d'une classe implémentant l'interface ResponsePrinterFactory.
 *</ul>
 *<p>La catégorie d'une réponse ne peut être définie qu'une seule fois. Des exceptions de type UnsupportedOperationException sont renvoyées en cas de tentative de redéfinition de la catégorie.
 *
 * @author Vincent Calame
 */
public class ResponseParameters implements CommandTest {
    public static final short HTMLPAGE_CATEGORY = 1;
    public static final short XMLDATA_CATEGORY = 2;
    public static final short CONTENTSTREAM_CATEGORY = 3;
    public static final short SPECIALRESPONSE_CATEGORY = 4;
    public static final short NOTFOUND_RESPONSE = 1;
    public static final short REDIRECT_RESPONSE = 2;
    public final static String PATH_PARAMNAME = "path";
    private Map<String, Object> paramMap;
    private short type = 0;
    private short category = 0;
    private CommandMessage message;
    private int profondeur = 0;
    private String reversePath = "";
    
    
    public ResponseParameters() {
    }
    
    public boolean isCategorySet() {
        return (category != 0);
    }
    
    /*public void setType(short categoryType, short type) {
        switch(categoryType) {
            case HTMLPAGE_CATEGORY :
                setHtmlPageType(type);
                break;
            case XMLDATA_CATEGORY :
                setXmlDataType(type);
                break;
        }
        
    }*/
    
    /*public short getType() {
        return type;
    }*/
    
      /**
      * Précise le type de page à utiliser. Ne peut être utilisé qu'une fois.
      * Les valeurs de l'argument <code>type</code> sont à la discrétion de la classe étendant ResponseParameters,
      *la valeur 0 étant cependant interdite.
      *
      *@throws UnsupportedOperationException si la catégorie de la page a déjà été précisée
      *@throws IllegalArgumentException si type = 0
      */
     public void setHtmlPageType(short type) {
         if (type == 0) throw new IllegalArgumentException("type == 0");
         if (category != 0) throw new UnsupportedOperationException("category is still defined");
         this.type = type;
         this.category = HTMLPAGE_CATEGORY;
     }
     
     /**
      * Précise le type de donnée xml à retourner. Ne peut être utilisé qu'une fois.
      *Les valeurs de l'argument <code>type</code> sont à la discrétion de la classe étendant ResponseParameters
      *
      *@throws UnsupportedOperationException si la catégorie de la page a déjà été précisée
      *@throws IllegalArgumentException si type = 0
      */
     public void setXmlDataType(short type) {
         if (type == 0) throw new IllegalArgumentException("type == 0");
         if (category != 0) throw new UnsupportedOperationException("category is still defined");
         this.type = type;
         this.category = XMLDATA_CATEGORY;
     }
     
     /**
      * Précise le type de contenu à fournir. Ne peut être utilisé qu'une fois.
      * Les valeurs de l'argument <code>type</code> sont à la discrétion de la classe étendant ResponseParameters,
      *la valeur 0 étant cependant interdite.
      *
      *@throws UnsupportedOperationException si la catégorie de la page a déjà été précisée
      */
     public void setContentStreamType(short type) {
         if (type == 0) throw new IllegalArgumentException("type == 0");
         if (category != 0) throw new UnsupportedOperationException("category is still defined");
         this.type = type;
         this.category = CONTENTSTREAM_CATEGORY;
     }
     
     /**
      * Précise le type de page spéciale. Ne peut être utilisé qu'une fois.
      * Les valeurs de l'argument <code>type</code> sont à la discrétion de la classe étendant ResponseParameters,
      * la valeur 0 étant cependant interdite.
      *
      *@throws UnsupportedOperationException si la catégorie de la page a déjà été précisée
      *@throws IllegalArgumentException si type = 0
      */
     public void setSpecialResponseType(short type, String path) {
         if ((type <1) || (type > 2)) throw new IllegalArgumentException("wrong type");
         if (category != 0) throw new UnsupportedOperationException("category is still defined");
         this.type = type;
         putParameter(PATH_PARAMNAME,path);
         this.category = SPECIALRESPONSE_CATEGORY;
     }
     
     public short getCategory() {
         return category;
     }
     
     /**
      * Retourne le type de page Html à afficher. Retourne 0 si 
      *
      *@throws UnsupportedOperationException si la catégorie n'est pas HTMLPAGE_CATEGORY
      */
     public short getHtmlPageType() {
         if (category != HTMLPAGE_CATEGORY) throw new UnsupportedOperationException("category != HTMLPAGE_CATEGORY");
         return type;
     }
     
     /**
      * Retourne le type de page Html à afficher. 
      *
      *@throws UnsupportedOperationException si la catégorie n'est pas HTMLPAGE_CATEGORY
      */
     public short getXmlDataType() {
         if (category != XMLDATA_CATEGORY) throw new UnsupportedOperationException("category != XMLDATA_CATEGORY");
         return type;
     }
     
     /**
      * Retourne le type de contenu à transmettre. 
      *
      *@throws UnsupportedOperationException si la catégorie n'est pas HTMLPAGE_CATEGORY
      */
     public short getContentStreamType() {
         if (category != CONTENTSTREAM_CATEGORY) throw new UnsupportedOperationException("category != CONTENTSTREAM_CATEGORY");
         return type;
     }
     
     /**
      * Retourne le type de page spécial. N'est pertinent
      *
      *@throws UnsupportedOperationException si la catégorie n'est pas HTMLPAGE_CATEGORY
      */
     public short getSpecialResponseType() {
         if (category != SPECIALRESPONSE_CATEGORY) throw new UnsupportedOperationException("category != CONTENTSTREAM_CATEGORY");
         return type;
     }
     
     public void putParameter(String key,Object value) {
         if (paramMap == null) paramMap = new HashMap<String, Object>();
         paramMap.put(key,value);
     }
     
     public Object getParameter(String key) {
         if (paramMap == null) return null;
         return paramMap.get(key);
     }
     
     /**
      *
      *@throws IllegalArgumentException si le messageType est incorrect.
      */
     public void setCommandMessage(short messageType,String messageKey, Object messageValue) {
        if (messageType == CommandMessage.DONE_MESSAGE) {
            setCommandMessage(CommandMessage.done(messageKey,messageValue));
            return;
        }
        if (messageType == CommandMessage.ERROR_MESSAGE) {
            setCommandMessage(CommandMessage.error(messageKey,messageValue));
            return;
        }
        throw new IllegalArgumentException("wrong messageType");
    }
     
     
     public void setCommandMessage(CommandMessage message) {
         this.message = message;
     }
     
     public void setDoneMessage(String messageKey, Object messageValue) {
         this.message = CommandMessage.done(messageKey,messageValue);
     }
     
    public void setErrorMessage(String messageKey, Object messageValue) {
         this.message = CommandMessage.error(messageKey,messageValue);
     }
    
    public void setMultiErrorMessage(String messageKey, List resourceItemList) {
         this.message = CommandMessage.multiError(new ResourceItem(messageKey),resourceItemList);
     }
     
     public CommandMessage getCommandMessage() {
         return message;
     }
     
     public boolean mayBeDone() {
         if ((message != null) && (message.isErrorMessage())) {
             return false;
         }
         else {
             return true;
         }
     }
     
     public CommandMessage getErrorMessage() {
         if ((message != null) && (message.isErrorMessage())) {
             return message;
         }
         else {
             return null;
         }
     }

    public int getProfondeur() {
        return profondeur;
    }

    public void setProfondeur(int profondeur) {
        this.profondeur = profondeur;
        if (profondeur < 1) {
            reversePath = "";
        }
        else {
            StringBuffer buf = new StringBuffer();
            for(int i = 0; i < profondeur; i++) {
                buf.append("../");
            }
            reversePath = buf.toString();
        }
    }
    
    public String getReversePath() {
        return reversePath;
    }
    
}
