/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections;

/**
 * Abstract base class for {@link IndexAndIntKeyMap}.
 *
 * L'objet est lui-même instance de {@link StableIntList}. <code>getKeyIntList()</code> retourne l'objet lui-même.
 *
 * @author  Vincent Calame
 */
public class AbstractIndexAndIntKeyMap implements IndexAndIntKeyMap {
    InternalStableIntList internalStableIntList = new InternalStableIntList();
    Capsule[] capsules;
    IntHashMap table = new IntHashMap();
    int enabledIndexCount = 0;
    int size = 0;
    
    
    /** Sole constructor. */
    protected AbstractIndexAndIntKeyMap() {
        capsules = new Capsule[32];
    }

    /**
     * Ajoute le lien entre une clé éntière et une valeur. Si l'index était inactif (isEnabledIndex(getIndexByKey(key)) == false), il est réactivé.
     */
    protected synchronized int putFromKey(int key,Object value) {
        Capsule capsule = (Capsule) table.get(key);
        if (capsule != null) {
            capsule.value = value;
            if (!capsule.enabled) {
                enabledIndexCount ++;
                capsule.enabled = true;
            }
            return capsule.index;
        }
        else {
            if (size == capsules.length) {
                Capsule[] tempcaps = new Capsule[size*2];
                for (int k = 0; k < size;k++) {
                    tempcaps[k] = capsules[k];
                }
                capsules = tempcaps;
            }
            capsule = new Capsule(key,size,value);
            capsules[size] = capsule;
            table.put(key,capsule);
            size ++;
            enabledIndexCount++;
            return size-1;
        }
    }
    
    /*
     * Retourne la capsule correspondant à l'index
     */
    private Capsule getCapsuleByIndex(int index) {
        if ((index < 0) || (index >= size)) throw new ArrayIndexOutOfBoundsException(index);
        return capsules[index];
    }
    
    /**
     * Modifie la valeur à un index.
     */
    protected synchronized void setFromIndex(int index,Object value) {
        Capsule capsule = getCapsuleByIndex(index);
        capsule.value = value;
    }
    
    
    /**
     * Supprime la clé entière (autrement dit, désactive l'index correspondant.
     * Résultat similaire à <code>disable(getIndexByKey(key))</code>.
     */
    protected synchronized boolean removeFromKey(int key) {
        Capsule capsule = (Capsule) table.get(key);
        if ((capsule != null) && (capsule.enabled)) {
            capsule.enabled = false;
            capsule.value = null;
            enabledIndexCount --;
            return true;
        }
        else return false;
    }
    
    /**
     * Désactive un index.
     */
    protected synchronized void disable(int index) {
        Capsule capsule = getCapsuleByIndex(index);
        if (capsule.enabled) {
            capsule.enabled = false;
            capsule.value = null;
            enabledIndexCount --;
        }
    }
    
    /**
     * Retourne la clé entière correspondant à l'index. Envoi l'exception DisabledIndexException si l'index a été déactivé.
     */
    public int getKeyByIndex(int index) throws DisabledIndexException {
        Capsule capsule = getCapsuleByIndex(index);
        if (!capsule.enabled) throw new DisabledIndexException(index);
        return capsule.key;
    }
    
    
    /**
     * Retourne l'instance <code>StableIntList</code> qui assure la correspondance entre valeur d'index et clés entières.
     */
    public StableIntList getIntKeyList() {
        return internalStableIntList;
    }
    
    /**
     * Retourne la valeur à l'index <code>index</code>.
     * Retourne <em>null</em> si l'index n'existe pas ou a été désactivé.
     */
    public Object getValueByIndex(int index) {
        Capsule capsule = getCapsuleByIndex(index);
        return capsule.value;
    }
    
    
    
    
    /**
     * Retourne la valeur correspondant à la clé entière <code>key</code>. Retourne <em>null</em> s'il n'y a pas de valeur pour cette clé.
     **/
    public Object getValueByKey(int key) {
        Capsule capsule = (Capsule) table.get(key);
        if (capsule == null) return null;
        return capsule.value;
    }
    
    /**
     * Retourne l'index de la clé entière. Retourne -1 si la clé n'existe pas, c'est à dire si elle n'a jamais été entrée ou si elle a été désactivée antérieurment.
     */
    public int getIndexByKey(int key) {
        Capsule capsule = (Capsule) table.get(key);
        if (capsule == null) return -1;
        if (!capsule.enabled) return -1;
        return capsule.index;
    }
    
    /**
     * Équivalent de getIntKeyList().size().
     **/
    public int length() {
        return size;
    }
    
    public int getEnabledIndexCount() {
        return enabledIndexCount;
    }
            
    
    
    /*
     * Capsule qui conserver la correspondance entre clé entière, index et objets.
     */
    private class Capsule {
        int key;
        int index;
        Object value;
        boolean enabled;
        /* enabled doit avoir la même valeur que (value != null) */
        
        Capsule(int key, int index, Object value) {
            this.key = key;
            this.index = index;
            this.value = value;
            enabled = true;
        }
    }
    
    private class InternalStableIntList extends AbstractStableIntList {
        
        /**
         * Retourne <em>true</em> si l'entier situé à l'index <em>index</em> est disponible ou non.
         */
        public boolean isEnabledIndex(int index) {
            Capsule capsule = getCapsuleByIndex(index);
            return capsule.enabled;
        }
        
        /**
         * Retourne le nombre d'index activés. Doit retourner la même valeur que la méthode <code>size()</code> de l'interface <code>List</code> si aucun index n'est désactivé.
         */
        public int getEnabledIndexCount() {
            return enabledIndexCount;
        }
        
        public int size() {
            return size;
        }
        
        /*
         * Returns the value of the element at the specified position within me.
         */
        public int get(int index) {
            Capsule capsule = getCapsuleByIndex(index);
            return capsule.key;
        }
        
        /**
         * Returns the index of the first and sole occurrence of the specified element within me, or -1 if I do not contain the element.
         *
         */
        public int indexOf(int i) {
            return getIndexByKey(i);
        }

        /**
         * Returns the index of the last occurrence of the specified element within me, or -1 if I do not contain the element.
         * Comme il n'y a pas de doublons, le résultat est le même que <code>indexOf(valeur]</code>
         */
        public int lastIndexOf(int valeur) {
            return indexOf(valeur);
        }
    }
    
}
