/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections;

import java.util.*;
/**
 * Implémentation de l'interface {@link StableList} qui utilise un objet <code>HashMap</code> pour retrouver rapidement l'index d'un élément de la liste.
 * <p>Autrement dit, la méthode <code>indexOf(Object obj)</code> ne fonctionne pas par une itération mais par un appel sur une <code>HasHMap</code>.
 * L'appel à cette méthode se fait donc à temps constant quelque soit la taille de la liste.</p>
 *<p>En outre, cette liste ne tolère aucun doublon.</p>
 *<p>Cette implémentation ne permet pas de désactiver un index.</p>
 *
 * @author  Vincent Calame - Exemole
 */
public class HashTicketAllocator extends AbstractStableList implements TicketAllocator {
    Object[] objects;
    HashMap map = new HashMap();
    int size = 0;
    
    /**  Constructs an empty list*/
    public HashTicketAllocator() {
        objects = new Object[64];
    }
    
    /*
     * Returns the element at the specified position in this list.
     */
    public Object get(int index) {
        if ((index < 0) || (index >= size)) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        return objects[index];
    }
    
    /**
    * Returns the index in this list of the first and sole occurrence of the specified element, or -1 if the List does not contain this element.
    */
     public int indexOf(Object obj) {
        Integer itg = (Integer) map.get(obj);
        if (itg == null) return -1;
        return itg.intValue();
    }
     
     /**
      * same result as <ccode>indexOf<:code>.
      */
     public int lastIndexOf(Object obj) {
         return indexOf(obj);
     }
     
     /**
     * Returns the number of elements in this list.
     **/
    public int size() {
        return size;
    }
    
    /**
     * Équivalent de <code>size()</code> puisqu'aucun index ne peut être désactivé.
     */
    public int getEnabledIndexCount() {
        return size();
    }
    
    /**
     * Retourne toujours <em>true<:em>;
     */
    public boolean isEnabledIndex(int index) {
        return true;
    }

    /**
     * Rajoute un objet dans la liste et renvoie le nouvel index. Si l'objet est présent, retourne l'index actuel.
     * L'objet <code>obj</code> ne peut pas être <em>obj</em>.
     *
     *@throws IllegalArgumentException si l'objet est nul.
     */
    public int allocateTicket(Object obj) {
        if (obj == null) throw new IllegalArgumentException("Argument obj is null");
        Integer itg = (Integer) map.get(obj);
        if (itg != null) return itg.intValue();
        map.put(obj,new Integer(size));
        checkLength();
        objects[size] = obj;
        size++;
        return size-1;
    }
    
    /*
     * Augmente la taille du tableau objects si nécessaire.
     */
    private void checkLength() {
        if (size == objects.length) {
            int taille = objects.length;
            Object[] result = new Object[taille*2];
            for (int i = 0; i < taille;i++) {
                result[i] = objects[i];
            }
            objects = result;
        }
        
    }
    
}
