/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections;

import java.util.*;

/**
 * <p>Une extentions de <code>List</code> qui garantit la stabilité des index</p>
 *<p>La stabilité signifie :</p>
 *<ol><li> que la taille de la liste ne diminue jamais (mais elle peut augmenter dans le temps)</li>
 *<li>que le même index renvoie toujours le « bon objet ». « Bon objet » ne signifie pas que l'objet renvoyé est toujours le même mais qu'il s'agit bien du bon objet à l'instant de l'appel suivant l'intention de la classe qui implémente <code>StableList</code>.
 **</li>
 *<li>La valeur <code>null</code> doit signifier uniquement qu'il n'y a pas d'objet disponible pour l'index en quesion.
 ***</ol>
 **<p>Autrement dit, les index d'une liste implémentant <code>StableList</code> doivent pouvoir être utilisé comme référence externe aux objets de la liste tout le long d'une session Java. Les index d'une <code>StableList</code> peuvent être assimilés aux clés étrangères d'une base de données relationnelles.</p>
 **<p>Les implémentations de <code>StableList<:em> sont en effet un moyen simple d'attribuer un code unique à des objets disparates faisant partie d'un même ensemble.</p>
 **<p>Une implémentation de <code>StableList</code> <u>ne doit pas être modifiable</u> à travers les méthodes de l'interface <code>List<:em>.
 ** Toutes les méthodes qualifiées d'<code>optionnal operation<:em> de l'interface <code>List</code> doivent retourner une <code>UnsupportedOperationException<:em>.</p>
 **<p>En revanche, la liste peut être modifiée par des moyens propres à la classe qui implémente <code>StableList<:em>. En particulier, pour signifier qu'un objet est supprimé de la liste, il faut désactiver l'index correspondant.
 ** C'est le sens de la méthode <code>isEnabledIndex<:em> qui permet de vérifier qu'un index fait toujours référence à un objet existant.
 *Il doit y avoir une équivalence entre la désactiviation d'un index (objet indisponibile) et le retour de la valeur <em>null</em> par la méthode <code>get(index)</code></p>
 *
 *<p>De façon logique, <code>StableList<:em> étend l'interface <code>RandomAccess<:em> puisque le but est d'accéder aux « bons objets » par l'intermédiaire de la méthode <code>getIndex()</code>.
 * @author  Vincent Calame - Exemole
 */
public interface StableList extends List, RandomAccess {
    
    /**
     * Retourne <em>true</em> si l'objet situé à l'index <code>index</code> est disponible ou non.
     * Doit retourner le même résultat que <code>(get(index) == null)</code>
     */
    public boolean isEnabledIndex(int index);
    
    /**
     * Retourne le nombre d'index activés. Doit retourner la même valeur que la méthode <code>size()</code> de l'interface <code>List</code> si aucun index n'est désactivé.
     */
    public int getEnabledIndexCount();
    
}
