/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections.crossing;
import net.mapeadores.util.primitives.BitmaskCounter;
import net.mapeadores.util.primitives.BitmaskUtils;
import net.mapeadores.util.collections.*;
/**
 * Croisement (<em>Crossing</em>) caractérisé par un <em>BitMask</em>.
 * Autrement dit, le croisement entre les deux index peut être caractérisé par 32 valeurs booléennes différentes.
 *<p>Cela permet d'indiquer par exemple que deux index sont croisés par rapport à une ou deux informations mais pas par rapport à dautres.
 *
 * @author  Vincent Calame - Exemole
 */
public abstract class BitmaskCrossing implements CrossingMap.Crossing {
    BitmaskCounter counter;
    int bitmask = 0;

    /**
     * Construit un nouvel objet BitmaskCrossing avec un BitmaskCounter.
     */
    protected BitmaskCrossing(BitmaskCounter counter) {
        this.counter = counter;
    }
    
    /**
     * Ajoute le BitmaskCrossingItem à l'ensemble.
     */
    protected void addBitmaskCrossingItem(BitmaskCrossingItem bitmaskitem) {
        int increasingbit = bitmaskitem.btmsk & (~bitmask);
        counter.increaseCount(increasingbit);
        bitmask = bitmask | bitmaskitem.btmsk;
    }
    
    /**
     * Supprimer le BitmaskCrossingItem à l'ensemble. Retourne <i>true</i> s'il est vide.
     */
    protected boolean removeBitmaskCrossingItem(BitmaskCrossingItem bitmaskitem) {
        int decreasingbit = bitmaskitem.btmsk & bitmask;
        counter.decreaseCount(decreasingbit);
        bitmask = bitmask & (~bitmaskitem.btmsk);
        return (bitmask == 0);
    }
    
     
    /**
     * Retourne la valeur booléenne correspondant au rang <code>rang</code> (de 0 à 31).
     */
    public boolean booleanValueAt(int rang) {
        return BitmaskUtils.booleanValueAt(bitmask,rang);
    }
    
    /**
     * Retourne le bitmask actuel.
     */
    public int getBitmask() {
        return bitmask;
    }
    
    /**
     * Met le bitmask à zéro (équivalent de mettre les 32 valeurs booléennes à <em>false</em>).
     */
    protected void clear() {
        counter.decreaseCount(bitmask);
        bitmask = 0;
    }
    
    /**
     * Doit être implémenté.
     */
    public abstract int getIndex1();
    
    /**
     * Doit être implémenté.
     */
    public abstract int getIndex2();

    
    /**
     * Implémentation de l'interface CrossingMap.CrossingItem qui doit être utilisé pour BitmaskCrossing.
     */
    static public class BitmaskCrossingItem implements CrossingMap.CrossingItem {
        int btmsk;
        
        /**
         * Construit un objet BitmaskCrossingItem qui, rajouté à un BitmaskCrossing, ajoutera le bitmask btmsk.
         */
        public BitmaskCrossingItem(int btmsk) {
            this.btmsk = btmsk;
        }

        
    }
    
}
