/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections.crossing;

import java.util.*;
import org.apache.commons.collections.primitives.ArrayIntList;
import net.mapeadores.util.collections.*;
/**
 *<p>Gère le croisement (<em>Crossing</em>) entre deux listes d'éléments.</p>
 *<p>Dans le cas d'une liste seule, les arguments <code>index1</code> et  <code>index2</code> sont
 * désignent des index de la même liste alors que dans le cas de listes différentes, <code>index1</code>
 *  fait toujours référence à un index de la première liste et <code>index2</code> à un index de la seconde liste.
 *
 * @author  Vincent Calame - Exemole
 */
public class CrossingMap {
    private CrossingModel crossingModel;
    SameIndexBag[] sameIndexBagArray;
    LinkedList chronology = new LinkedList();
    int sameIndexBagCount = 0;
    boolean loneList = false;
    
    /** Creates a new instance of CrossingMap */
    public CrossingMap(CrossingModel crossingModel, int initialCapacity) {
        this(crossingModel,initialCapacity,false);
    }
    
    public CrossingMap(CrossingModel crossingModel, int initialCapacity, boolean loneList) {
        this.crossingModel = crossingModel;
        sameIndexBagArray = new SameIndexBag[initialCapacity];
        this.loneList = loneList;
    }
    
    /**
     * Ajoute l'objet crossingItem au croisement entre l'index1 et l'index2. <code>index1</code>
     * est un index de la première liste, <code>index2</code> un index de la seconde liste.
     * Dans le cas d'une liste seule,
     */
    public void putCrossingItem(int index1,int index2,CrossingItem crossingItem) {
        CrossingEntry existingCrossingEntry = getCrossingEntry(1,index1,index2);
        if (existingCrossingEntry  != null) {
            crossingModel.addCrossingItem(existingCrossingEntry.getCrossing(),crossingItem);
        } else {
            Crossing newCrossing = crossingModel.createCrossing(index1,index2,crossingItem);
            if (newCrossing != null) {
                chronology.add(newCrossing);
                putCrossing(newCrossing);
            }
        }
    }
    
    /**
     * Supprime l'objet CrossingItem au croisement entre l'index1 et l'index2.
     * Retourne <em>true</em> si cela entraîne une suppression totale du croisement
     */
    public boolean removeCrossingItem(int index1,int index2,CrossingItem crossingItem) {
        CrossingEntry existingCrossingEntry = getCrossingEntry(1,index1,index2);
        if (existingCrossingEntry  == null) return false;
        boolean suppression = crossingModel.removeCrossingItem(existingCrossingEntry.getCrossing(),crossingItem);
        if (suppression) {
            chronology.remove(existingCrossingEntry.getCrossing());
            SameIndexBag bag1 = existingCrossingEntry.getSameIndexBag();
            bag1.removeCrossingEntry(1,existingCrossingEntry);
            if (bag1.isEmpty()) {
                removeIndex(index1);
            }
            int otherNum = getSecondNumero();
            CrossingEntry secondCrossingEntry = getCrossingEntry(otherNum,index2,index1);
            if (secondCrossingEntry != null) {
                SameIndexBag otherBag = secondCrossingEntry.getSameIndexBag();
                otherBag.removeCrossingEntry(otherNum,secondCrossingEntry);
                if (otherBag.isEmpty()) {
                    removeIndex(index2);
                }
            }
        }
        return suppression;
    }
    
    
    /*
     * Retourne le second numéro qui est 1 dans le cas d'une liste seule, 2 sinon.
     */
    private int getSecondNumero() {
        if (loneList) return 1;
        return 2;
    }
    
    /**
     * Supprime tous les croisements (<em>Crossing</em>) pour l'index <code>index</code> de la liste <code>numeroliste</code>.
     * Dans le cas d'une liste seule, il n'y a pas de différence de résultat si numeroliste égale 1 ou 2.
     * Retourne le nombre supprimé.
     */
    public int removeAllCrossing(int numeroliste, int index) {
        int otherNum = (numeroliste == 1)?getSecondNumero():1;
        SameIndexBag sameIndexBag = getSameIndexBag(index);
        if (sameIndexBag == null) return 0;
        CrossingEntry crossingEntry = sameIndexBag.getFirstCrossingEntry(numeroliste);
        if (crossingEntry == null) return 0;
        while(crossingEntry != null) {
            chronology.remove(crossingEntry.getCrossing());
            crossingModel.clearCrossing(crossingEntry.getCrossing());
            int otherIndex = crossingEntry.getOtherIndex();
            CrossingEntry otherEntry = getCrossingEntry(otherNum,otherIndex,index);
            if (otherEntry != null) {
                SameIndexBag otherBag = otherEntry.getSameIndexBag();
                otherBag.removeCrossingEntry(otherNum,otherEntry);
                if (otherBag.isEmpty()) {
                    removeIndex(otherIndex);
                }
            }
            crossingEntry = crossingEntry.next();
        }
        int result = sameIndexBag.clear(numeroliste);
        if (sameIndexBag.isEmpty()) {
            removeIndex(index);
        }
        return result;
    }
    
    
    
    /**
     * Retourne <em>true</em> s'il n'y a aucun croisement, <em>false</em> sinon.
     */
    public boolean isEmpty() {
        return (getSameEntryBagCount() == 0);
    }
    
    /**
     * Précise si le croisement se fait entre des éléments d'une même liste seule ou entre des éléments de deux listes différentes.
     */
    public boolean isLoneList() {
        return loneList;
    }
    
    /**
     * Retourne le nombre de Crossing. Raccourci (en légèrement plus rapide) de getCrossingList().size().
     */
    public int getCrossingCount() {
        return chronology.size();
    }
    
    
    
    /**
     * Retourne l'objet Crossing correspondant au croisement entre un index de la première liste (index1) et un index de la seconde liste (index2).
     * Dans le cas d'une liste seule, il s'agit de deux index de cette liste.
     * Retourne <em>null</em> s'il n'y a pas de croisment.
     */
    public Crossing getCrossing(int index1, int index2) {
        CrossingEntry crossingEntry = getCrossingEntry(1,index1,index2);
        if (crossingEntry == null) return null;
        return crossingEntry.getCrossing();
    }
    
    
    /**
     * Retourne un iterator sur les objets <code>Crossing</code> de l'index <em>index</em> de la liste indiquée par <em>numeroliste</em>.
     * <em>numeroliste</em> doit avoir 1 ou 2 comme valeur.
     * Dans le cas d'une liste seule, il n'y a pas de différence de résultat si numeroliste égale 1 ou 2.
     */
    public CrossingIterator getCrossingIterator(int numeroliste,int index) {
        SameIndexBag sameIndexBag = getSameIndexBag(index);
        if (sameIndexBag == null) return new CrossingIterator();
        return new CrossingIterator(sameIndexBag,numeroliste);
    }
    
    /**
     * Retourne la liste (non modifiable et actualisé avec le crossingMap) de tous les objets Crossing ordonnés suivant l'ordre chronologique de saisie.
     * Attention, lorsqu'un crossing est supprimé puis rajouté, l'ordre chronologique sera celui du premier ajout. Cette liste n'implémente pas RandomAccess.
     */
    public List getCrossingList() {
        return Collections.unmodifiableList(chronology);
    }
    
    /**
     * Retourne une copie de tous les index utilisés pour la liste en question.
     */
    public ArrayIntList getIndexIntList(int numeroliste) {
        IntHashMap intHashMap = new IntHashMap();
        ArrayIntList arrayIntList = new ArrayIntList(chronology.size());
        for (Iterator it = chronology.iterator();it.hasNext();) {
            Crossing crossing = (Crossing) it.next();
            int index = (numeroliste == 1)?crossing.getIndex1():crossing.getIndex2();
            if (!intHashMap.containsKey(index)) {
                arrayIntList.add(index);
                intHashMap.put(index,Boolean.TRUE);
            }
        }
        arrayIntList.trimToSize();
        return arrayIntList;
    }
    
    
    
    /**
     * Retourne <em>true</em> s'il y a au moins un croisement (<em>Crossing</em>) pour l'index <code>index</code> de la liste numeroliste, <em>false</em> sinon.
     */
    public boolean hasCrossing(int numeroliste,int index) {
        return (getCrossingCount(numeroliste,index) != 0);
    }
    
    public int getCrossingCount(int numeroliste,int index) {
        SameIndexBag sameIndexBag = getSameIndexBag(index);
        if (sameIndexBag == null) return 0;
        return sameIndexBag.getSize(numeroliste);
    }
    
    
    /*
     * Rajoute le crossing pour l'index de la liste numeroliste avec l'autre index otherIndex de l'autre liste.
     */
    private void putCrossing(Crossing crossing) {
        getOrCreateSameIndexBag(crossing.getIndex1()).addCrossing(1,crossing,crossing.getIndex2());
        getOrCreateSameIndexBag(crossing.getIndex2()).addCrossing(getSecondNumero(),crossing,crossing.getIndex1());
    }
    
    
    /*
     * Retourne le goupre des CrossingEntry pour l'index.
     */
    protected SameIndexBag getSameIndexBag(int index) {
        if (index >= sameIndexBagArray.length) return null;
        return sameIndexBagArray[index];
    }
    
    /*
     * Retourne le groupe des CrossingeEntry pour l'index en le créant au besoin.
     */
    protected SameIndexBag getOrCreateSameIndexBag(int index) {
        int length = sameIndexBagArray.length;
        if (index >= length) {
            int nvln = index * 2;
            if (nvln == 0) nvln = 32;
            SameIndexBag[] nv = new SameIndexBag[nvln];
            System.arraycopy(sameIndexBagArray,0,nv,0,length);
            sameIndexBagArray = nv;
        }
        SameIndexBag sameIndexBag = sameIndexBagArray[index];
        if (sameIndexBag == null) {
            sameIndexBag = SameIndexBag.newInstance(index,loneList);
            sameIndexBagArray[index] = sameIndexBag;
            sameIndexBagCount ++;
        }
        return sameIndexBag;
    }
    
    protected int getSameEntryBagCount() {
        return sameIndexBagCount;
    }
    
    protected void removeIndex(int index) {
        if (sameIndexBagArray[index] != null) {
            sameIndexBagArray[index] = null;
            sameIndexBagCount --;
        }
    }
    
    /*
     * Retourne le crossingEntry avec otHerIndex dans la chaîne initiée par EntriesStart
     */
    private CrossingEntry getCrossingEntry(int numeroliste,int index,int otherIndex) {
        SameIndexBag sameIndexBag = getSameIndexBag(index);
        if (sameIndexBag == null) return null;
        CrossingEntry crossingEntry = sameIndexBag.getFirstCrossingEntry(numeroliste);
        if (crossingEntry == null) return null;
        return searchCrossingEntry(crossingEntry,otherIndex);
    }
    
    /*
     * Retourne crossingEntry si crossingEntry.otherIndex = otherIndex, sinon passe au suivant dans la chaîne.
     */
    private CrossingEntry searchCrossingEntry(CrossingEntry crossingEntry,int otherIndex) {
        if (crossingEntry == null) return null;
        if (crossingEntry.getOtherIndex() == otherIndex) {
            return crossingEntry;
        }
        return searchCrossingEntry(crossingEntry.next(),otherIndex);
    }
    
    
    
    
    /**
     * Interface qui sert à indiquer que l'objet est un croisement (<em>Crossing</em>) entre deux index.
     * Les deux index sont retournés par les deux méthodes <code>getIndex1()</code> et <code>getIndex2()</code>.
     */
    public static interface Crossing {
        
        /**
         * Retourne le premier index du croisement.
         */
        public int getIndex1();
        
        /**
         * Retourne le second index du croisement.
         */
        public int getIndex2();
        
    }
    
    /**
     * Interface implémentée par les objets qui veulent utiliser les capacités de <code>CrossingMap</code>.
     * Les méthodes de cette interface sont appelée par l'instance de <code>CrossingMap</code> dans des méthodes comme <code>putCrossing</code>
     */
    public static interface CrossingModel {
        
        /**
         * Demande de supprimer tous les <code>CrossingItem</code> pour l'objet crossing.
         */
        public void clearCrossing(CrossingMap.Crossing crossing);
        
        /**
         * Demande de créer une nouvelle instance de <code>Crossing</code> comprenant crossingitem.
         * Retourne <em>null</em> si le crossing ne peut pas être créé.
         */
        public Crossing createCrossing(int index1, int index2,CrossingMap.CrossingItem crossingitem);
        
        /**
         * Demande de rajouter crossingItem dans crossing.
         */
        public void addCrossingItem(CrossingMap.Crossing crossing,CrossingMap.CrossingItem crossingItem);
        
        /**
         * Demande de supprimer le crossingItem dans crossing. Retourne <i>true</i> s'il faut supprimer toute référence au Crossing.
         */
        public boolean removeCrossingItem(CrossingMap.Crossing crossing,CrossingMap.CrossingItem crossingItem);
    }
    
    
    /**
     * Interface qui ne sert à rien d'autres qu'indiquer qu'un objet peut faire partie d'un croisement (<em>Crossing</em>).
     */
    public static interface CrossingItem {
        
    }
    
}
