/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.collections.crossing;

import net.mapeadores.util.collections.crossing.CrossingMap;

/**
 * Cette classe conserver les instances de CrossingEntry qui ont à voir avec un
 * index donné.
 *
 * @author Vincent Calame
 */
public abstract class SameIndexBag {
    private int index;
    
    
    private SameIndexBag(int index) {
        this.index = index;
    }
    
    /**
     *Construit une instance de SameIndexBag en précisant l'index en question
     * et si le croisement porte sur une liste seule croisée avec elle-même (loneList = true) ou sur deux listes différentes.
     */
    public static SameIndexBag newInstance(int index, boolean loneList) {
        if (loneList) {
            return new LoneBag(index);
        } else {
            return new CoupleBag(index);
        }
    }
    
    public int getIndex() {
        return index;
    }
    
    public abstract int getSize(int numeroliste);
    
    public abstract boolean isEmpty();
    
    public abstract CrossingEntry getFirstCrossingEntry(int numeroliste);
    
    /**
     * La velur de numeroliste n'a pas d'importance s'il s'agit d'une liste seule.
     */
    public void addCrossing(int numeroliste,CrossingMap.Crossing crossing,int otherIndex) {
        int size = getSize(numeroliste);
        if (size == 0) {
            setFirstCrossingEntry(numeroliste,new CrossingEntry(crossing,otherIndex,this));
            increaseSize(numeroliste);
        }
        else {
            CrossingEntry iteratingCrossingEntry = getFirstCrossingEntry(numeroliste);
            boolean done = false;
            for (int i = 0; i < size;i++) {
                if (iteratingCrossingEntry.otherIndex == otherIndex) {
                    iteratingCrossingEntry.crossing = crossing;
                    done = true;
                    break;
                }
                CrossingEntry next = iteratingCrossingEntry.next();
                if (next != null) {
                    iteratingCrossingEntry = next;
                }
                else if (i != (size-1)) throw new IllegalStateException("should not occur");
            }
            if (!done) {
                iteratingCrossingEntry.append(crossing,otherIndex);
                increaseSize(numeroliste);
            }
        }
    }
    
    /**
     *
     *@throws IllegalArgumentException si CrossingEntry n'appartient pas au groupe.
     */
    public void removeCrossingEntry(int numeroliste,CrossingEntry crossingEntry) {
        if (crossingEntry.getSameIndexBag() != this) throw new IllegalArgumentException();
        if (crossingEntry.previousCrossingEntry != null) {
            crossingEntry.previousCrossingEntry.nextCrossingEntry = crossingEntry.nextCrossingEntry;
            if (crossingEntry.nextCrossingEntry != null) crossingEntry.nextCrossingEntry.previousCrossingEntry = crossingEntry.previousCrossingEntry;
        }
        else {
            if (crossingEntry.nextCrossingEntry != null) crossingEntry.nextCrossingEntry.previousCrossingEntry = null;
            setFirstCrossingEntry(numeroliste, crossingEntry.nextCrossingEntry);
        }
        decreaseSize(numeroliste);
    }
    
    /**
     * Ne tient pas compte de numeroliste si la liste est seule.
     */
    public abstract int clear(int numeroliste);
    
    
    abstract void increaseSize(int numeroliste);
    
    abstract void decreaseSize(int numeroliste);
    
    abstract void setFirstCrossingEntry(int numeroliste,CrossingEntry crossingEntry);
    
    static private class LoneBag extends SameIndexBag {
        private CrossingEntry startingCrossingEntry;
        private int size = 0;
        
        LoneBag(int index) {
            super(index);
        }
        
        public int getSize(int numeroliste) {
            return size;
        }
        
        public boolean isEmpty() {
            return (startingCrossingEntry == null);
        }
        
        public CrossingEntry getFirstCrossingEntry(int numeroliste) {
            return startingCrossingEntry;
        }
        
        public int clear(int numeroliste) {
            startingCrossingEntry = null;
            int oldSize = size;
            size = 0;
            return oldSize;
        }
        
        void setFirstCrossingEntry(int numeroliste,CrossingEntry crossingEntry) {
            startingCrossingEntry = crossingEntry;
        }
        
        void increaseSize(int numeroliste) {
            size++;
        }
        
        void decreaseSize(int numeroliste) {
            size--;
        }
    }
    
    static private class CoupleBag extends SameIndexBag {
        private CrossingEntry startingCrossingEntry1;
        private CrossingEntry startingCrossingEntry2;
        private int size1 = 0;
        private int size2 = 0;
        
        CoupleBag(int index) {
            super(index);
        }
        
        public int getSize(int numeroliste) {
            switch(numeroliste) {
                case 1 :
                    return size1;
                case 2 :
                    return size2;
                default :
                    throw new IllegalArgumentException("wrong value " + numeroliste);
            }
        }
        
        public boolean isEmpty() {
            return ((startingCrossingEntry1 == null) && (startingCrossingEntry2 == null));
        }
        
        public CrossingEntry getFirstCrossingEntry(int numeroliste) {
            switch(numeroliste) {
                case 1 :
                    return startingCrossingEntry1;
                case 2 :
                    return startingCrossingEntry2;
                default :
                    throw new IllegalArgumentException("wrong value " + numeroliste);
            }
        }
        
        public int clear(int numeroliste) {
            int oldSize = 0;
            switch(numeroliste) {
                case 1 :
                    startingCrossingEntry1 = null;
                    oldSize = size1;
                    size1 = 0;
                    break;
                case 2 :
                    startingCrossingEntry2 = null;
                    oldSize = size2;
                    size2 = 0;
                    break;
                default :
                    throw new IllegalArgumentException("wrong value " + numeroliste);
            }
            return oldSize;
        }
        
        void setFirstCrossingEntry(int numeroliste,CrossingEntry crossingEntry) {
            switch(numeroliste) {
                case 1 :
                    startingCrossingEntry1 = crossingEntry;
                    break;
                case 2 :
                    startingCrossingEntry2 = crossingEntry;
                    break;
                default :
                    throw new IllegalArgumentException("wrong value " + numeroliste);
            }
        }
        
        void increaseSize(int numeroliste) {
            switch(numeroliste) {
                case 1 :
                    size1++;
                    break;
                case 2 :
                    size2++;
                    break;
                default :
                    throw new IllegalArgumentException("wrong value " + numeroliste);
            }
            
        }

        void decreaseSize(int numeroliste) {
            switch(numeroliste) {
                case 1 :
                    size1--;
                    break;
                case 2 :
                    size2--;
                    break;
                default :
                    throw new IllegalArgumentException("wrong value " + numeroliste);
            }
        }
    }
}
