/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.collections.multilist;

import net.mapeadores.util.primitives.BitmaskUtils;

import java.util.List;
import java.util.AbstractList;
import java.util.RandomAccess;

/**
 *
 * @author Vincent Calame
 */
public final class MultiList {
    private MultiListItem[] primaryArray = new MultiListItem[16];
    private int primarySize = 0;
    private MultiListItem[][] secondaryArrays;
    private int[] secondarySizes;
    private InternalList itemPrimaryList = new InternalList(-1,false);
    private InternalList[] itemSecondaryLists;
    private InternalList wrappedObjectPrimaryList = new InternalList(-1,true);
    private InternalList[] wrappedObjectSecondaryLists;
    
    public MultiList(int secondaryListCount) {
        secondaryArrays = new MultiListItem[secondaryListCount][16];
        secondarySizes = new int[secondaryListCount];
        itemSecondaryLists = new InternalList[secondaryListCount];
        wrappedObjectSecondaryLists = new InternalList[secondaryListCount];
        for(int i = 0; i < secondaryListCount;i++) {
            itemSecondaryLists[i] = new InternalList(i,false);
            wrappedObjectSecondaryLists[i] = new InternalList(i,true);
        }
    }
    
    public int getPrimarySize() {
        return primarySize;
    }
    
    public int getSecondarySize(int p) {
        return secondarySizes[p];
    }
    
    public int getSecondaryListCount() {
        return secondarySizes.length;
    }
    
    public List getItemPrimaryList() {
        return itemPrimaryList;
    }
    
    public List getItemSecondaryList(int p) {
        return itemSecondaryLists[p];
    }
    
    public List getWrappedObjectPrimaryList() {
        return wrappedObjectPrimaryList;
    }
    
    public List getWrappedObjectSecondaryList(int p) {
        return wrappedObjectSecondaryLists[p];
    }
    
    public void add(MultiListItem multiListItem) {
        if (!multiListItem.isCleared()) throw new IllegalArgumentException("multiListItem is not cleared");
        multiListItem.setMultiList(this);
        if (primarySize == primaryArray.length) {
            MultiListItem[] ol = new MultiListItem[primarySize*2];
            System.arraycopy(primaryArray,0, ol,0, primarySize);
            primaryArray = ol;
        }
        multiListItem.setPrimaryIndex(primarySize);
        primaryArray[primarySize] = multiListItem;
        primarySize++;
        int mask = multiListItem.getMask();
        for (int i = 0 ; i < secondaryArrays.length;i++) {
            if (BitmaskUtils.booleanValueAt(mask,i)) {
                addInSecondaryList(multiListItem, i);
            } else multiListItem.setSecondaryIndex(i, -1);
        }
    }
    
    public void remove(MultiListItem multiListItem) {
        int oldindex = multiListItem.getPrimaryIndex();
        int mask = multiListItem.getMask();
        for (int i = oldindex+1;i < primarySize;i++) {
            MultiListItem next = primaryArray[i];
            primaryArray[i-1] = next;
            next.decreasePrimaryIndex();
        }
        if (mask > 0) {
            for (int i = 0;i < secondaryArrays.length;i++) {
                if (BitmaskUtils.booleanValueAt(mask,i)) {
                    removeInSecondaryList(multiListItem, i,false);
                }
            }
        }
        multiListItem.clear();
        primarySize--;
        primaryArray[primarySize] =null;
    }
    
    public void removeAllInSecondaryList(int p) {
        List list = itemSecondaryLists[p];
        int size = list.size();
        for(int i = (size-1);i >= 0;i--) {
            MultiListItem multiListItem = (MultiListItem) list.get(i);
            remove(multiListItem);
        }
    }    
    
    public boolean translate(MultiListItem multiListItem,int newIndex) {
        if ((newIndex < 0) || (newIndex >= primarySize)) throw new IndexOutOfBoundsException(String.valueOf(newIndex));
        int oldIndex = multiListItem.getPrimaryIndex();
        if (oldIndex  == newIndex) return false;
        int mask = multiListItem.getMask();
        if (mask > 0) {
            for (int p = 0;p < secondaryArrays.length;p++) {
                if (BitmaskUtils.booleanValueAt(mask,p)) {
                    int oldidx = multiListItem.getSecondaryIndex(p);
                    int nwidx = getNewSecondaryIndex(p,oldIndex, newIndex);
                    if (nwidx != oldidx) translateInSecondaryList(multiListItem, p, nwidx);
                }
            }
        }
        if (oldIndex < newIndex) {
            for (int i = oldIndex;i< newIndex;i++) {
                MultiListItem o = primaryArray[i+1];
                primaryArray[i] = o;
                o.decreasePrimaryIndex();
            }
        } else {
            for (int i = oldIndex;i> newIndex;i--) {
                
                MultiListItem o = primaryArray[i-1];
                primaryArray[i] = o;
                o.increasePrimaryIndex();
            }
        }
        primaryArray[newIndex] = multiListItem;
        multiListItem.setPrimaryIndex(newIndex);
        return true;
    }
    
    public int[] disable(MultiListItem multiListItem,int mask) {
        int[] result = new int[secondaryArrays.length];
        for (int p = 0;p < secondaryArrays.length;p++) {
                if (BitmaskUtils.booleanValueAt(mask,p)) {
                    int oldidx = multiListItem.getSecondaryIndex(p);
                    if (oldidx != -1) {
                        removeInSecondaryList(multiListItem, p,true);
                        result[p] = oldidx;
                    }
                    else result[p] = -1;
                }
                else {
                    result[p] = -1;
                }
            }
        return result;
    }
    
     public int[] enable(MultiListItem multiListItem,int mask) {
         int primaryIndex = multiListItem.getPrimaryIndex();
        int[] result = new int[secondaryArrays.length];
        for (int p = 0;p < secondaryArrays.length;p++) {
                if (BitmaskUtils.booleanValueAt(mask,p)) {
                    int oldidx = multiListItem.getSecondaryIndex(p);
                    if (oldidx == -1) {
                        enableInSecondaryList(multiListItem,p);
                        result[p] = multiListItem.getSecondaryIndex(p);
                    }
                    else result[p] = -1;
                }
                else {
                    result[p] = -1;
                }
            }
        return result;
    }
    
    private int getNewSecondaryIndex(int p,int oldIndex,int newIndex) {
        MultiListItem[] array = secondaryArrays[p];
        if (oldIndex < newIndex) {
            for (int i = newIndex;i >= oldIndex;i--) {
                MultiListItem o = primaryArray[i];
                int idx = o.getSecondaryIndex(p);
                if (idx != -1) return idx;
            }
        } else {
            for (int i = newIndex;i <= oldIndex;i++) {
                MultiListItem o = primaryArray[i];
                int idx = o.getSecondaryIndex(p);
                if (idx != -1) return idx;
            }
        }
        throw new IllegalStateException("should not occur");
    }
    
    private void translateInSecondaryList(MultiListItem objectInList,int p, int newIndex) {
        MultiListItem[] array = secondaryArrays[p];
        int oldIndex = objectInList.getSecondaryIndex(p);
        if (oldIndex < newIndex) {
            for (int i = oldIndex;i< newIndex;i++) {
                MultiListItem o = array[i+1];
                array[i] = o;
                o.decreaseSecondaryIndex(p);
            }
        } else {
            for (int i = oldIndex;i> newIndex;i--) {
                MultiListItem o = array[i-1];
                array[i] = o;
                o.increaseSecondaryIndex(p);
            }
        }
        array[newIndex] = objectInList;
        objectInList.setSecondaryIndex(p,newIndex);
    }
    
    private void addInSecondaryList(MultiListItem objectInList,int p) {
        MultiListItem[] array = secondaryArrays[p];
        int size = secondarySizes[p];
        if (size == array.length) {
            MultiListItem[] ol = new MultiListItem[size*2];
            System.arraycopy(array,0, ol,0,size);
            array = ol;
            secondaryArrays[p] = array;
        }
        array[size] = objectInList;
        objectInList.setSecondaryIndex(p,size);
        secondarySizes[p]++;
    }
    
    private void removeInSecondaryList(MultiListItem objectInList,int p, boolean maskchange) {
        int oldindex = objectInList.getSecondaryIndex(p);
        if (oldindex == -1) throw new IllegalStateException("should not occur");
        MultiListItem[] array = secondaryArrays[p];
        int size = secondarySizes[p];
        for (int i = oldindex+1;i < size;i++) {
            MultiListItem next = array[i];
            array[i-1] = next;
            next.decreaseSecondaryIndex(p);
        }
        if (maskchange) objectInList.setSecondaryIndex(p, -1);
        secondarySizes[p]--;
        array[size-1] = null;
    }
     
    private void enableInSecondaryList(MultiListItem multiListItem,int p) {
        MultiListItem[] array = secondaryArrays[p];
        int secondarySize = secondarySizes[p];
        int primaryIndex = multiListItem.getPrimaryIndex();
        int newSecondaryIndex = 0;
        for (int i = 0; i < secondarySize;i++) {
            if (array[i].getPrimaryIndex() < primaryIndex) {
                newSecondaryIndex++;
            }
            else break;
        }
        if (secondarySize == array.length) {
            MultiListItem[] ol = new MultiListItem[secondarySize*2];
            System.arraycopy(array,0, ol,0,secondarySize);
            array = ol;
            secondaryArrays[p] = array;
        }
        if (newSecondaryIndex < secondarySize) {
            for (int i = secondarySize; i > newSecondaryIndex; i--) {
                MultiListItem previous = array[i-1];
                array[i] = previous;
                previous.increaseSecondaryIndex(p);
            }
        }
        array[newSecondaryIndex] = multiListItem;
        multiListItem.setSecondaryIndex(p,newSecondaryIndex);
        secondarySizes[p]++;
    }
    
    class InternalList extends AbstractList implements RandomAccess {
        private int listIndex;
        boolean wrappedObject;
        
        InternalList(int listIndex,boolean wrappedObject) {
            this.listIndex = listIndex;
            this.wrappedObject = wrappedObject;
        }
        
        public int size() {
            if (listIndex == -1) return primarySize;
            else return secondarySizes[listIndex];
        }
        
        public Object get(int index) {
            if (index < 0) throw new IndexOutOfBoundsException("index < 0");
            if (listIndex == -1) {
                if (index >= primarySize) throw new IndexOutOfBoundsException("index >= size()");
                return (wrappedObject)?primaryArray[index].getWrappedObject():primaryArray[index];
            }
            else {
                if (index >= secondarySizes[listIndex]) throw new IndexOutOfBoundsException("index >= size()");
                return (wrappedObject)?secondaryArrays[listIndex][index].getWrappedObject():secondaryArrays[listIndex][index];
            }
        }
    }
}
