/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections.xml;

import net.mapeadores.util.xml.XMLUtils;
import java.io.*;
import java.util.*;
import org.w3c.dom.*;

/**
 * Cet objet sert à conserver le croisement entre des ids de deux ensembles différents désignés par des noms. Ces croisements sont qualifiés par un entier.
 * Idscroises peut être utilisé pour reconstituer un objet de type BitmaskCrossing.
 *
 * @author  Vincent Calame
 */
public final class Idscroises implements Serializable {
    String nom1;
    String nom2;
    List listeCrx = new LinkedList();
    
    /** Creates a new instance of Idscroises */
    public Idscroises(String nom1, String nom2) {
        this.nom1 = nom1;
        this.nom2 = nom2;
    }
    
    /* Constructeur JDom désactivé
    public Idscroises(org.jdom.Element element_jdom) {
        nom1 = element_jdom.getAttributeValue("nom1");
        if (nom1 == null) nom1 = "";
        nom2 = element_jdom.getAttributeValue("nom2");
        if (nom2 == null) nom2 = "";
        List list = element_jdom.getChildren("crx");
        for (Iterator it = list.iterator();it.hasNext();) {
            org.jdom.Element el = (org.jdom.Element) it.next();
            try {
                Crx crx = new Crx(el);
                listeCrx.add(crx);
            }
            catch(NumberFormatException nfe) {
                
            }
            catch(NullPointerException npe) {
                
            }
            
        }
    }
    */
    
    public Idscroises(org.w3c.dom.Element element_xml) {
        nom1 = element_xml.getAttribute("nom1");
        nom2 = element_xml.getAttribute("nom2");
        org.w3c.dom.NodeList nodelist = element_xml.getChildNodes();
        for (int i = 0; i < nodelist.getLength();i++) {
            org.w3c.dom.Node node = nodelist.item(i);
            if (node instanceof org.w3c.dom.Element) {
                try {
                    Crx crx = new Crx((org.w3c.dom.Element)node);
                    listeCrx.add(crx);
                }
                catch(NumberFormatException nfe) {
                    
                }
            }
        }
    }
    
    /**
     * Retourne le nom du premier ensemble.
     */
    public String getNom1() {
        return nom1;
    }
    
    /**
     * Retourne le nom du second ensemble.
     */
    public String getNom2() {
        return nom2;
    }
    
    /**
     * Ajoute le croisement entreRetourne le nom du premier ensemble.
     */
    public void appendCrx(int id1,int id2, int pds) {
        listeCrx.add(new Crx(id1,id2,pds));
    }
    
    /**
     * Retourne la liste (non modifiable) des objets Crx.
     */
    public List getCrxList() {
        return Collections.unmodifiableList(listeCrx);
    }
    
    
    /**
     * Écrit l'objet au format XML dans le fichier f. Si <code>smartPrinting</code> est égal à <i>true</i>, des sauts de lignes sont insérés.
     */
    public void writeXML(File f, boolean smartPrinting) throws IOException {
        writeXML(new FileOutputStream(f),smartPrinting);
    }
    
    /**
     * Écrit l'objet au format XML dans l'outputstream. Si <code>withDeclaration</code> est égal à <i>true</i>, &lt;?xml version='1.0' encoding='UTF-8'?&gt; est ajouté au début.
     * Si <code>smartPrinting</code> est égal à <i>true</i>, des sauts de lignes sont insérés.
     */
    public void writeXML(OutputStream output, boolean smartPrinting) throws IOException {
        writeXML(new OutputStreamWriter(output,"UTF-8"),true,smartPrinting);
    }
    
    /**
     * Écrit l'objet au format XML dans le writer. Si <code>withDeclaration</code> est égal à <i>true</i>, &lt;?xml version='1.0' encoding='UTF-8'?&gt; est ajouté au début.
     * Si <code>smartPrinting</code> est égal à <i>true</i>, des sauts de lignes sont insérés.
     */
    public void writeXML(Writer writer,boolean withDeclaration, boolean smartPrinting) throws IOException {
        BufferedWriter buf = new BufferedWriter(writer);
        if (withDeclaration) buf.write("<?xml version='1.0' encoding='UTF-8'?>");
        if (smartPrinting) buf.write("\n");
        buf.write("<idscroises nom1=\"");
        XMLUtils.escapeXML(nom1,buf);
        buf.write("\" nom2=\"");
        XMLUtils.escapeXML(nom2,buf);
        buf.write("\">");
        for(Iterator it = listeCrx.iterator();it.hasNext();) {
            Crx crx = (Crx) it.next();
            crx.writeXML(buf,smartPrinting);
        }
        if (smartPrinting) buf.write("\n");
        buf.write("</idscroises>");
        buf.close();
    }
    
   /**
    * Cette classe interne représente le croisement entre deux ids.
    */ 
   static public final class Crx implements Serializable {
       int id1;
       int id2;
       int pds;
       
       Crx(int id1,int id2,int pds) {
           this.id1 = id1;
           this.id2 = id2;
           this.pds = pds;
       }
       
       /*
        * voir la note du constructeur ci-dessus
       Crx(org.jdom.Element element_jdom) throws NumberFormatException{
           id1 = Integer.parseInt(element_jdom.getAttributeValue("id1"));
           id2 = Integer.parseInt(element_jdom.getAttributeValue("id2"));
           String strg_pds = element_jdom.getAttributeValue("pds");
           if ((strg_pds == null) || (strg_pds.length() == 0)) {
               pds = 1;
           }
           else {
               pds =  Integer.parseInt(strg_pds);
           }
       }
        */
       
       Crx(org.w3c.dom.Element element) throws NumberFormatException {
           id1 = Integer.parseInt(element.getAttribute("id1"));
           id2 = Integer.parseInt(element.getAttribute("id2"));
           String strg_pds = element.getAttribute("pds");
           if (strg_pds.length() == 0) {
               pds = 1;
           }
           else {
               pds =  Integer.parseInt(strg_pds);
           }
       }
       
       /**
        * Retourne la valeur du premier identifiant.
        */
       public int getId1() {
           return id1;
       }
       
       /**
        * Retourne la valeur du second identifiant.
        */
       public int getId2() {
           return id2;
       }
       
       /**
        * Retourne la valeur du pôids.
        */
       public int getPds() {
           return pds;
       }
       
       void writeXML(Writer writer,boolean smartPrinting) throws IOException{
           if (smartPrinting) writer.write("\n\t");
           writer.write("<crx id1=\"");
           writer.write(String.valueOf(id1));
           writer.write("\" id2=\"");
           writer.write(String.valueOf(id2));
           writer.write("\" pds=\"");
           writer.write(String.valueOf(pds));
           writer.write("\"/>");
       }
   }
}
