/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.html;

import java.util.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.logging.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class HTMLPage extends HTMLPrinter {
    public static final String COMMANDKEY_DIVCLASS="d_GLB_CommandKey";
    public static final String SUBMIT_DIVCLASS="d_GLB_Submit";
    public static final String ERRORMESSAGE_SPANCLASS="s_GLB_ErrorMessage";
    public static final String DONEMESSAGE_SPANCLASS="s_GLB_DoneMessage";
    private final static HTMLAttributes SUBMITDIV_ATTR = new HTMLAttributes();
    static {
        SUBMITDIV_ATTR.setClasses(SUBMIT_DIVCLASS);
    }
    private String htmlpage_title;
    private HTMLAttributes bodyHTMLAttributes;
    private String[] jsUrlArray;
    private ResourceLocalisation commandKeyLocalisation;
    private CommandBoxParameters commandBoxParameters = new CommandBoxParameters();
    {
        commandBoxParameters.getCommandKeyDivAttributes().setClasses(COMMANDKEY_DIVCLASS);
        commandBoxParameters.getSubmitParameters().getDivAttributes().setClasses(SUBMIT_DIVCLASS);
    }
    private HTMLAttributes commandFormAttributes;
    private boolean onCommandBox = false;
    private String currentCommandKey = null;
    private List cssUrlList = new ArrayList();
    private String iconPng = null;
    
    
    public HTMLPage() {
    }
    
    
    public void setCommandKeyLocalisation(ResourceLocalisation commandKeyLocalisation) {
        this.commandKeyLocalisation = commandKeyLocalisation;
    }
    
    public ResourceLocalisation getCommandKeyLocalisation() {
        return commandKeyLocalisation;
    }
    
    public CommandBoxParameters getCommandBoxParameters() {
        return commandBoxParameters;
    }
    
    
    public void setTitle(String title) {
        this.htmlpage_title = title;
    }
    
    public void localizeTitle(String key, Object value) {
        this.htmlpage_title = getLocalization(key,value);
        if (htmlpage_title == null) htmlpage_title = key;
    }
    
    public void setBodyHTMLAttributes(HTMLAttributes bodyHTMLAttributes) {
        this.bodyHTMLAttributes = bodyHTMLAttributes;
    }
    
    public void setJsUrlArray(String[] jsUrlArray) {
        this.jsUrlArray = jsUrlArray;
    }
    
    public void addCssUrl(String cssUrl) {
        cssUrlList.add(cssUrl);
    }
    
    public void start(int langInteger) {
        appendTransitionalDocType();
        startHtml(langInteger);
        startHead();
        startTitle();
        escape(htmlpage_title);
        endTitle();
        append("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">");
        breakLine();
        if (jsUrlArray != null) {
            append("<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">");
            breakLine();
        }
        for(Iterator it = cssUrlList.iterator(); it.hasNext();) {
            String cssUrl = (String) it.next();
            append("<link href=\"");
            append(cssUrl);
            append("\" rel=\"stylesheet\" type=\"text/css\">");
            breakLine();
        }
        if (iconPng != null) {
            append("<link href=\"");
            append(iconPng);
            append("\" rel=\"icon\" type=\"image/png\">");
            breakLine();
        }
        if (jsUrlArray != null) {
            HTMLAttributes scriptAttributes = new HTMLAttributes();
            scriptAttributes.setType("text/javascript");
            for(int i = 0; i < jsUrlArray.length;i++) {
                scriptAttributes.setSrc(jsUrlArray[i]);
                startScript(scriptAttributes);
                endScript();
                breakLine();
            }
        }
        endHead();
        startBody(bodyHTMLAttributes);
    }
    
    public void startForm(HTMLAttributes formAttributes,String commandString) {
        startForm(formAttributes);
        appendHiddenInput("cmd",commandString);
    }
    
    public void startCommandBox(String commandString, String commandKey) {
        startCommandBox(commandString, new ResourceItem(commandKey), null);
    }
    
    public void startCommandBox(String commandString, String commandKey,Map hiddenAttributesMap) {
        startCommandBox(commandString, new ResourceItem(commandKey),hiddenAttributesMap);
    }
    
    public void startCommandBox(String commandString, ResourceItem commandResourceItem,Map hiddenAttributesMap) {
        if (onCommandBox) throw new IllegalStateException("last CommandBox is not ended");
        onCommandBox = true;
        startForm(commandBoxParameters.getFormAttributes(), commandString);
        if (hiddenAttributesMap != null) {
            for(Iterator it = hiddenAttributesMap.entrySet().iterator(); it.hasNext();) {
                Map.Entry entry = (Map.Entry) it.next();
                appendHiddenInput((String) entry.getKey(), (String) entry.getValue());
            }
        }
        startFieldset(commandBoxParameters.getFieldsetAttributes());
        if (commandResourceItem != null) {
            startLegend(commandBoxParameters.getLegendAttributes());
            escape(commandKeyLocalisation.toString(commandResourceItem));
            endLegend();
            currentCommandKey = commandResourceItem.getResourceKey();
        }
    }
    
    public void endCommandBox(String submitKey) {
        if (!onCommandBox) throw new IllegalStateException("no CommandBox started yet");
        if (submitKey != null) {
            SubmitParameters submitParameters = commandBoxParameters.getSubmitParameters();
            addSubmit(new ResourceItem(submitKey),submitParameters);
        }
        if (currentCommandKey != null) {
            startDiv(commandBoxParameters.getCommandKeyDivAttributes());
            append(currentCommandKey);
            endDiv();
        }
        endFieldset();
        endForm();
        currentCommandKey = null;
        onCommandBox = false;
    }
    
    
    public void end() {
        if (onCommandBox) throw new IllegalStateException("last CommandBox is not ended");
        endBody();
        endHtml();
    }
    
    public void addSubmit(String submitKey, boolean encloseInDiv) {
        SubmitParameters submitParameters = new SubmitParameters();
        submitParameters.setEncloseInDiv(encloseInDiv);
        addSubmit(new ResourceItem(submitKey),submitParameters);
    }
    
    public void addSubmit(ResourceItem submitResourceItem, SubmitParameters submitParameters) {
        boolean encloseInDiv = submitParameters.isEncloseInDiv();
        if (encloseInDiv) {
            HTMLAttributes divAttr = submitParameters.getDivAttributes();
            if (divAttr == null) divAttr = SUBMITDIV_ATTR;
            startDiv(divAttr);
        }
        HTMLAttributes inputAttr = submitParameters.getInputAttributes();
        if (inputAttr == null) {
            inputAttr = new HTMLAttributes();
        } else {
            inputAttr = inputAttr.copy();
        }
        if (submitResourceItem != null) {
            inputAttr.setValue(getLocalization(submitResourceItem));
        }
        inputAttr.setType("submit");
        appendInput(inputAttr);
        if (encloseInDiv) {
            endDiv();
        }
    }
    
    public void printMessage(CommandMessage message) {
        startP();
        startSpan(getMessageSpanClass(message));
        localize(message.getKeyResourceItem());
        endSpan();
        if (message.getType() == CommandMessage.MULTIERROR_MESSAGE) {
            List list = message.getMultiErrorList();
            startSpan(ERRORMESSAGE_SPANCLASS);
            startSmall();
            for(Iterator it = list.iterator();it.hasNext();) {
                appendBr();
                localize((ResourceItem)it.next());
            }
            endSmall();
            endSpan();
        } else if (message.getType() == CommandMessage.RESULTLOG_MESSAGE) {
            ProcessLog processLog = message.getProcessLog();
            if (processLog.hasLog()) {
                appendBr();
                startPre();
                escape(ProcessLogUtils.toString(processLog),true);
                endPre();
            }
        }
        endP();
    }
    
    public static String getMessageSpanClass(CommandMessage commandMessage) {
        switch(commandMessage.getType()) {
            case CommandMessage.ERROR_MESSAGE :
            case CommandMessage.MULTIERROR_MESSAGE :
                return ERRORMESSAGE_SPANCLASS;
            default :
                return DONEMESSAGE_SPANCLASS;
        }
    }
    
    public void printColon() {
        append("&nbsp;: ");
    }
    
    public void printLibColon(String libKey) {
        localize(libKey);
        printColon();
    }
    
    public void printLibCells(String libKey,boolean topAlign) {
        if (topAlign) {
            startCellWithTopAlign();
        } else {
            startCell();
        }
        localize(libKey);
        endCell();
        if (topAlign) {
            startCellWithTopAlign();
        } else {
            startCell();
        }
        append("&nbsp;:&nbsp;");
        endCell();
    }
    
    public void printLitteralLibCells(String value,boolean topAlign) {
        if (topAlign) {
            startCellWithTopAlign();
        } else {
            startCell();
        }
        escape(value);
        endCell();
        if (topAlign) {
            startCellWithTopAlign();
        } else {
            startCell();
        }
        append("&nbsp;:&nbsp;");
        endCell();
    }

    public void setIconPng(String iconPng) {
        this.iconPng = iconPng;
    }
    
}
