/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.langinteger;

import java.util.*;
import net.mapeadores.util.collections.*;
import java.io.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.common.*;
import net.mapeadores.util.primitives.*;

/**
 *
 * @author Vincent Calame
 */
public class LocaleItem {
    private static final short LANG_RESOURCE = 1;
    private static final short COUNTRY_RESOURCE = 2;
    static LocaleItem defaultLocaleItem;
    private Locale locale;
    private TreeMap languageSortedMap = new TreeMap();
    private TreeMap countrySortedMap = new TreeMap();
    private SortedMap unmodifiableLanguageSortedMap = Collections.unmodifiableSortedMap(languageSortedMap);
    private SortedMap unmodifiableCountrySortedMap = Collections.unmodifiableSortedMap(countrySortedMap);
    private IntHashMap codeMap = new IntHashMap();
    private SortedCollatedKeyMap collatedKeyMap;
    private HashMap geoDirectionMap = new HashMap();
    
    /**
     * Appelé quand le fichier existe dans le logiciel
     */
    LocaleItem(String langString) {
        this.locale = new Locale(langString);
        initResource(langString,LANG_RESOURCE);
        initResource(langString, COUNTRY_RESOURCE);
        if (defaultLocaleItem == null) {
            defaultLocaleItem = this;
        }
        checkResourceValues(langString);
        initGeoDirection(langString);
        collate();
    }
    
    /**
     * Fait appel aux données de Java
     */
    LocaleItem(Locale locale) {
        this.locale = locale;
        initGeoDirection(locale.getLanguage());
        initFromJRE();
        collate();
    }
    
    public Locale getLocale() {
        return locale;
    }
    
    public SortedMap getLanguageSortedMap() {
        return unmodifiableLanguageSortedMap;
    }
    
    public SortedMap getCountrySortedMap() {
        return unmodifiableCountrySortedMap;
    }
    
    public String getCardinalDirection(short direction, boolean onlyInitiale) {
        String initiale = CommonUtils.getGeoCode(direction);
        String key = null;
        if (onlyInitiale) {
            key = initiale + "i";
        }
        else {
            key = initiale + "t";
        }
        String lib = (String) geoDirectionMap.get(key);
        if (lib == null) return initiale;
        return lib;
    }
    
    
    
    public String getLangLibelle(int langInteger, boolean onlyMain) {
        if (langInteger >= 0) return "";
        Variante variante = (Variante) codeMap.get(langInteger);
        if (variante != null) {
            if (onlyMain) return variante.getMainString();
            else return variante.getSourceString();
        }
        int countryInteger = LangInteger.toCountryInteger(langInteger);
        if (countryInteger == LangInteger.NO_LINGUISTIC_CONTENT) {
            return LangInteger.toString(langInteger) + "?";
        } else {
            int langPart = LangInteger.getLangPart(langInteger);
            variante = (Variante) codeMap.get(langPart);
            String lib = " (" + getCountryLibelle(countryInteger, true) + ")";
            if (variante == null) {
                return LangInteger.toString(langPart) + "?" + lib;
            } else {
                return variante.getMainString() + lib;
            }
        }
    }
    
    public String getCountryLibelle(int langInteger, boolean onlyMain) {
        int countryInteger = LangInteger.toCountryInteger(langInteger);
        if (countryInteger == LangInteger.NO_LINGUISTIC_CONTENT) return "";
        Variante variante = (Variante) codeMap.get(countryInteger);
        if (variante == null) return LangInteger.getCountry(countryInteger) + "?";
        if (onlyMain) return variante.getMainString();
        else return variante.getSourceString();
    }
    
    public Integer getLangInteger(String lib) {
        return (Integer) collatedKeyMap.getValue(lib);
    }
    
    public int[] getLangIntegerArray(String searchString, int type) {
        List searchResultItemList = collatedKeyMap.search(searchString,type);
        int size = searchResultItemList.size();
        LinkedHashSet resultSet = new LinkedHashSet();
        for(int i = 0; i < size;i++) {
            SearchResultItem entry = (SearchResultItem) searchResultItemList.get(i);
            Integer itg = (Integer) entry.getValue();
            int langInteger = itg.intValue();
            if (LangInteger.getLangPart(langInteger) != LangInteger.NO_LINGUISTIC_CONTENT) {
                resultSet.add(itg);
            }
        }
        size = resultSet.size();
        int[] result = new int[size];
        int p =0;
        for(Iterator it = resultSet.iterator(); it.hasNext();) {
            result[p] = ((Integer) it.next()).intValue();
            p++;
        }
        return result;
    }
    
    public int[] getCountryIntegerArray(String searchString, int type) {
        List resultItemList = collatedKeyMap.search(searchString,type);
        int size = resultItemList.size();
        
        LinkedHashSet resultSet = new LinkedHashSet();
        for(int i = 0; i < size;i++) {
            SearchResultItem entry = (SearchResultItem) resultItemList.get(i);
            Integer itg = (Integer) entry.getValue();
            int langInteger = itg.intValue();
            if ((LangInteger.getLangPart(langInteger) == LangInteger.NO_LINGUISTIC_CONTENT) && (langInteger != LangInteger.NO_LINGUISTIC_CONTENT)) {
                resultSet.add(itg);
            }
            
        }
        size = resultSet.size();
        int[] result = new int[size];
        int p =0;
        for(Iterator it = resultSet.iterator(); it.hasNext();) {
            result[p] = ((Integer) it.next()).intValue();
            p++;
        }
        return result;
    }
    
   
    private void initResource(String l10nString, short type) {
        String prefix = null;
        switch(type) {
            case LANG_RESOURCE :
                prefix = "lang";
                break;
            case COUNTRY_RESOURCE :
                prefix = "country";
                break;
            default :
                throw new SwitchException("type = " + type);
        }
        try {
            InputStream is = getClass().getResourceAsStream("l10n/" + prefix + "_" + l10nString + ".csv");
            if (is == null) return;
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is,"UTF-8"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx = ligne.indexOf(',');
                    if (idx != -1) {
                        String codeString = ligne.substring(0, idx);
                        int codeInteger = 0;
                        switch(type) {
                            case LANG_RESOURCE :
                                codeInteger = LangInteger.parse(codeString, true);
                                break;
                            case COUNTRY_RESOURCE :
                                codeInteger = CountryUtils.parseCountryCode(codeString);
                                break;
                        }
                        putCode(codeInteger, ligne.substring(idx+1).trim(), type);
                    }
                }
            }
        } catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        } catch(LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
    }
    
    private void putCode(int codeInteger,String lib, short type) {
        Variante variante = new Variante(codeInteger,lib);
        String codeString = null;
        Map map = null;
        switch(type) {
            case LANG_RESOURCE :
                codeString = LangInteger.toString(codeInteger);
                map = languageSortedMap;
                break;
            case COUNTRY_RESOURCE :
                codeString =LangInteger.getCountry(codeInteger);
                map = countrySortedMap;
                break;
            default :
                throw new SwitchException("type = " + type);
        }
        codeMap.put(codeInteger,variante);
        map.put(codeString, variante.getSourceString());
    }
    
    private void checkResourceValues(String langString) {
        HashSet countrySet = new HashSet(CountryUtils.getIsoCodeSet());
        for(Iterator it = countrySortedMap.keySet().iterator(); it.hasNext();) {
            String code = (String) it.next();
            boolean done = countrySet.remove(code);
            if (!done) {
                throw new InternalResourceException("code : " + code + " présent dans l10n/country_" + langString + ".csv absent de iso31166.csv et iso3166-complement.csv");
            }
        }
        if (!countrySet.isEmpty()) {
            for(Iterator it = countrySet.iterator(); it.hasNext();) {
                String code = (String) it.next();
                System.out.println("code : " + code + " absent de l10n/country_" + langString + ".csv");
                try {
                    int codeInteger = CountryUtils.parseCountryCode(code);
                    putCode(codeInteger, code, COUNTRY_RESOURCE);
                }
                catch(LangIntegerException lie) {
                    throw new ShouldNotOccurException(lie);
                }
            }
        }
        HashSet langSet = new HashSet(LangUtils.getIsoMixedCodeSet());
        for(Iterator it = languageSortedMap.keySet().iterator(); it.hasNext();) {
            String code = (String) it.next();
            boolean done = langSet.remove(code);
            if (!done) {
                System.out.println("manque le code iso :" + code);
            }
        }
        if (!langSet.isEmpty()) {
            for(Iterator it = langSet.iterator(); it.hasNext();) {
                String code = (String) it.next();
                System.out.println("code : " + code + " absent de l10n/lang_" + langString + ".csv");
                try {
                    int codeInteger = LangInteger.parse(code, true);
                    putCode(codeInteger, code, LANG_RESOURCE);
                }
                catch(LangIntegerException lie) {
                    throw new ShouldNotOccurException(lie);
                }
            }
        }
        
    }
    
    private void initFromJRE() {
        for(Iterator it = defaultLocaleItem.codeMap.keySet().iterator();it.hasNext();) {
            int codeInteger = ((Integer)it.next()).intValue();
            Variante variante = (Variante) defaultLocaleItem.codeMap.get(codeInteger);
            codeMap.put(codeInteger, variante);
            String codeString = null;
            Map map = null;
            if (LangInteger.isCountryInteger(codeInteger)) {
                codeString = LangInteger.getCountry(codeInteger);
                map = countrySortedMap;
            }
            else {
                codeString = LangInteger.toString(codeInteger);
                map = languageSortedMap;
            }
            map.put(codeString,variante.getSourceString());
        }
        
        String[] codes = Locale.getISOLanguages();
        for (int i = 0; i < codes.length;i++) {
            String code = codes[i];
            Locale loc = new Locale(code,"");
            String lib = loc.getDisplayLanguage(locale);
            try {
                int langInteger = LangInteger.parse(code);
                putCode(langInteger,lib, LANG_RESOURCE);
            } catch(LangIntegerException lie) {
                throw new InternalResourceException(lie);
            }
        }
        String[] codesCountry = Locale.getISOCountries();
        for (int i = 0; i < codesCountry.length;i++) {
            String code = codesCountry[i].intern();
            Locale loc = new Locale("",code);
            try {
                int countryCode = CountryUtils.parseCountryCode(code);
                putCode(countryCode, loc.getDisplayCountry(locale),COUNTRY_RESOURCE);
            } catch(LangIntegerException lie) {
                throw new InternalResourceException(lie);
            }
        }
    }
    
   
    private void collate() {
        collatedKeyMap = new SortedCollatedKeyMap(locale);
        for(Iterator it = codeMap.keySet().iterator();it.hasNext();) {
            Integer itg = (Integer)it.next();
            Variante variante = (Variante) codeMap.get(itg.intValue());
            collatedKeyMap.putValue(variante.getMainString(),itg);
            int count = variante.getVarianteCount();
            for (int i = 0; i < count;i++) {
                collatedKeyMap.putValue(variante.getVariante(i),itg);
            }
        }
    }
    
    private class Variante {
        String sourceString;
        String mainString;
        String[] variantes;
        
        Variante(int langInteger,String sourceString) {
            this.sourceString = sourceString.trim();
            parse(langInteger);
        }
        
        public String getSourceString() {
            return sourceString;
        }
        
        public String getMainString() {
            return mainString;
        }
        
        public int getVarianteCount() {
            if (variantes == null) return 0;
            return variantes.length;
        }
        
        public String getVariante(int i) {
            return variantes[i];
        }
        
        private void parse(int langInteger) {
            String[] tokens = StringUtils.getTokens(sourceString, ';',StringUtils.EMPTY_EXCLUDE);
            int length = tokens.length;
            if (length == 0) throw new InternalResourceException("empty line = " + LangInteger.toString(langInteger));
            mainString = simplify(tokens[0]);
            if (length > 1) {
                variantes = new String[length -1];
                for (int i =0; i< (length-1);i++) {
                    variantes[i] = simplify(tokens[i+1]);
                }
            }
        }
        
        private String simplify(String s) {
            int idx = s.indexOf('(');
            if (idx != -1) {
                s = s.substring(0,idx).trim();
            }
            int idx2 = s.indexOf(',');
            if (idx2 != -1) {
                s = s.substring(idx2 +1).trim() + " " + s.substring(0,idx2).trim();
            }
            return s.intern();
        }
    }
    
    private void initGeoDirection(String l10nString) {
        try {
            InputStream is = getClass().getResourceAsStream("l10n/geo_" + l10nString + ".csv");
            if (is == null) return;
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is,"UTF-8"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx = ligne.indexOf(',');
                    if (idx != -1) {
                        String key = ligne.substring(0, idx).trim();
                        String lib = ligne.substring(idx+1).trim();
                        if (lib.length() != 0) {
                            geoDirectionMap.put(key,lib);
                        }
                    }
                }
            }
        } catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        } 
    }
    
     
}
