/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Sformatte 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.localisation;

import java.util.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.langinteger.*;

/**
 *
 * @author Vincent Calame
 */
public final class LocalisationInfo {
    private int workingLangInteger;
    private Locale formatLocale;
    private int formatLocaleInteger;
    private boolean same;
    
    public LocalisationInfo(int langInteger) {
        this.workingLangInteger = langInteger;
        this.formatLocale = LangInteger.getLocale(langInteger);
        same = true;
        initFormatLocaleInteger();
    }
    
    /**
     *@throws IllegalArgumentException si locale n'a pas d'information de langue (ou une information incorrecte).
     */
    public LocalisationInfo(int workingLangInteger, Locale locale) {
        String lang = locale.getLanguage();
        if (lang.length() == 0) throw new IllegalArgumentException("locale.getLanguage() is empty");
        if (lang.length() != 2) throw new IllegalArgumentException("locale.getLanguage() is wrong : " + lang);
        String country = locale.getCountry();
        if ((country.length() != 0) && (country.length() != 2)) throw new IllegalArgumentException("locale.getCountry() is wrong : " + country);
        if (locale.getVariant().length() > 0) locale = new Locale(lang,country,"");
        this.workingLangInteger = workingLangInteger;
        this.formatLocale = locale;
        if (locale.equals(LangInteger.getLocale(workingLangInteger))) {
            same = true;
        }
        else {
            same = false;
        }
        initFormatLocaleInteger();
    }
    
    public LocalisationInfo(LocalisationInfoBean bean) {
        workingLangInteger = LangInteger.UNDETERMINED;
        String workingLang = bean.getWorkingLang();
        if (workingLang != null) {
            try {
                workingLangInteger = LangInteger.parse(workingLang);
            }
            catch(LangIntegerException lie) {
                
            }
        }
        String formatLang = bean.getFormatLang();
        if (formatLang == null) {
            same = true;
            formatLocale = LangInteger.getLocale(workingLangInteger);
        }
        else {
            same = false;
            String formatCountry = bean.getFormatCountry();
            if (formatCountry == null) {
                formatLocale = new Locale(formatLang);
            }
            else {
                formatLocale = new Locale(formatLang,formatCountry);
            }
        }
        initFormatLocaleInteger();
    }
    
    private void initFormatLocaleInteger() {
        if (same) {
            formatLocaleInteger = workingLangInteger;
        }
        else {
            formatLocaleInteger = LangInteger.fromLocale(formatLocale);
        }
    }
    
    public int getWorkingLangInteger() {
        return workingLangInteger;
    }
    
    public Locale getFormatLocale() {
        return formatLocale;
    }
    
    public int getFormatLocaleInteger() {
        return formatLocaleInteger;
    }
    
    public LocalisationInfoBean toBean() {
        LocalisationInfoBean bean = new LocalisationInfoBean();
        bean.setWorkingLang(LangInteger.toString(workingLangInteger));
        if (!same) {
            bean.setFormatLang(formatLocale.getLanguage());
            String country = formatLocale.getCountry();
            if (country.length() > 0) bean.setFormatCountry(country);
        }
        return bean;
    }
    
    public int hashCode() {
        return workingLangInteger + formatLocale.hashCode();
    }
    
    public boolean isSame() {
        return same;
    }
    
    public boolean equals(Object obj) {
        if (obj == this) return true;
        if (!(obj instanceof LocalisationInfo)) return false;
        LocalisationInfo other = (LocalisationInfo) obj;
        if (other.workingLangInteger != this.workingLangInteger) return false;
        if (!other.formatLocale.equals(this.formatLocale)) return false;
        return true;
    }
    
    public String toString() {
        if (same) return LangInteger.toString(workingLangInteger);
        StringBuffer buf = new StringBuffer();
        LangInteger.appendString(buf,workingLangInteger);
        buf.append('_');
        buf.append(formatLocale.getLanguage());
        String country = formatLocale.getCountry();
        if (country.length() > 0) {
            buf.append('-');
            buf.append(country);
        }
        return buf.toString();
    }
    
    /**
     * Retourne null en cas d'erreur.
     */
    public static LocalisationInfo parse(String localisationSuffix) {
        String langString = null;
        String localeString = null;
        int idx = localisationSuffix.indexOf('_');
        if (idx == -1) {
            langString = localisationSuffix;
        }
        else {
            langString = localisationSuffix.substring(0, idx);
            localeString = localisationSuffix.substring(idx+1);
        }
        int langInteger = 0;
        try {
            langInteger = LangInteger.parse(langString);
        }
        catch(LangIntegerException ile) {
            return null;
        }
        if (localeString == null) {
            return new LocalisationInfo(langInteger);
        }
        Locale locale = parseLocale(localeString);
        if (locale == null) return null;
        return new LocalisationInfo(langInteger,locale);
    } 
    
    private static Locale parseLocale(String localeString) {
        int length = localeString.length();
        if ((length != 2) && (length != 5)) return null;
        try {
            int langInteger = LangInteger.parse(localeString);
            return LangInteger.getLocale(langInteger);
        }
        catch(LangIntegerException ile) {
            return null;
        }
    }
    
    
    
}
