/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.localisation;

import net.mapeadores.util.text.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.primitives.*;
import java.util.*;
import java.io.*;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.lexie.LexieFilter;

/**
 *
 * @author Vincent Calame
 */
public class LocalisationLexieFilters  {
    static final DefaultWordFilter defaultWordFilter = new DefaultWordFilter();
    static final IntHashMap currentFilter = new IntHashMap();
    
    private LocalisationLexieFilters() {
    }
    
    public static LexieFilter getLexieFilter(int langInteger) {
        LexieFilter wordFilter = (LexieFilter) currentFilter.get(langInteger);
        if (wordFilter == null) {
            wordFilter = init(langInteger);
            currentFilter.put(langInteger, wordFilter);
        }
        return wordFilter;
    }
    
    
    private static LexieFilter init(int langInteger) {
        Class currentClass = LocalisationLexieFilters.class;
        String langString = LangInteger.toString(langInteger);
        InputStream propertiesStream = currentClass.getResourceAsStream("resources/wordfilter_" + langString + ".properties");
        InputStream forbiddenStream = currentClass.getResourceAsStream("resources/forbiddenwords_" + langString + ".txt");
        try {
            LexieFilter wordFilter = init(langInteger,propertiesStream,forbiddenStream);
            if (wordFilter == null) wordFilter = defaultWordFilter;
            return wordFilter;
        }
        catch(IOException ioe) {
            ioe.printStackTrace();
            return defaultWordFilter;
        }
    }
    
    private static LexieFilter init(int langInteger,InputStream propertiesStream, InputStream forbiddenStream) throws IOException {
        if (propertiesStream == null) return null;
        if (forbiddenStream == null) return null;
        Properties properties = new Properties();
        properties.load(propertiesStream);
        propertiesStream.close();
        List forbiddenList = new ArrayList();
        BufferedReader reader = new BufferedReader(new InputStreamReader(forbiddenStream,"UTF-8"));
        String ligne = null;
        while((ligne = reader.readLine()) != null) {
            ligne = ligne.trim();
            if ((ligne.length() > 0) && (ligne.charAt(0) != '#')) {
                forbiddenList.add(ligne);
            }
        }
        reader.close();
        LocalisationWordFilter wordFilter = new LocalisationWordFilter(langInteger,properties, forbiddenList);
        return wordFilter;
    }
    
    private static class DefaultWordFilter implements LexieFilter {
        
        public boolean acceptLexie(String word) {
            int length = word.length();
            if (length == 0) return false;
            else if (length == 1) {
                return StringUtils.isIdeogram(word.charAt(0));
            }
            else if (length == 2) return false;
            else return true;
        }

    }
    
    private static class LocalisationWordFilter implements LexieFilter {
        int minlength = 3;
        int minpartlength = 2;
        int maxForbiddenWordLength = 0;
        HashSet forbiddenWordSet = null;
        Locale locale;
        
        LocalisationWordFilter(int langInteger,Properties properties,List forbiddenList) {
            String minlengthString = properties.getProperty("minlength");
            if (minlengthString != null) {
                try {
                    minlength = Integer.parseInt(minlengthString);
                }
                catch(NumberFormatException nfe) {
                    
                }
            }
            String minpartlengthString = properties.getProperty("minpartlength");
            if (minpartlengthString != null) {
                try {
                    minpartlength = Integer.parseInt(minpartlengthString);
                }
                catch(NumberFormatException nfe) {
                    
                }
            }
            locale = LangInteger.getLocale(langInteger);
            if (forbiddenList.size() > 0) {
                forbiddenWordSet = new HashSet();
                for(Iterator it = forbiddenList.iterator();it.hasNext();) {
                    String s= (String) it.next();
                    s = s.toLowerCase(locale);
                    forbiddenWordSet.add(s);
                    maxForbiddenWordLength = Math.max(maxForbiddenWordLength,s.length());
                }
            }
        }
        
        public boolean acceptLexie(String word) {
            int length = word.length();
            if (length == 1) {
                if (StringUtils.isIdeogram(word.charAt(0))) return true;
            }
            if (length < minlength) {
                if (length > 1) {
                    try {
                        Integer.parseInt(word);
                        return true;
                    }
                    catch(NumberFormatException nfe) {
                        
                    }
                }
                return false;
            }
            if (length > maxForbiddenWordLength) return true;
            if (forbiddenWordSet == null) return true;
            return !forbiddenWordSet.contains(word.toLowerCase(locale));
        }
        
    }
    
}
