/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.sxio;

import net.mapeadores.util.io.*;

import java.io.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;

/**
 *
 * @author Vincent Calame
 */
public class SXStreamTransformer implements StreamTransformer {
    private String extension;
    private String mimeType;
    private String name;
    private File stylesFile;
    private Templates contentTemplate;
    
    private SXStreamTransformer(String extension, String mimeType) {
        this.extension = extension;
        this.mimeType = mimeType;
    }
    
    private void initStyles( InputStream stylesInputStream) throws IOException {
        stylesFile = File.createTempFile("styles",".xml");
        OutputStream outputStream = new FileOutputStream(stylesFile);
        IOUtils.copy(stylesInputStream, outputStream);
        outputStream.close();
    }
    
    private void initTemplates(Source source) throws TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        contentTemplate = tFactory.newTemplates(source);
    }
    
    /**
     *
     *@throws IllegalArgumentException si sxDirectory n'est pas valide
     */
    /*public SXStreamTransformer(File sxDirectory) throws TransformerException {
        if (!sxDirectory.exists()) throw new IllegalArgumentException("sxDirectory doesn't exist : " + sxDirectory.getPath());
        if (!sxDirectory.isDirectory()) throw new IllegalArgumentException("sxDirectory is not a directory : " + sxDirectory.getPath());
        this.name = sxDirectory.getName();
        int idx =  name.lastIndexOf('.');
        if (idx == -1) throw new IllegalArgumentException("bad Extension : " + sxDirectory.getPath());
        extension =  name.substring(idx+1);
        mimeType = SXIO.getMimeType(extension);
        if (mimeType.length() == 0) throw new IllegalArgumentException("bad Extension : " + sxDirectory.getPath());
        stylesFile = new File(sxDirectory,"styles.xml");
        if (!stylesFile.exists()) throw new IllegalArgumentException(stylesFile.getPath()  + " does not exist");
        File contentFile  =new File(sxDirectory,"content.xsl");
        if (!contentFile.exists()) throw new IllegalArgumentException(contentFile.getPath() + " does not exist");
        TransformerFactory tFactory = TransformerFactory.newInstance();
        contentTemplate = tFactory.newTemplates(new StreamSource(contentFile));
    }*/
    
    public static SXStreamTransformer buildFromDirectory(String extension,File sxDirectory, StreamTransformerErrorHandler streamTransformerErrorHandler) throws IOException {
        String mimeType = SXIO.getMimeType(extension);
        if (mimeType.length() == 0) throw new IllegalArgumentException("wrong extension = " + extension);
        SXStreamTransformer sxStreamTransformer = new SXStreamTransformer(extension, mimeType);
        File stylesFile =new File(sxDirectory,"styles.xml");
        File contentFile  =new File(sxDirectory,"content.xsl");
        boolean error = false;
        if (!stylesFile.exists()) {
            streamTransformerErrorHandler.missingResourceError(stylesFile.toURI().toString());
            error = true;
        }
        else if (stylesFile.isDirectory()) {
            streamTransformerErrorHandler.missingResourceError(stylesFile.toURI().toString());
            error = true;
        }
        else {
            FileInputStream stylesInputStream = new FileInputStream(stylesFile);
            sxStreamTransformer.initStyles(stylesInputStream);
            stylesInputStream.close();
        }
        if (!contentFile.exists()) {
            streamTransformerErrorHandler.missingResourceError(contentFile.toURI().toString());
            error = true;
        }
        else if (contentFile.isDirectory()) {
            streamTransformerErrorHandler.missingResourceError(contentFile.toURI().toString());
            error = true;
        }
        else {
            try {
                sxStreamTransformer.initTemplates(new StreamSource(contentFile));
            }
            catch(TransformerException te) {
                streamTransformerErrorHandler.wrongResourceError(contentFile.toURI().toString(), te.getMessageAndLocation());
                error = true;
            }
        }
        if (error) {
            return null;
        }
        return sxStreamTransformer;
    }
    
    public String getName() {
        return name;
    }

    public String getExtension() {
        return extension;
    }

    public String getMimeType() {
        return mimeType;
    }

    public void transform(Source source,OutputStream outputStream) throws TransformerException {
        Transformer transformer = contentTemplate.newTransformer();
        SXContentSource sxContentSource = new TransformSXContentSource(extension,transformer,source);
        SXStylesSource sxStylesSource = new InputStreamSXStylesSource(stylesFile,"UTF-8");
        try {
            SXIO.write(outputStream, sxContentSource,sxStylesSource);
        }
        catch(SXIOException sxioe) {
            Throwable exception = sxioe.getCause();
            if (exception != null) {
                if (exception instanceof TransformerException) {
                    throw (TransformerException) exception;
                }
                else {
                    throw new TransformerException(exception);
                }
            }
            else {
                throw new TransformerException(sxioe);
            }
        }
    }
    
}
