/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.text;

import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.*;

/**
 * Cette classe gère un ensemble de libellés pour des langues données. Les chaînes sont systématiques nettoyées avec
 * XMLUtils.cleanString() et une chaîne de longueur nulle est assimiliée à <em>null</em>.
 *
 * @author  Vincent Calame
 */
public class LibelleByLangMap {
    static final String DEFAULT_LIBELLESTRING = "?";
    InternalLib defaultInternalLib;
    private LinkedIntHashMap internalLibMap;
    private String defaultLibelleString = null;
    
    /**
     * Avec ce constructeur le libellé par défaut est le libellé du premier libellé en vo.
     */
    public LibelleByLangMap() {

    }
    
    /**
     * Cette méthode fixe un libellé par défaut à utiliser lorsque le libellé n'existe pas.
     * L'argument est nettoyé et ne doit pas être vide.
     *
     *@throws IllegalArgumentException si defaultLibelleString est vide ou null.
     */
    public LibelleByLangMap(String defaultLibelleString) {
        if (defaultLibelleString == null) throw new IllegalArgumentException("defaultLibelleString == null");
        defaultLibelleString = StringUtils.cleanString(defaultLibelleString);
        if (defaultLibelleString.length() == 0) throw new IllegalArgumentException("cleaned defaultLibelleString is empty");
        this.defaultLibelleString = defaultLibelleString;
    }
    
    public List getLibelleList() {
        return new LibelleList();
    }
    
    public String getLibelleString(int langInteger, boolean defaut) {
        Libelle lib = getLibelle(langInteger);
        if (lib == null) {
            if (defaut) {
                if (defaultLibelleString != null) return defaultLibelleString;
                if (defaultInternalLib == null) return DEFAULT_LIBELLESTRING;
                else return defaultInternalLib.getLibelleString();
            }
            else return null;
        }
        return lib.getLibelleString();
     }
    
    /**
     * 
     *Le premier Libellé créé détermine la langue par défaut. Cependant, s'il n'est pas vo, le prochain vo qui le recontrera sera pris comme langue par défaut.
     */
    public Libelle createLibelle(int langInteger,String nvlibelle,boolean isvo)  {
        nvlibelle = StringUtils.cleanString(nvlibelle);
        if ((nvlibelle == null) || (nvlibelle.length() == 0)) return null;
        InternalLib lib = new InternalLib(langInteger,nvlibelle,isvo);
        if ((defaultInternalLib == null) || (defaultInternalLib.getLangInteger() == langInteger)) {
            defaultInternalLib = lib;
            return lib;
        }
        else {
            if (internalLibMap == null) internalLibMap = new LinkedIntHashMap();
            if (!(defaultInternalLib.isVo()) && isvo) {
                internalLibMap.put(defaultInternalLib.getLangInteger(),defaultInternalLib);
                defaultInternalLib = lib;
                internalLibMap.remove(langInteger);
            }
            else internalLibMap.put(langInteger,lib);
        }
        return lib;
    }
    
    public Libelle getLibelle(int langInteger) {
        if (defaultInternalLib == null) return null;
        if (langInteger == defaultInternalLib.getLangInteger()) return defaultInternalLib;
        if (internalLibMap == null) return null;
        return (InternalLib) internalLibMap .get(langInteger);
    }
    
    public Libelle removeLibelle(int langInteger) {
        if (defaultInternalLib == null) return null;
        if (langInteger == defaultInternalLib.getLangInteger()) {
            InternalLib oldlib = defaultInternalLib;
            if (internalLibMap  == null) {
                defaultInternalLib = null;
            }
            else {
                defaultInternalLib = (InternalLib) internalLibMap.getValueAt(0);
                internalLibMap.remove(defaultInternalLib.getLangInteger());
                if (internalLibMap.size() == 0) internalLibMap = null;
            }
            return oldlib;
        }
        else if (internalLibMap != null) {
            InternalLib lib = (InternalLib) internalLibMap.remove(langInteger);
            if (internalLibMap.size() == 0) internalLibMap = null;
            return lib;
        }
        return null;
    }
    
    public LibelleBean[] toLibelleBeanArray() {
        if (defaultInternalLib == null) return new LibelleBean[0];
        if (internalLibMap == null) {
            LibelleBean[] array = new LibelleBean[1];
            array[0] = new LibelleBean(defaultInternalLib);
            return array;
        }
        int size = internalLibMap.size();
        LibelleBean[] array = new LibelleBean[1 + size];
        array[0] = new LibelleBean(defaultInternalLib);
        for(int i = 0; i < size;i++) {
            array[i+1] = new LibelleBean((InternalLib)internalLibMap.getValueAt(i));
        }
        return array;
    }
    
    /**
     * Modifie les valeus du libellé.
     * Si <code>nvlibelle</code> est null ou de longueur nulle après nettoyage des espaces blancs superflus, la méthode est équivalent à <em>removeLibelle(lib.getLangInteger)</em>.
     * Retourne <em>true</em> si un changement a réellement eu lieu, <em>false</em> sinon.
     */
    public boolean setLibelle(Libelle lib,String nvlibelle,boolean isVo) {
        nvlibelle = StringUtils.cleanString(nvlibelle);
        if ((nvlibelle == null) || (nvlibelle.length() == 0)) {
            Libelle oldLib = removeLibelle(lib.getLangInteger());
            return (oldLib != null);
        }
        InternalLib intlib = (InternalLib) lib;
        if ((intlib.isVo == isVo) && (intlib.libelle.equals(nvlibelle))) return false;
        intlib.setLibelle(nvlibelle);
        intlib.setVo(isVo);
        return true;
    }
    
    /**
     * Crèe le libellé s'il n'existait pas.
     */
    public boolean setOrCreateLibelle(int langInteger,String libelleString,boolean isvo) {
        Libelle libelle = getLibelle(langInteger);
        if (libelle == null) {
            libelle = createLibelle(langInteger, libelleString,true);
            return true;
        }
        else {
            return setLibelle(libelle, libelleString, isvo);
        }
    }
    
    private class InternalLib implements Libelle {
        String libelle;
        boolean isVo;
        int langInteger;
        
        InternalLib(int langInteger,String libelle,boolean isVo) {
            this.langInteger = langInteger;
            setVo(isVo);
            setLibelle(libelle);
        }
        
        public boolean isVo() {
            return isVo;            
        }
        
        public String getLibelleString() {
            return libelle;
        }
        
        public void setVo(boolean isVo) {
            this.isVo = isVo;
        }
        
        /**
         * Le libelle ne doit pas être null.
         */
        public void setLibelle(String libelle) {
            if (libelle == null) throw new IllegalArgumentException("ibelle argument is null");
            this.libelle = libelle;
        }
        
        public int getLangInteger() {
            return langInteger;
        }
        
    }
    
     class LibelleList extends AbstractList implements RandomAccess {
         
         public Object get(int index) {
             if (defaultInternalLib == null) throw new IndexOutOfBoundsException();
             if (index == 0) return defaultInternalLib;
             if (internalLibMap  == null) throw new IndexOutOfBoundsException();
             return internalLibMap.getValueAt(index-1);
         }
         
         public int size() {
             if (defaultInternalLib == null) return 0;
             if (internalLibMap == null) return 1;
             else return 1 + internalLibMap.size();
         }
     }
}
