/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.text;

import java.util.*;

import java.io.Serializable;
import net.mapeadores.util.text.lexie.LexieFilter;
import net.mapeadores.util.text.lexie.LexieParser;
import net.mapeadores.util.text.lexie.LexieParserListener;


/**
 *
 * @author  Vincent Calame
 */
public class SearchSequence implements Serializable {
    private Token[] tokens;
    private String source;
    private int[] nonpertinentIndexArray;
    
    private SearchSequence(String source) {
        this.source = source;
    }
    
    private SearchSequence(String source, List list) {
        this.source = source;
        tokens = new Token[list.size()];
        int p = 0;
        for(Iterator it = list.iterator();it.hasNext();) {
            Token token = (Token) it.next();
            tokens[p] = token;
            p++;
        }
    }
    
    public static SearchSequence createSearchSequence(String chaine) {
        if (chaine == null) return null;
        chaine = chaine.trim();
        if (chaine.length() == 0) return null;
        SearchSequenceParser parser = new SearchSequenceParser(chaine);
        parser.parse();
        SearchSequence sequence = new SearchSequence(chaine);
        sequence.tokens = parser.getTokenArray();
        return sequence;
    }
    
    public SearchSequence filter(LexieFilter wordFilter,int defaultSearchType) {
        List tokenResult = new ArrayList();
        int npIdx = 0;
        int[] npIdxArray = new int[tokens.length];
        for(int i = 0; i < tokens.length;i++) {
            Token token = tokens[i];
            if (token.isMultiToken()) {
                MultiToken multiToken = (MultiToken) token;
                SimpleToken[] array = multiToken.getSubtokenArray();
                List list2 = new ArrayList();
                int q = 0;
                for(int j= 0; j < array.length;j++) {
                    SimpleToken simpleToken = convert(q,array[j],wordFilter,0);
                    if (simpleToken != null) {
                        list2.add(simpleToken);
                        q++;
                    }
                }
                if (list2.size() > 0) {
                    tokenResult.add(new MultiToken(i,list2));
                } else {
                    npIdxArray[npIdx] = i;
                    npIdx ++;
                }
            } else {
                SimpleToken simpleToken = convert(i, (SimpleToken)token, wordFilter, defaultSearchType);
                if (simpleToken != null) {
                    tokenResult.add(simpleToken);
                } else {
                    npIdxArray[npIdx] = i;
                    npIdx ++;
                }
            }
        }
        SearchSequence result = new SearchSequence(source,tokenResult);
        if (npIdx > 0) {
            int[] np2 = new int[npIdx];
            System.arraycopy(npIdxArray,0, np2, 0, npIdx);
            result.nonpertinentIndexArray = np2;
        }
        return result;
    }
    
    private SimpleToken convert(int tokenIndex,SimpleToken simpleToken, LexieFilter wordFilter,int defaultSearchType) {
        if (simpleToken.searchType == TextConstants.MATCHES) {
            if (!wordFilter.acceptLexie(simpleToken.tokenString)) return null;
        }
        int newSearchType = simpleToken.searchType | defaultSearchType;
        return new SimpleToken(tokenIndex,simpleToken.tokenString, newSearchType);
    }
    
    public int getTokenCount() {
        return tokens.length;
    }
    
    public boolean hasNonPertinentTokens() {
        return (nonpertinentIndexArray != null);
    }
    
    public int[] getNonPertinentIndexArray() {
        return nonpertinentIndexArray;
    }
    
    
    public Token getToken(int i) {
        return tokens[i];
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < tokens.length;i++) {
            if (i > 0) buf.append(' ');
            Token token = tokens[i];
            if (token.isMultiToken()) {
                buf.append('\"');
                SimpleToken[] array = ((MultiToken) token).getSubtokenArray();
                for (int j = 0; j < array.length;j++) {
                    if (j > 0) buf.append(' ');
                    buf.append(array[j].toString(true));
                }
                buf.append('\"');
            } else {
                buf.append(((SimpleToken) token).toString(true));
            }
        }
        return buf.toString();
    }
    
    public String getSource() {
        return source;
    }
    
    
    static public abstract class Token implements Serializable {
        int tokenIndex;
        
        Token(int tokenIndex) {
            this.tokenIndex = tokenIndex;
        }
        
        public abstract boolean isMultiToken();
        
        public int getTokenIndex() {
            return tokenIndex;
        }
        
    }
    
    static public class SimpleToken extends Token {
        int searchType;
        String tokenString;
        
        SimpleToken(int tokenIndex,String tokenString,int searchType) {
            super(tokenIndex);
            this.tokenString = tokenString;
            this.searchType = searchType;
        }
        
        public boolean isMultiToken() {
            return false;
        }
        
        public int getSearchType() {
            return searchType;
        }
        
        public String toString(boolean searchTypeInclude) {
            if ((searchType == TextConstants.MATCHES) || (!searchTypeInclude)) {
                return tokenString;
            } else {
                StringBuffer buf = new StringBuffer(tokenString.length() + 2);
                if ((searchType & TextConstants.ENDSWITH) != 0) buf.append('*');
                buf.append(tokenString);
                if ((searchType & TextConstants.STARTSWITH) != 0) buf.append('*');
                return buf.toString();
            }
        }
    }
    
    static public class MultiToken extends Token {
        SimpleToken[] subtokenArray;
        
        MultiToken(int tokenIndex,SimpleToken[] subtokenArray) {
            super(tokenIndex);
            this.subtokenArray = subtokenArray;
        }
        
        MultiToken(int tokenIndex,List list) {
            super(tokenIndex);
            int size2 = list.size();
            subtokenArray = new SimpleToken[size2];
            for (int j = 0; j < size2;j++) {
                SimpleToken simpleToken = (SimpleToken) list.get(j);
                subtokenArray[j] = simpleToken;
            }
        }
        
        public boolean isMultiToken() {
            return true;
        }
        
        public SimpleToken[] getSubtokenArray() {
            return subtokenArray;
        }
        
        public int getSubtokenCount() {
            return subtokenArray.length;
        }
        
    }
    
    static private class SearchSequenceParser implements LexieParserListener {
        String source;
        List tokenList = new ArrayList();
        int currentType = TextConstants.MATCHES;
        String currentWord = null;
        boolean onQuote = false;
        List quoteList = new ArrayList();
        
        SearchSequenceParser(String source) {
            this.source = source;
        }
        
        void parse() {
            LexieParser.parse(source,this);
            if (onQuote) {
                flushQuote();
            } else flushCurrentWord();
        }
        
        public void flushLexie(String word, int wordStart) {
            flushCurrentWord();
            currentWord = word;
        }
        
        private void flushCurrentWord() {
            if (currentWord != null) {
                if (onQuote) {
                    int p = quoteList.size();
                    quoteList.add(new SimpleToken(p,currentWord,currentType));
                } else {
                    int p = tokenList.size();
                    tokenList.add(new SimpleToken(p,currentWord,currentType));
                }
                currentWord = null;
                currentType = TextConstants.MATCHES;
            }
        }
        
        private void flushQuote() {
            flushCurrentWord();
            int size = quoteList.size();
            if (size > 0) {
                SimpleToken[] array = new SimpleToken[size];
                for (int j = 0; j < size;j++) {
                    array[j] = (SimpleToken) quoteList.get(j);
                }
                int p = tokenList.size();
                tokenList.add(new MultiToken(p,array));
            }
            onQuote = false;
            quoteList.clear();
        }
        
        public void checkSpecialChar(char carac) {
            if (carac == '*') {
                if (currentWord == null) {
                    currentType = TextConstants.ENDSWITH;
                } else {
                    currentType = currentType | TextConstants.STARTSWITH;
                    flushCurrentWord();
                }
            } else if (carac == '\"') {
                if (onQuote) {
                    flushQuote();
                } else {
                    flushCurrentWord();
                    onQuote = true;
                }
            } else {
                flushCurrentWord();
            }
        }
        
        
        public Token[] getTokenArray() {
            int size = tokenList.size();
            Token[] result = new Token[size];
            for(int i = 0; i < size; i++) {
                Token token = (Token) tokenList.get(i);
                result[i] = token;
            }
            return result;
        }
        
        
    }
}
