/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text;

import java.io.Serializable;

/**
 * SubstringPosition est utilisée pour conserver les informations d'une sous-chaîne à l'intérieure d'une chaîne.
 * SubstringPosition ne se préoccupe pas des chaînes et sous-chaînes en question, cette classe se contente de manipuler l'index de départ de la sous-chaîne dans la châine
 * et la longueur de la sous-chaîne.
 *
 * @author Vincent Calame
 */
public final class SubstringPosition implements Serializable,Comparable{
    private int beginIndex;
    private int length;
    
    /**
     * Crèe une instance de SubstringPosition avec beginIndex comme index de départ de la sous-chaîne et length comme longueur de la sous-chaîne.
     */
    public SubstringPosition(int beginIndex, int length) {
        this.beginIndex = beginIndex;
        this.length = length;              
    }
    
    /**
     * Retourne l'index de départ de la sous-chaîne dans la châine.
     */
    public int getBeginIndex() {
        return beginIndex;
    }
    
    /**
     * Retourne la longueur de la sous-chaîne.
     */
    public int getLength() {
        return length;
    }
    
    /**
     * Retourne l'index dans la chaîne du dernier caractère de la sous-chaîne.
     * Méthode raccourci équivalente à getBeginIndex() + getLength() - 1
     */
    public int getEndIndex() {
        return beginIndex + length - 1;
    }
    
    /**
     * Une instance de SubstringPosition est égale à une autre si getBeginIndex() et getLength() ont des valeurs égales.
     */
    public boolean equals(Object obj) {
        if (obj == this) return true;
        if (!(obj instanceof SubstringPosition)) return false;
        SubstringPosition sp = (SubstringPosition) obj;
        return (sp.beginIndex == this.beginIndex) && (sp.length == this.length);
    }
    
    /**
     * Réécriture de la méthode hashCode pour la rendre cohérente avec la méthode equals().
     * Retourne getBeginIndex() * 1000 + length;
     */
    public int hashCode() {
        return beginIndex * 1000 + length;
    }

    /**
     * Implémentation de l'interface Comparable. Une instance de SubstringPosition est « inférieure « à une autre
     * si elle est plus proche de l'origine de la chaîne.
     * Retourne -1 si getBeginIndex() < ((SubstringPosition) obj).getBeginIndex()
     * Retourne 1 si getBeginIndex() > ((SubstringPosition) obj).getBeginIndex()
     * Si getBeginIndex() = ((SubstringPosition) obj).getBeginIndex(),
     * retourne -1 si getLength() < (SubstringPosition) obj).getLength()
     * et retourne 1 si getLength() > (SubstringPosition) obj).getLength()
     *
     */
    public int compareTo(Object obj) {
        if (obj == this) return 0;
        SubstringPosition sp = (SubstringPosition) obj;
        if (this.beginIndex == sp.beginIndex) {
            if (this.length == sp.length) return 0;
            else return (this.length < sp.length)?-1:1;
        }
        return (this.beginIndex < sp.beginIndex)?-1:1;
    }
    
    /**
     * Vérifie si l'instance de SubstringPosition contient une autre instance de SubstringPosition.
     * Une instance de SubstringPosition en contient une autre si <em>(substringPosition.beginIndex >= this.beginIndex) && (substringPosition.length <= this.length)</em>
     */
    public boolean contains(SubstringPosition substringPosition) {
       return (substringPosition.beginIndex >= this.beginIndex) && (substringPosition.length <= this.length);
    }
    
    public static SubstringPosition merge(SubstringPosition sp1,SubstringPosition sp2) {
        int comp = sp1.compareTo(sp2);
        if (comp == 0) return sp1;
        if (comp < 0) return mergeInOrder(sp1,sp2);
        else return mergeInOrder(sp2,sp1);
    }
    
    private static SubstringPosition mergeInOrder(SubstringPosition sp1,SubstringPosition sp2) {
        int start = sp1.beginIndex;
        int length = sp2.beginIndex - sp1.beginIndex + sp2.length;
        return new SubstringPosition(start,length);
    }
}
