/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.text.collation;

import java.text.CollationElementIterator;
import java.text.RuleBasedCollator;
import net.mapeadores.util.collections.IntArrayBuffer;
import java.io.Serializable;

/**
 * La classe CollationUnit représente une chaîne « nettoyée » par une instance de RuleBasedCollator.
 * Par « nettoyée », on entend le fait que chaque caractère de la chaîne source est évaluée par le RuleBasedCollator et remplacé le ou les caractères qui le représente dans le premier ordre (primaryOrder) du RuleBasedCollator. 
 * <p>Par exemple, « É » sera traité comme un « e » et « œ » comme « oe ». Un caractère de la chaîne d'origine peut être remplacé par 0, 1 ou plusieurs caractères dans la chaîne « collatée ».
 * <p>Un tableau de correspondance est conservé afin de signaler à quel caractère de la chaîne d'origine correspond un caractère de la chaîne « nettoyée ».
 * <p>Les instances de CollationUnit sont utilisées pour faire des tris et des recherches sur des chaînes en tenant compte des règles d'un RuleBaseCollator.
 * <p>L'utilisation de CollationUnit poursuit les mêmes but que la classe CollationKey du paquetage java.text avec les différences suivantes :
 * <ul>
 * <li>CollationUnit ne tient compte que du premier ordre (voir le concept de collator.getStength() == Collator.PRIMARY). Il occupe deux fois moins de place.
 * <li>Le résultat de CollationUnit est une chapine (obtenue avec la méthode toString()).
 * Il est donc possible de bénéficier de toutes les méthodes sur les chaînes pour faire des recherches ou des tris.
 * </ul>
 * <p>Attention, la chaîne retournée par toString() permet des traitements de classements mais n'est pas lisible pour autant.
 * Autrement dit, pour la chaîne d'origine « Œuvre », toString() ne retourne pas « oeuvre » mais « aWghdW » (que renvoie également « oeuvre »).
 * Il n'est pas possible de reconstituer cette chaîne simplifiée.
 * <p>Voir le code source du constructeur pour plus de détails.
 * 
 * @author Vincent Calame
 */
public final class CollationUnit implements Serializable {
    private String sourceString;
    private String collatedString;
    private int[] correspondances;
    
    /**
     * Crèe une nouvelle instance de CollationUnit avec <code>sourceString</code> comme chaîne d'origine en utilisant les règles de « collation » fixée par <code>collator</code>.
     * <p>Note :  <code>collator </code> doit être une instance de RuleBasedCollator et non de Collator car le constructeur se base sur une instance de CollationElementIterator.
     */
    public CollationUnit(String sourceString, RuleBasedCollator collator) {
        this.sourceString = sourceString;
        CollationElementIterator collationElementIterator = collator.getCollationElementIterator(sourceString);
        StringBuffer buf = new StringBuffer();
        IntArrayBuffer intBuffer = new IntArrayBuffer(sourceString.length() + 10);
        while (true) {
            int order = collationElementIterator.next();
            int offset = collationElementIterator.getOffset() - 1;
            if (order == CollationElementIterator.NULLORDER) break;
            int primary = CollationElementIterator.primaryOrder(order);
            if (primary > 0) {
                buf.append((char)primary);
                intBuffer.add(offset);
            }
        }
        collatedString = buf.toString();
        correspondances = intBuffer.toArray();
    }
    
    /**
     * Retourne la chaîne à l'origine 
     */
    public String getSourceString() {
        return sourceString;
    }
    
    public String getCollatedString() {
        return collatedString;
    }
    
    /**
     * Retourne le nombre de caractères de la chaîne après « collation ». On n'a pas forcément <em>getSourceString().length() == lenght()</em>.
     *<p>Cette méthode est raccourci de <em>toString().length()</em>
     */
    public int length() {
        return collatedString.length();
    }
    
    /**
     * Retourne sous forme d'objet String la chaîne après « collation ».
     */
    public String toString() {
        return collatedString;
    }
    
    /*
     * Retourne l'index dans la chaîne d'origine du caractère qui a produit le caractère à la position <code>collatedIndex</code> dans la chaîne « nettoyée » résultant.
     */
    public int sourceIndexOf(int collatedIndex) {
        return correspondances[collatedIndex];
    }
    
    
    /**
     * Méthode statique qui retourne le même résultat que (new CollationUnit(sourceString,collationElementIterator)).toString(), à ceci près qu'elle ne fait pas de traitement sur les correspondances entre les caractères de la source et ceux du résultat.
     */
    public static String collate(String sourceString,RuleBasedCollator collator) {
        CollationElementIterator collationElementIterator = collator.getCollationElementIterator(sourceString);
        StringBuffer buf = new StringBuffer();
        while (true) {
            int order = collationElementIterator.next();
            if (order == CollationElementIterator.NULLORDER) break;
            int primary = CollationElementIterator.primaryOrder(order);
            if (primary > 0) {
                buf.append((char)primary);
            }
        }
        return buf.toString();
    }
    
    
}
