/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.text.collation.map;

import java.util.*;
import java.text.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.text.collation.*;

/**
 * Classe permettant de stocker des valeurs suivant des clés qui sont des chaînes traités par « collation ».
 * Lorsque une valeur est stocké suivant une clé, celle-ci subit d'abord une « collation » (dont le résultat s'appele collatedKey).
 * Autrement dit, deux chaînes key1 et key2 sont des clés identiques non pas si key1.equals(key2) mais si toCollatedString(key1).equals(toCollatedString(key2)) 
 *
 * @author  Vincent Calame
 */
public class SortedCollatedKeyMap implements CollatedKeyMap {
    private RuleBasedCollator collator;
    private Locale locale;
    private TreeMap valuesMap = new TreeMap();
    
    /**
     * Construit une instance de SortedCollatedKeyMap en prenant comme « collator » Collator.getInstance(locale)
     */
    public SortedCollatedKeyMap(Locale locale) {
        this.locale = locale;
        collator = (RuleBasedCollator) Collator.getInstance(locale);
        collator.setStrength(Collator.PRIMARY);
    }
    
    /**
     * Construit une instance de SortedCollatedKeyMap en prenant comme « collator » Collator.getInstance(LangInteger.getLocale(langInteger))
     */
     public SortedCollatedKeyMap(int langInteger) {
        this.locale = LangInteger.getLocale(langInteger);
        collator = (RuleBasedCollator) Collator.getInstance(locale);
        collator.setStrength(Collator.PRIMARY);
    }
     
     public Collection values() {
         return valuesMap.values();
     }
     
     /**
      * Retourne l'instance de Collator utilisée pour la création des « collatedKey » et la recherche.
      */
     public Collator getCollator() {
         return collator;
     }
     
     /**
      * Retourne l'objet Locale qui a servi à la construction de l'objet.
      */
     public Locale getLocale() {
         return locale;
     }
     
     /**
     * Retourne le nombre de valeurs stockées dans l'instance de SortedCollatedKeyMap.
     */
     public int size() {
         return valuesMap.size();
     }
    
     /**
      * Retourne la valeur stockée à la clé « s ». La chaîne s subit d'abord une « collation ».
      * Cette méthode est équivalente à getValueByCollatedKey(toCollateKey(key))
      */
    public Object getValue(String key) {
        String collatedKey = CollationUnit.collate(key,collator);
        return valuesMap.get(collatedKey);
    }
    
    
    
    /**
     * Retourne la valeur stockée à la clé déjà « collatée », collatedKey.
     */
    public Object getValueByCollatedKey(String collatedKey) {
        return valuesMap.get(collatedKey);
    }
    
    /**
     * Stocke la valeur à la clé Key. Équivalent de putValueByCollatedKey(toCollatedKey(key),value).
     */
    public void putValue(String key,Object value) {
        String collatedKey = CollationUnit.collate(key,collator).intern();
        valuesMap.put(collatedKey,value);
    }
    
    /**
     * Stocke la valeur à la clé déjà « collatée » collatedKey.
     */
    public void putValueByCollatedKey(String collatedKey,Object value) {
        valuesMap.put(collatedKey,value);
    }
    
    /**
     * Supprime la valeur référencée à la clé key.
     */
    public Object removeValue(String key) {
        String collatedKey = CollationUnit.collate(key,collator);
        return valuesMap.remove(collatedKey);
    }
    
    /**
     * Supprime la valeur à la clé déjà « collatée » collatedKey.
     */
    public Object removeValueByCollatedKey(String collatedKey) {
        return valuesMap.remove(collatedKey);
    }
    
    
    /**
     * Crèe un objet CollationUnit avec comme source s et en se basant sur le Collator de l'instance de SortedCollatedKeyMap
     * Équivalent de new CollationUnit(s,getCollator().
     */
    public CollationUnit collateString(String s) {
        return new CollationUnit(s,collator);
    }
    
    /**
     * Retourne la forme « collatée » d'une clé.
     */
    public String toCollatedKey(String key) {
        return CollationUnit.collate(key,collator);
    }
    
    /**
     * Effectue une recherche sur les clés de l'instance de SortedCollatedKeyMap et retourne le résultat sous la forme d'une instance de searchResultItemList.
     * Cette méthode vérifie la présence de searchText dans les clés suivant trois conditions : TextConstants.CONTAINS, TextConstants.ENDSWITH, TextConstants.STARTSWITH
     * searchText subit évidemment une « collation » pour que les conditions répondent aux règles du Collator.
     */
    public List search(String searchText,int type) {
        switch(type) {
            case TextConstants.CONTAINS :
                return getValuesContaining(searchText);
            case TextConstants.ENDSWITH :
                return getValuesEndingWith(searchText);
            case TextConstants.MATCHES :
                String collatedKey = CollationUnit.collate(searchText,collator);
                Object value = valuesMap.get(collatedKey);
                if (value == null) return Collections.EMPTY_LIST;
                else {
                    return Collections.singletonList(new InternalSearchResultItem(value, collatedKey,new SubstringPosition(0, collatedKey.length())));
                } 
            case TextConstants.STARTSWITH :
                return getValuesStartingWith(searchText);
            default :
                throw new IllegalArgumentException("unknwon TextConstants type : " + type);
        }
    }
    
    public void clear() {
        valuesMap.clear();
    }
    
    /*
     * Méthode appelée par search(). 
     * Retourne la liste des valeurs dont la clé commence par s
     */
    private List getValuesStartingWith(String s) {
        String searchkey = CollationUnit.collate(s,collator);
        List searchResultItemList = new ArrayList();
        int minlength = searchkey.length();
        SortedMap sorted = valuesMap.tailMap(searchkey);
        for (Iterator it = sorted.entrySet().iterator();it.hasNext();) {
            Map.Entry couple = (Map.Entry) it.next();
            String currentkey = (String) couple.getKey();
            if (currentkey.length() < minlength) break;
            else if (currentkey.startsWith(searchkey)) {
                InternalSearchResultItem searchResultItem = new InternalSearchResultItem(couple.getValue(),currentkey,new SubstringPosition(0,minlength));
                searchResultItemList.add(searchResultItem);
            }
            else break;
        }
        return searchResultItemList;
    }
    
    /*
     * Méthode appelée par search(). 
     * Retourne la liste des valeurs dont la clé termine par s
     */
    private List getValuesEndingWith(String s) {
        String searchkey = CollationUnit.collate(s,collator);
        List searchResultItemList = new ArrayList();
        int minlength = searchkey.length();
        for (Iterator it = valuesMap.entrySet().iterator();it.hasNext();) {
            Map.Entry couple = (Map.Entry) it.next();
            String currentkey = (String) couple.getKey();
            if (currentkey.length() < minlength) continue;
            else if (currentkey.endsWith(searchkey)) {
                InternalSearchResultItem searchResultItem = new InternalSearchResultItem(couple.getValue(),currentkey,new SubstringPosition(currentkey.length() - minlength,minlength));
                searchResultItemList.add(searchResultItem);
            }
            else continue;
        }
        return searchResultItemList;
    }
    
    /*
     * Méthode appelée par search(). 
     * Retourne la liste des valeurs dont la clé contient s
     */
    private List getValuesContaining(String s) {
        String searchkey = CollationUnit.collate(s,collator);
        List searchResultItemList = new ArrayList();
        int minlength = searchkey.length();;
        List liste = new LinkedList();
        for (Iterator it = valuesMap.entrySet().iterator();it.hasNext();) {
            Map.Entry couple = (Map.Entry) it.next();
            String currentkey = (String) couple.getKey();
            if (currentkey.length() < minlength) continue;
            else {
                int idx = currentkey.indexOf(searchkey);
                if (idx != -1) {
                    InternalSearchResultItem searchResultItem = new InternalSearchResultItem(couple.getValue(),currentkey,new SubstringPosition(idx,minlength));
                    searchResultItemList.add(searchResultItem);
                }
            }
        }
        return searchResultItemList;
    }
    
   
    
    /**
     * La classe InternalSearchResultItem stocke les informations d'un résultat de recherche effectuée via la méthode
     */
    static class InternalSearchResultItem implements SearchResultItem {
        private Object value;
        private SubstringPosition collatedSearchTextPosition;
        private String collatedKey;

        /*
         * Construction du résultat de recherche.
         */
        private InternalSearchResultItem(Object value, String collatedKey,SubstringPosition collatedSearchTextPosition) {
            this.value = value;
            this.collatedKey = collatedKey;
            this.collatedSearchTextPosition = collatedSearchTextPosition;
        }

        /**
         * Retourne la valeur trouvée par la recherche.
         */
        public Object getValue() {
            return value;
        }

        /**
         * Retourne la position du texte de recherche (après « collation ») au sein de la clé.
         *
         */
        public SubstringPosition getSearchTextPositionInCollatedKey() {
            return collatedSearchTextPosition;
        }
        
        /**
         * Retourne la clé à laquelle est stockée la valeur trouvée dans l'instance SortedCollatedKeyMap.
         */
        public String getCollatedKey() {
            return collatedKey;
        }
    }
}
