/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.insertpattern.impl;

import net.mapeadores.util.text.insertpattern.api.*;
import net.mapeadores.util.text.*;
import java.io.*;
import java.util.*;
import java.text.*;


/**
 *
 * @author Vincent Calame
 */
public class InstructionParser {
    final static int WAITING_FROM_ARGUMENT_END_STEP = 3;
    final static int WAITING_FROM_KEY_STEP = 4;
    final static int WAITING_FROM_VALUE_STEP = 5;
    final static int WAITING_FROM_EQUAL_STEP = 6;
    final static int KEY_STEP = 7;
    final static int QUOTED_KEY_STEP = 8;
    final static int VALUE_STEP = 9;
    final static int QUOTED_VALUE_STEP = 10;
    List argumentList = new ArrayList();
    ArgumentParser currentArgumentParser;
    int currentLineNumber = 0;
    ErrorHandler currentErrorHandler;
    
    public InstructionParser() {
    }
    
    public Instruction parse(String s, ErrorHandler errorHandler) {
        argumentList.clear();
        this.currentErrorHandler = errorHandler;
        currentLineNumber = 0;
        currentArgumentParser = new ArgumentParser();
        try {
            StringReader stringReader = new StringReader(s);
            LineNumberReader bufreader = new LineNumberReader(stringReader);
            String line= null;
            while((line = bufreader.readLine()) != null) {
                try {
                    parseLine(line, bufreader.getLineNumber());
                 } catch(IllegalArgumentException ise) {
                    break;
                }
            }
        }
        catch(IOException ioe) {
            
        }
        Argument arg = currentArgumentParser.toArgument();
        if ((arg.getKey().length() != 0) || (argumentList.size() > 0)) {
            argumentList.add(arg);
        }
        currentArgumentParser = null;
        if (argumentList.size() == 0) return null;
        return new ImplInstruction(argumentList);
    }
    
    public static interface ErrorHandler {
        
        public void invalidAsciiCharacterError(String part,int row, int col);
        
        public void invalidEndCharacterError(String part,int row, int col);
        
        public void invalidSeparatorCharacterError(String part,int row,int col);
    }
    
    private void parseLine(String line,int lineNumber) {
        currentLineNumber = lineNumber;
        int length = line.length();
        for (int i = 0; i < length; i++) {
            i = currentArgumentParser.parseCharAt(line, i);
        }
    }
    
    private int getCurrentLineNumber() {
        return currentLineNumber;
    }
    
    private void endArgument() {
        argumentList.add(currentArgumentParser.toArgument());
        currentArgumentParser = new ArgumentParser();
    }
    
    private class ArgumentParser  {
        int keyRow;
        int keyCol;
        int valueRow;
        int valueCol;
        StringBuffer keyBuffer = new StringBuffer();
        StringBuffer valueBuffer = new StringBuffer();
        boolean previousWhite = false;
        int step = WAITING_FROM_KEY_STEP;
        
        ArgumentParser() {
            
        }
        
        Argument toArgument() {
            ImplArgument arg = new ImplArgument(keyBuffer.toString(),keyRow,keyCol);
            if (valueCol > 0) {
                arg.setValue(valueBuffer.toString(),valueRow,valueCol);
            }
            return arg;
        }
        
        void testWhiteChar(char carac) {
            switch(step) {
                case WAITING_FROM_ARGUMENT_END_STEP :
                case WAITING_FROM_KEY_STEP :
                case WAITING_FROM_EQUAL_STEP :
                    break;
                case KEY_STEP :
                    step = WAITING_FROM_EQUAL_STEP;
                    break;
                case QUOTED_KEY_STEP :
                    keyBuffer.append(carac);
                    break;
                case VALUE_STEP :
                    previousWhite = true;
                    break;
                case QUOTED_VALUE_STEP :
                    valueBuffer.append(carac);
                    break;
            }
            
        }
        
        boolean testEnd(char carac) {
            if (carac == ',') {
                endArgument();
                return true;
            }
            return false;
        }
        
        int parseCharAt(String line, int index) {
            char carac = line.charAt(index);
            if (Character.isWhitespace(carac)) {
                testWhiteChar(carac);
                return index;
            }
            if ((step != QUOTED_KEY_STEP) && (step != QUOTED_VALUE_STEP)) {
                if (testEnd(carac)) {
                    return index;
                }
            }
            switch(step) {
                case WAITING_FROM_ARGUMENT_END_STEP :
                    valueBuffer.append(carac);
                    currentErrorHandler.invalidEndCharacterError(valueBuffer.toString(),getCurrentLineNumber(),index);
                    throw new IllegalArgumentException();
                case WAITING_FROM_KEY_STEP :
                    keyRow = getCurrentLineNumber();
                    keyCol = index+1;
                    if (carac == '\"') {
                        step = QUOTED_KEY_STEP;
                    } else if (testAsciiChar(carac,index,keyBuffer)) {
                        step = KEY_STEP;
                    }
                    break;
                case WAITING_FROM_EQUAL_STEP :
                    if (carac == '=') {
                        step = WAITING_FROM_VALUE_STEP;
                    } else {
                        keyBuffer.append(carac);
                        currentErrorHandler.invalidSeparatorCharacterError(keyBuffer.toString(),getCurrentLineNumber(),index);
                        throw new IllegalArgumentException();
                    }
                    break;
                case WAITING_FROM_VALUE_STEP :
                    valueRow = getCurrentLineNumber();
                    valueCol = index+1;
                    if (carac == '\"') {
                        step = QUOTED_VALUE_STEP;
                    } else if (testAsciiChar(carac,index,valueBuffer)) {
                        step = VALUE_STEP;
                    }
                    break;
                case KEY_STEP :
                    if (carac == '=') {
                        step = WAITING_FROM_VALUE_STEP;
                    } else testAsciiChar(carac,index,keyBuffer);
                    break;
                case QUOTED_KEY_STEP :
                    if (carac == '\"') {
                        step = WAITING_FROM_EQUAL_STEP;
                    } else index = parseQuoteCharAt(carac,line,index,keyBuffer);
                    break;
                case VALUE_STEP :
                    if (previousWhite) valueBuffer.append(' ');
                    testAsciiChar(carac,index,valueBuffer);
                    break;
                case QUOTED_VALUE_STEP :
                    if (carac == '\"') {
                        step = WAITING_FROM_ARGUMENT_END_STEP;
                    } else index = parseQuoteCharAt(carac,line,index,valueBuffer);
                    break;
            }
            previousWhite = false;
            return index;
        }
        
        boolean testAsciiChar(char carac, int index, StringBuffer buf) {
            if (testEnd(carac)) {
                return false;
            } else {
                if (StringUtils.isValidAsciiChar(carac)) {
                    buf.append(carac);
                    return true;
                } else {
                    buf.append(carac);
                    currentErrorHandler.invalidAsciiCharacterError(buf.toString(),getCurrentLineNumber(), index);
                    throw new IllegalArgumentException();
                }
            }
        }
        
        
        int parseQuoteCharAt(char carac,String line, int index, StringBuffer buf) {
            if (carac == '\\') {
                if (index < (line.length()-1)) {
                    char next = line.charAt(index+1);
                    if (next == 'n') buf.append('\n');
                    else buf.append(next);
                    index = index + 1;
                }
            } else {
                buf.append(carac);
                
            }
            return index;
        }
        
    }
 
}

