/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.lexie;

import net.mapeadores.util.text.*;
import java.util.*;
import java.text.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.localisation.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieMap {
    private SortedCollatedKeyMap collatedKeyMap;
    private UnmodifiableCollatedKeyMap unmodifiableCollatedKeyMap;
    private LexieFilter lexieFilter;
    private Map lexieSourceMap = new LinkedHashMap();
    private Map unmodifLexieSourceMap = Collections.unmodifiableMap(lexieSourceMap);
    
    public LexieMap(Locale locale) {
        collatedKeyMap = new SortedCollatedKeyMap(locale);
        init(LangInteger.fromLocale(locale));
    }
    
    public LexieMap(int langInteger) {
        collatedKeyMap = new SortedCollatedKeyMap(langInteger);
        init(langInteger);
    }
    
    private void init(int langInteger) {
        unmodifiableCollatedKeyMap = new UnmodifiableCollatedKeyMap(collatedKeyMap);
        lexieFilter = LocalisationLexieFilters.getLexieFilter(langInteger);
    }
    
    public CollatedKeyMap getCollatedKeyMap() {
        return unmodifiableCollatedKeyMap;
    }
    
    public Map getParsedLexieSourceMap() {
        return unmodifLexieSourceMap;
    }
    
    public void setLexieFilter(LexieFilter lexieFilter) {
        this.lexieFilter = lexieFilter;
    }
    
    public void addLexieSource(LexieSource lexieSource) {
        ImplParsedLexieSource ipls = new ImplParsedLexieSource(lexieSource);
        lexieSourceMap.put(lexieSource.getSourceId(),ipls);
        
        InternalLexieParserListener internalLexieParserListener = new InternalLexieParserListener(ipls);
        LexieParser.parse(lexieSource.getSourceText(),internalLexieParserListener);
    }
    
    private class InternalLexieParserListener implements LexieParserListener {
        ImplParsedLexieSource parsedLexieSource;
        
        InternalLexieParserListener(ImplParsedLexieSource parsedLexieSource) {
            this.parsedLexieSource = parsedLexieSource;
        }
        
        public void flushLexie(String lexie, int lexieStartIndex) {
            if ((lexieFilter != null) && (!lexieFilter.acceptLexie(lexie))) return;
            String collatedLexie = collatedKeyMap.toCollatedKey(lexie);
            ImplLexieUnit lexieUnit = (ImplLexieUnit) collatedKeyMap.getValueByCollatedKey(collatedLexie);
            if (lexieUnit == null) {
                lexieUnit = new ImplLexieUnit(collatedLexie,collatedKeyMap.size() +1);
                collatedKeyMap.putValueByCollatedKey(collatedLexie,lexieUnit);
            }
            SubstringPosition substringPosition = new SubstringPosition(lexieStartIndex, lexie.length());
            ImplOccurence implOccurence = new ImplOccurence(lexieUnit,parsedLexieSource,substringPosition);
            parsedLexieSource.addOccurence(implOccurence);
            lexieUnit.addOccurence(implOccurence);
            lexieUnit.checkCanonical(lexie);
        }
        
        public void checkSpecialChar(char carac) {
            
        }
        
    }
    
    private class ImplParsedLexieSource implements ParsedLexieSource {
        private LexieSource lexieSource;
        List occurenceList = new ArrayList();
        private List unmodifiableOccurenceList = Collections.unmodifiableList(occurenceList);
        
        ImplParsedLexieSource(LexieSource lexieSource) {
            this.lexieSource = lexieSource;
        }
        
        public LexieSource getLexieSource() {
            return lexieSource;
        }
        
        public List getOccurenceList() {
            return unmodifiableOccurenceList;
        }
        
        void addOccurence(Occurence occurence) {
            occurenceList.add(occurence);
        }
    }
    
    private class ImplOccurence implements Occurence {
        ImplLexieUnit implLexieUnit;
        SubstringPosition substringPosition;
        ImplParsedLexieSource implParsedLexieSource;
        int occurencePosition;
        
        ImplOccurence(ImplLexieUnit implLexieUnit,ImplParsedLexieSource implParsedLexieSource,SubstringPosition substringPosition) {
            this.implLexieUnit = implLexieUnit;
            this.substringPosition = substringPosition;
            this.implParsedLexieSource = implParsedLexieSource;
        }
        
        public TextLexieUnit getLexieUnit() {
            return implLexieUnit;
        }
        
        public SubstringPosition getSubstringPosition() {
            return substringPosition;
        }
        
        public ParsedLexieSource getParsedLexieSource() {
            return implParsedLexieSource;
        }
        
        public int getOccurencePosition() {
            return occurencePosition;
        }
        
        void setOccurencePosition(int occurencePosition) {
            this.occurencePosition = occurencePosition;
        }

    }
    
    
    private  class ImplLexieUnit implements TextLexieUnit {
        private String collatedLexie;
        private String canonicalLexie;
        private List occurenceList = new ArrayList();
        private List unmodifiableOccurenceList = Collections.unmodifiableList(occurenceList);
        private String lexieId;
        
        
        public ImplLexieUnit(String collatedLexie, int id) {
            this.collatedLexie = collatedLexie;
            this.lexieId = String.valueOf(id);
        }
        
        public String getCollatedLexie() {
            return collatedLexie;
        }
        
        public String getCanonicalLexie() {
            return canonicalLexie;
        }
        
        public List getOccurenceList() {
            return unmodifiableOccurenceList;
        }
        
        public String getLexieId() {
            return lexieId;
        }
        
        void addOccurence(ImplOccurence occurence) {
            int occurencePosition = 1;
            if (occurenceList.size() > 0) {
                ImplOccurence last = (ImplOccurence) occurenceList.get(occurenceList.size() -1);
                if (last.getParsedLexieSource().equals(occurence.getParsedLexieSource())) {
                    occurencePosition = last.getOccurencePosition() + 1;
                }
            }
            occurence.setOccurencePosition(occurencePosition);
            occurenceList.add(occurence);
        }
        
        void checkCanonical(String lexie) {
            if (canonicalLexie == null) canonicalLexie = lexie;
            else {
                if (lexie.length() < canonicalLexie.length()) {
                    canonicalLexie = lexie;
                }
            }
        }
        
    }
    
}
