/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.xml;

import java.io.*;
import net.mapeadores.util.xml.ns.NameSpace;

/**
 *
 * @author Vincent Calame
 */
public class DefaultXMLWriter implements XMLWriter {
    private XMLDestination xmlDestination;
    private int indentLength = 0;
    private boolean aposEscaped = true;
    
    public DefaultXMLWriter() {
    }
    
    public XMLDestination getXMLDestination() throws IOException {
        return xmlDestination;
    }
    
    public void setXmlDestination(XMLDestination xmlDestination) {
        this.xmlDestination = xmlDestination;
    }
    
    public int getCurrentIndentValue() {
        return indentLength;
    }
    
    public void setIndentLength(int indentLength) {
        this.indentLength = indentLength;
    }
    
    public void increaseIndentValue() {
        indentLength++;
    }
    
    public void decreaseIndentValue() {
        indentLength--;
    }
    
    public boolean isAposEscaped() {
        return aposEscaped;
    }
    
    public void setAposEscaped(boolean aposEscaped) {
        this.aposEscaped = aposEscaped;
    }
    
    public void appendXMLDeclaration() throws IOException {
        append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    }
    
    public void appendStylesheet(String href, String type) throws IOException {
        appendIndent();
        append("<?xml-stylesheet href=\"");
        append(href);
        append("\" type=\"");
        append(type);
        append("\"?>");
    }
    
    public void appendCssStyleSheet(String href) throws IOException {
        appendStylesheet(href,"text/css");
    }
    
    public void appendDocType(String doctype) throws IOException {
        appendIndent();
        append("<!DOCTYPE ");
        append(doctype);
        append(">");
    }
    
    public void addEmptyElement(String tagname) throws IOException {
        startOpenTag(tagname);
        closeEmptyTag();
    }
    
    public void addSimpleElement(String tagname,String value) throws IOException{
        if ((value == null) || (value.length() == 0)) return;
        startOpenTag(tagname);
        endOpenTag(false);
        escape(value);
        closeTag(tagname,false,false);
    }
    
    public void addSimpleElement(String tagname, String attribute, String attributeValue,String value) throws IOException {
        startOpenTag(tagname);
        addAttribute(attribute, attributeValue);
        endOpenTag(false);
        escape(value);
        closeTag(tagname,false,false);
    }
    
    
    public void startOpenTag(String name) throws IOException {
        startOpenTag(name,true);
    }
    
    public void startOpenTag(String name, boolean indentBefore) throws IOException {
        if (indentBefore) appendIndent();
        append('<');
        append(name);
    }
    
    public void endOpenTag() throws IOException {
        endOpenTag(true);
    }
    
    public void endOpenTag(boolean increaseIndent) throws IOException {
        append('>');
        if (increaseIndent) increaseIndentValue();
    }
    
    public void closeEmptyTag() throws IOException {
        append('/');
        append('>');
    }
    
    public void openTag(String name) throws IOException {
        openTag(name,true,true);
    }
    
    
    public void openTag(String name, boolean indentBefore, boolean indentIncrease) throws IOException {
        if(indentBefore) appendIndent();
        append('<');
        append(name);
        append('>');
        if (indentIncrease) increaseIndentValue();
    }
    
    public void closeTag(String name) throws IOException {
        closeTag(name, true, true);
    }
    
    public void closeTag(String name, boolean indentBefore, boolean indentDecrease) throws IOException{
        if (indentDecrease) decreaseIndentValue();
        if (indentBefore) appendIndent();
        append('<');
        append('/');
        append(name);
        append('>');
    }
    
    public void escape(CharSequence charSequence) throws IOException {
        if (charSequence == null) return;
        int length = charSequence.length();
        for (int i = 0; i < length;i++) {
            char carac = charSequence.charAt(i);
            escape(carac);
        }
    }
    
    public void escape(char carac) throws IOException {
        if (carac < 32) append(' ');
        else {
            switch(carac) {
                case '&' :
                    append("&amp;");
                    break;
                case '"' :
                    append("&quot;");
                    break;
                case '\'' :
                    if (aposEscaped) append("&apos;");
                    else append(carac);
                    break;
                case '<' :
                    append("&lt;");
                    break;
                case '>' :
                    append("&gt;");
                    break;
                case '\u00A0' :
                    append("&#x00A0;");
                    break;
                default :
                    append(carac);
            }
        }
    }
    
    
    public void append(char c) throws IOException {
        xmlDestination.append(c);
    }
    
    public void append(CharSequence charSequence) throws IOException {
        xmlDestination.append(charSequence);
    }
    
    public void appendIndent() throws IOException {
        if (indentLength > - 1) {
            append('\n');
            for (int i = 0; i < indentLength;i++) {
                append('\t');
            }
        }
    }
    
    public void addNameSpaceAttribute(NameSpace nameSpace) throws IOException {
        addNameSpaceAttribute(nameSpace.getPrefix(), nameSpace.getName());
    }
    
    public void addNameSpaceAttribute(String prefix, String nameSpace) throws IOException {
        if ((nameSpace == null) || (nameSpace.length() == 0)) return;
        append(' ');
        append("xmlns");
        if ((prefix != null) && (prefix.length() > 0)) {
            append(':');
            append(prefix);
        }
        append('=');
        append('\"');
        escape(nameSpace);
        append('\"');
    }
    
    public void addAttribute(String attributeName,String value) throws IOException{
        if ((value == null) || (value.length() == 0)) return;
        append(' ');
        append(attributeName);
        append('=');
        append('\"');
        escape(value);
        append('\"');
    }
    
    public void addAttribute(String attributeName,char c) throws IOException{
        append(' ');
        append(attributeName);
        append('=');
        append('\"');
        escape(c);
        append('\"');
    }
    
    public void addAttribute(String attributeName,int i) throws IOException{
        append(' ');
        append(attributeName);
        append('=');
        append('\"');
        append(String.valueOf(i));
        append('\"');
    }
    
    
}
